@@ -6,7 +6,7 @@ rm -f config.cache
 srcdir=`dirname $0`
 test -z "$srcdir" && srcdir=.
 
-m4_files="_xine.m4 ORBit.m4 xine.m4 aa.m4 gettext.m4 glibc21.m4 iconv.m4 lcmessage.m4 progtest.m4 codeset.m4 isc-posix.m4"
+m4_files="_xine.m4 ORBit.m4 xine.m4 aa.m4 gettext.m4 glibc21.m4 iconv.m4 lcmessage.m4 progtest.m4 codeset.m4 isc-posix.m4 readline.m4"
 if test -d m4; then
     rm -f acinclude.m4
     for m4f in $m4_files; do
