@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: control.c,v 1.44 2002/04/21 22:17:11 f1rmb Exp $
+ * $Id: control.c,v 1.58 2002/08/01 08:38:58 f1rmb Exp $
  *
  *
  */
@@ -36,17 +36,15 @@
 #include <xine.h>
 #include <xine/xineutils.h>
 
-#include "xitk.h"
-
 #include "Imlib-light/Imlib.h"
+#include "panel.h"
 #include "event.h"
 #include "actions.h"
 #include "skins.h"
 #include "errors.h"
+#include "i18n.h"
 
-#ifndef __GNUC__
-#define __FUNCTION__    __func__
-#endif
+#include "xitk.h"
 
 extern gGui_t          *gGui;
 
@@ -101,6 +99,36 @@ static int set_current_prop(int prop, int value) {
   return (gGui->vo_driver->set_property(gGui->vo_driver, prop, value));
 }
 
+/*
+ * Enable or disable video settings sliders.
+ */
+static void active_sliders_video_settings(void) {
+  int vidcap;
+  
+  if((vidcap = gGui->vo_driver->get_capabilities(gGui->vo_driver)) > 0) {
+    
+    if(vidcap & VO_CAP_BRIGHTNESS)
+      xitk_enable_widget(control->bright);
+    else
+      xitk_disable_widget(control->bright);
+    
+    if(vidcap & VO_CAP_SATURATION)
+      xitk_enable_widget(control->sat);
+    else
+      xitk_disable_widget(control->sat);
+    
+    if(vidcap & VO_CAP_HUE)
+      xitk_enable_widget(control->hue);
+    else
+      xitk_disable_widget(control->hue);
+    
+    if(vidcap & VO_CAP_CONTRAST)
+      xitk_enable_widget(control->contr);
+    else
+      xitk_disable_widget(control->contr);
+  }
+}
+
 /*
  * Update silders positions
  */
@@ -300,23 +328,11 @@ void control_toggle_visibility (xitk_widget_t *w, void *data) {
  */
 void control_handle_event(XEvent *event, void *data) {
 
+  /*
   switch(event->type) {
 
-  case KeyPress:
-    gui_handle_event(event, data);
-    break;
-    
-  case MappingNotify:
-    XLockDisplay(gGui->display);
-    XRefreshKeyboardMapping((XMappingEvent *) event);
-    XUnlockDisplay(gGui->display);
-    break;
-
-  case ConfigureNotify:
-    /*  xitk_combo_update_pos(control->combo); */
-    break;
-
   }
+  */
 }
 
 /*
@@ -335,7 +351,7 @@ void control_change_skins(void) {
     
     if(!(new_img = Imlib_load_image(gGui->imlib_data,
 				    xitk_skin_get_skin_filename(gGui->skin_config, "CtlBG")))) {
-      xine_error(_("%s(): couldn't find image for background\n"), __FUNCTION__);
+      xine_error(_("%s(): couldn't find image for background\n"), __XINE_FUNCTION__);
       exit(-1);
     }
     
@@ -371,6 +387,8 @@ void control_change_skins(void) {
     
     xitk_change_skins_widget_list(control->widget_list, gGui->skin_config);
     xitk_paint_widget_list(control->widget_list);
+
+    active_sliders_video_settings();
   }
 }
 
@@ -378,11 +396,10 @@ void control_change_skins(void) {
  *
  */
 static void control_select_new_skin(xitk_widget_t *w, void *data) {
-  skins_locations_t **sks = get_available_skins();
   int                 selected = (int)data;
 
   xitk_browser_release_all_buttons(control->skinlist);
-  select_new_skin(sks[selected]);
+  select_new_skin(selected);
 }
 
 /*
@@ -393,7 +410,7 @@ void control_panel(void) {
   XSizeHints                 hint;
   XWMHints                  *wm_hint;
   XSetWindowAttributes       attr;
-  char                       title[] = {"Xine Control Panel"};
+  char                      *title;
   Atom                       prop, XA_WIN_LAYER;
   MWMHints                   mwmhints;
   XClassHint                *xclasshint;
@@ -411,6 +428,8 @@ void control_panel(void) {
       return;
   }
   
+  xine_strdupa(title, _("Xine Control Panel"));
+
   XITK_WIDGET_INIT(&br, gGui->imlib_data);
   XITK_WIDGET_INIT(&lb, gGui->imlib_data);
   XITK_WIDGET_INIT(&lbl, gGui->imlib_data);
@@ -493,7 +512,7 @@ void control_panel(void) {
   /* set xclass */
 
   if((xclasshint = XAllocClassHint()) != NULL) {
-    xclasshint->res_name = "Xine Control Panel";
+    xclasshint->res_name = title;
     xclasshint->res_class = "Xine";
     XSetClassHint(gGui->display, control->window, xclasshint);
   }
@@ -516,13 +535,10 @@ void control_panel(void) {
    */
   control->widget_list                = xitk_widget_list_new();
   control->widget_list->l             = xitk_list_new ();
-  control->widget_list->focusedWidget = NULL;
-  control->widget_list->pressedWidget = NULL;
   control->widget_list->win           = control->window;
   control->widget_list->gc            = gc;
   
   { /* All of sliders are disabled by default*/
-    int vidcap = 0;
     int min, max, cur;
 
     lbl.window = control->widget_list->win;
@@ -542,14 +558,15 @@ void control_panel(void) {
     sl.motion_callback   = set_hue;
     sl.motion_userdata   = NULL;
     xitk_list_append_content(control->widget_list->l,
-			    (control->hue = xitk_slider_create(gGui->skin_config, &sl)));
+	     (control->hue = xitk_slider_create(control->widget_list, gGui->skin_config, &sl)));
     xitk_slider_set_pos(control->widget_list, control->hue, cur);
     xitk_set_widget_tips(control->hue, _("Control HUE value"));
   
     lbl.skin_element_name = "CtlHueLbl";
     lbl.label             = _("Hue");
+    lbl.callback          = NULL;
     xitk_list_append_content(control->widget_list->l,
-			    xitk_label_create(gGui->skin_config, &lbl));
+			    xitk_label_create(control->widget_list, gGui->skin_config, &lbl));
     xitk_disable_widget(control->hue);
 
     /* SATURATION */
@@ -566,14 +583,15 @@ void control_panel(void) {
     sl.motion_callback   = set_saturation;
     sl.motion_userdata   = NULL;
     xitk_list_append_content(control->widget_list->l,
-	      (control->sat = xitk_slider_create(gGui->skin_config, &sl)));
+	      (control->sat = xitk_slider_create(control->widget_list, gGui->skin_config, &sl)));
     xitk_slider_set_pos(control->widget_list, control->sat, cur);
     xitk_set_widget_tips(control->sat, _("Control SATURATION value"));
 
     lbl.skin_element_name = "CtlSatLbl";
     lbl.label             = _("Sat");
+    lbl.callback          = NULL;
     xitk_list_append_content(control->widget_list->l,
-			    xitk_label_create(gGui->skin_config, &lbl));
+			    xitk_label_create(control->widget_list, gGui->skin_config, &lbl));
     xitk_disable_widget(control->sat);
       
     /* BRIGHTNESS */
@@ -590,14 +608,15 @@ void control_panel(void) {
     sl.motion_callback   = set_brightness;
     sl.motion_userdata   = NULL;
     xitk_list_append_content(control->widget_list->l,
-	    (control->bright = xitk_slider_create(gGui->skin_config, &sl)));
+	    (control->bright = xitk_slider_create(control->widget_list, gGui->skin_config, &sl)));
     xitk_slider_set_pos(control->widget_list, control->bright, cur);
     xitk_set_widget_tips(control->bright, _("Control BRIGHTNESS value"));
 
     lbl.skin_element_name = "CtlBrightLbl";
     lbl.label             = _("Brt");
+    lbl.callback          = NULL;
     xitk_list_append_content(control->widget_list->l,
-			    xitk_label_create(gGui->skin_config, &lbl));
+			    xitk_label_create(control->widget_list, gGui->skin_config, &lbl));
     xitk_disable_widget(control->bright);
       
     /* CONTRAST */
@@ -614,33 +633,18 @@ void control_panel(void) {
     sl.motion_callback   = set_contrast;
     sl.motion_userdata   = NULL;
     xitk_list_append_content(control->widget_list->l,
-	      (control->contr = xitk_slider_create(gGui->skin_config, &sl)));
+	      (control->contr = xitk_slider_create(control->widget_list, gGui->skin_config, &sl)));
     xitk_slider_set_pos(control->widget_list, control->contr, cur);
     xitk_set_widget_tips(control->contr, _("Control CONTRAST value"));
 
     lbl.skin_element_name = "CtlContLbl";
     lbl.label             = _("Ctr");
+    lbl.callback          = NULL;
     xitk_list_append_content(control->widget_list->l,
-			    xitk_label_create(gGui->skin_config, &lbl));
+			    xitk_label_create(control->widget_list, gGui->skin_config, &lbl));
     xitk_disable_widget(control->contr);
 
-    /*
-     * Enable only supported settings.
-     */
-    if((vidcap = gGui->vo_driver->get_capabilities(gGui->vo_driver)) > 0) {
-
-      if(vidcap & VO_CAP_BRIGHTNESS)
-	xitk_enable_widget(control->bright);
-      
-      if(vidcap & VO_CAP_SATURATION)
-      	xitk_enable_widget(control->sat);
-      
-      if(vidcap & VO_CAP_HUE)
-	xitk_enable_widget(control->hue);
-      
-      if(vidcap & VO_CAP_CONTRAST)
-	xitk_enable_widget(control->contr);
-    }
+    active_sliders_video_settings();
   }
 
   {
@@ -656,23 +660,22 @@ void control_panel(void) {
   
   lbl.skin_element_name = "CtlSkLbl";
   lbl.label             = _("Choose a Skin");
+  lbl.callback          = NULL;
   xitk_list_append_content(control->widget_list->l,
-			   xitk_label_create(gGui->skin_config, &lbl));
+			   xitk_label_create(control->widget_list, gGui->skin_config, &lbl));
 
   br.arrow_up.skin_element_name    = "CtlSkUp";
   br.slider.skin_element_name      = "SliderCtlSk";
   br.arrow_dn.skin_element_name    = "CtlSkDn";
   br.browser.skin_element_name     = "CtlSkItemBtn";
-  br.browser.max_displayed_entries = 5;
   br.browser.num_entries           = control->skins_num;
   br.browser.entries               = control->skins;
   br.callback                      = control_select_new_skin;
   br.dbl_click_callback            = NULL;
-  br.dbl_click_time                = DEFAULT_DBL_CLICK_TIME;
   br.parent_wlist                  = control->widget_list;
   xitk_list_append_content (control->widget_list->l, 
 			   (control->skinlist = 
-			    xitk_browser_create(gGui->skin_config, &br)));
+			    xitk_browser_create(control->widget_list, gGui->skin_config, &br)));
 
   xitk_browser_update_list(control->skinlist, 
 			   control->skins, control->skins_num, 0);
@@ -697,9 +700,11 @@ void control_panel(void) {
   lb.state_callback    = NULL;
   lb.userdata          = NULL;
   xitk_list_append_content (control->widget_list->l, 
-			    (w = xitk_labelbutton_create (gGui->skin_config, &lb)));
+		    (w = xitk_labelbutton_create (control->widget_list, gGui->skin_config, &lb)));
   xitk_set_widget_tips(w, _("Close control window"));
 
+  control_show_tips(panel_get_tips_enable(), panel_get_tips_timeout());
+
   XMapRaised(gGui->display, control->window); 
 
   XUnlockDisplay (gGui->display);
