@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: event.c,v 1.84 2002/04/27 13:26:38 f1rmb Exp $
+ * $Id: event.c,v 1.111 2002/07/31 00:50:40 f1rmb Exp $
  *
  * gui initialization and top-level event handling stuff
  *
@@ -31,17 +31,13 @@
 #include <X11/Xutil.h>
 #include <X11/keysym.h>
 #include <X11/Xmd.h>
-#ifdef HAVE_DPMS
-# include <X11/extensions/dpms.h>
-#endif
+
 #include <errno.h>
 #include <pthread.h>
 #include <signal.h>
 #include <limits.h>
 #include <zlib.h>
 
-#include "xitk.h"
-
 #include "Imlib-light/Imlib.h"
 
 #include "event.h"
@@ -54,11 +50,13 @@
 #include <xine/video_out_x11.h>
 #include <xine.h>
 #include <xine/xineutils.h>
-#include "xscreensaver-remote.h"
 #include "mrl_browser.h"
 #include "skins.h"
 #include "errors.h"
 #include "network.h"
+#include "i18n.h"
+
+#include "xitk.h"
 
 #ifdef HAVE_LIRC
 extern int no_lirc;
@@ -94,38 +92,13 @@ static unsigned char xine_bits[] = {
    0x00, 0x00, 0xf8, 0x1f, 0x00, 0x00, 0x00, 0xf8
 };
 
-/*
- * Screensavers parameters
- */
-typedef struct {
-
-  /* XFree86 one */
-  struct {
-    int           timeout;
-    int           interval;
-    int           prefer_blanking;
-    int           allow_exposures;
-  } screensaver;
-
-  /* Jamie's Zawinski xscreensaver */
-  struct {
-    int           was_running;
-  } xscreensaver;
-
-#ifdef HAVE_DPMS
-  /* XFree DPMS */
-  struct {
-    int           was_running;
-    CARD16        standby;
-    CARD16        suspend;
-    CARD16        off;
-    CARD16        level;
-  } xdpms;
-#endif
-
-} screen_savers_t;
-
-static screen_savers_t    ssavers;
+static void skip_by_chapter_cb(void *data, cfg_entry_t *cfg) {
+  gGui->skip_by_chapter = cfg->num_value;
+  panel_update_nextprev_tips();
+}
+static void ssaver_timeout_cb(void *data, cfg_entry_t *cfg) {
+  gGui->ssaver_timeout = cfg->num_value;
+}
 
 /*
  * Callback for snapshots saving location.
@@ -134,126 +107,16 @@ static void snapshot_loc_cb(void *data, cfg_entry_t *cfg) {
   gGui->snapshot_location = cfg->str_value;
 }
 
-static int actions_on_start(action_id_t actions[], action_id_t a) {
-  int i = 0;
+int actions_on_start(action_id_t actions[], action_id_t a) {
+  int i = 0, num = 0;
   while(actions[i] != ACTID_NOKEY) {
     if(actions[i] == a)
-      return 1;
+      num++;
     i++;
   }
-  return 0;
+  return num;
 }
 
-/**
- * Disable all screensavers.
- */
-static void disable_screensavers(void) {
-  int dummy;
-  
-#ifdef HAVE_DPMS
-  /* 
-   * XFree DPMS
-   */
-  ssavers.xdpms.was_running = 0;
-  
-  if(DPMSQueryExtension(gGui->display, &dummy, &dummy)) {
-    BOOL   enabled;
-    
-    DPMSInfo(gGui->display, &ssavers.xdpms.level, &enabled);
-    
-    if(enabled) {
-      
-      if(DPMSGetTimeouts(gGui->display, 
-			 &ssavers.xdpms.standby,
-			 &ssavers.xdpms.suspend, &ssavers.xdpms.off) != True) {
-	fprintf(stderr, "DPMSGetTimeouts() failed\n");
-      }
-
-      /* monitor powersave off */
-      (void) DPMSDisable(gGui->display);
-      ssavers.xdpms.was_running = 1;
-    }
-  }
-#endif
-  
-  /*
-   * XFree screensaver
-   */
-  XGetScreenSaver(gGui->display ,&ssavers.screensaver.timeout, 
-		  &ssavers.screensaver.interval,
-		  &ssavers.screensaver.prefer_blanking, 
-		  &ssavers.screensaver.allow_exposures);
-  
-  if((XSetScreenSaver(gGui->display, 0, 0, 
-		      DontPreferBlanking, DontAllowExposures)) == BadValue) {
-    fprintf(stderr, "XSetScreenSaver() failed: %s\n", strerror(errno));
-  }
-  
-  /*
-   * XScreenSaver specific.
-   */
-  xscreensaver_remote_init(gGui->display);
-  ssavers.xscreensaver.was_running = is_xscreensaver_running(gGui->display);
-  
-  if(ssavers.xscreensaver.was_running == 1) {
-    if(xscreensaver_kill_server(gGui->display) < 0)
-      ssavers.xscreensaver.was_running = 0;
-  }
-}
-
-/**
- * Re-enabling previously disabled screensavers.
- */
-static void reenable_screensavers(void) {
-  int dummy;
-  
-#ifdef HAVE_DPMS
-  /*
-   * XFree DPMS
-   */
-  if(ssavers.xdpms.was_running) {
-
-    if(DPMSQueryExtension(gGui->display, &dummy, &dummy)) {
-      
-      /* restoring power saving settings */
-      if((DPMSEnable(gGui->display)) == True) {
-	CARD16 state;
-	BOOL   enabled;
-	
-	(void) DPMSSetTimeouts(gGui->display, ssavers.xdpms.standby, 
-			       ssavers.xdpms.suspend, ssavers.xdpms.off);
-
-	(void) DPMSForceLevel(gGui->display, ssavers.xdpms.level);
-	
-	/* DPMS does not seem to be enabled unless we call DPMSInfo */
-	DPMSInfo(gGui->display, &state, &enabled);
-	
-	if(enabled)
-	  ssavers.xdpms.was_running = 0;
-	
-      }
-    }
-  }
-#endif
-  
-  /*
-   * XFree screensaver
-   */
-  if((XSetScreenSaver(gGui->display, ssavers.screensaver.timeout, 
-		      ssavers.screensaver.interval, 
-		      ssavers.screensaver.prefer_blanking, 
-		      ssavers.screensaver.allow_exposures)) == BadValue) {
-    fprintf(stderr, "XSetScreenSaver() failed: %s\n", strerror(errno));
-  }
-
-  /*
-   * Restart XScreenSaver.
-   */
-  if(ssavers.xscreensaver.was_running == 1)
-    xscreensaver_start_server();
-}
-
-
 void config_save(void) {
 
   gGui->config->save(gGui->config);
@@ -312,6 +175,7 @@ static void gui_signal_handler (int sig, void *data) {
     }
     break;
   }
+
 }
 
 /*
@@ -319,12 +183,54 @@ static void gui_signal_handler (int sig, void *data) {
  */
 void gui_execute_action_id(action_id_t action) {
   xine_event_t   xine_event;
+  
+  if(action & ACTID_IS_INPUT_EVENT) {
+    
+    if((action >= ACTID_EVENT_NUMBER_0) && (action <= ACTID_EVENT_NUMBER_9)) {
+
+      if(!gGui->numeric.set) {
+	gGui->numeric.set = 1;
+	gGui->numeric.arg = 0;
+      }
+      
+      if(((gGui->numeric.arg * 10) + (action - ACTID_EVENT_NUMBER_0)) <= INT_MAX) {
+	gGui->numeric.arg *= 10;
+	gGui->numeric.arg += (action - ACTID_EVENT_NUMBER_0);
+      }
+      else
+	printf("WARNING: Input number canceled, avoid overflow\n");
+
+    } 
+    else if(action == ACTID_EVENT_NUMBER_10_ADD) {
+
+      if(!gGui->numeric.set) {
+	gGui->numeric.set = 1;
+	gGui->numeric.arg = 0;
+      }
+      
+      if((gGui->numeric.arg + 10) <= INT_MAX)
+	gGui->numeric.arg += 10;
+      else
+	printf("WARNING: Input number canceled, avoid overflow\n");
+
+    }
+    else {
+      gGui->numeric.set = 0;
+      gGui->numeric.arg = 0;
+    }
+
+    /* events for advanced input plugins. */
+    xine_event.type = action & ~ACTID_IS_INPUT_EVENT;
+    xine_send_event(gGui->xine, &xine_event);
+    return;
+  }
 
   switch(action) {
     
   case ACTID_WINDOWREDUCE:
     video_window_set_mag (0.8 * video_window_get_mag());
     break;
+
   case ACTID_WINDOWENLARGE:
     video_window_set_mag (1.2 * video_window_get_mag());
     break;
@@ -333,19 +239,27 @@ void gui_execute_action_id(action_id_t action) {
   case ACTID_WINDOW100:
     video_window_set_mag (1.0);
     break;
+
   case ACTID_WINDOW200:
     video_window_set_mag (2.0);
     break;
+
   case ACTID_WINDOW50:
     video_window_set_mag (0.5);
     break;
 
   case ACTID_SPU_NEXT:
-    gui_change_spu_channel(NULL, (void*)GUI_NEXT);
+    if(!gGui->numeric.set)
+      gui_change_spu_channel(NULL, (void*)GUI_NEXT);
+    else
+      gui_direct_change_spu_channel(NULL, (void*)GUI_NEXT, gGui->numeric.arg);
     break;
     
   case ACTID_SPU_PRIOR:
-    gui_change_spu_channel(NULL, (void*)GUI_PREV);
+    if(!gGui->numeric.set)
+      gui_change_spu_channel(NULL, (void*)GUI_PREV);
+    else
+      gui_direct_change_spu_channel(NULL, (void*)GUI_PREV, gGui->numeric.arg);
     break;
     
   case ACTID_CONTROLSHOW:
@@ -355,13 +269,19 @@ void gui_execute_action_id(action_id_t action) {
   case ACTID_TOGGLE_WINOUT_VISIBLITY:
     gui_toggle_visibility (NULL, NULL);
     break;
-      
+    
   case ACTID_AUDIOCHAN_NEXT:
-    gui_change_audio_channel(NULL, (void*)GUI_NEXT);
+    if(!gGui->numeric.set)
+      gui_change_audio_channel(NULL, (void*)GUI_NEXT);
+    else
+      gui_direct_change_audio_channel(NULL, (void*)GUI_NEXT, gGui->numeric.arg);
     break;
     
   case ACTID_AUDIOCHAN_PRIOR:
-    gui_change_audio_channel(NULL, (void*)GUI_PREV);
+    if(!gGui->numeric.set)
+      gui_change_audio_channel(NULL, (void*)GUI_PREV);
+    else
+      gui_direct_change_audio_channel(NULL, (void*)GUI_PREV, gGui->numeric.arg);
     break;
     
   case ACTID_PAUSE:
@@ -377,7 +297,7 @@ void gui_execute_action_id(action_id_t action) {
     break;
 
   case ACTID_TOGGLE_FULLSCREEN:
-    gui_toggle_fullscreen(NULL, NULL);
+    gui_set_fullscreen_mode(NULL, NULL);
     break;
 
   case ACTID_TOGGLE_ASPECT_RATIO:
@@ -405,17 +325,29 @@ void gui_execute_action_id(action_id_t action) {
     break;
 
   case ACTID_MRL_NEXT:
-    gui_nextprev(NULL, (void*)GUI_NEXT);
+    if(!gGui->numeric.set)
+      gui_nextprev(NULL, (void*)GUI_NEXT);
+    else
+      gui_direct_nextprev(NULL, (void*)GUI_NEXT, gGui->numeric.arg);
     break;
-
+    
   case ACTID_MRL_PRIOR:
-    gui_nextprev(NULL, (void*)GUI_PREV);
+    if (!gGui->numeric.set)
+      gui_nextprev(NULL, (void*)GUI_PREV);
+    else
+      gui_direct_nextprev(NULL, (void*)GUI_PREV, gGui->numeric.arg);
     break;
       
   case ACTID_EJECT:
     gui_eject(NULL, NULL);
     break;
 
+  case ACTID_SET_CURPOS:
+    /* Number is a percentage */
+    gGui->numeric.arg %= 100; /* range [0..100] */
+    gui_set_current_position((65534 * gGui->numeric.arg) / 100);
+    break;
+
   case ACTID_SET_CURPOS_10:
     gui_set_current_position (6553);
     break;
@@ -456,6 +388,18 @@ void gui_execute_action_id(action_id_t action) {
     gui_set_current_position (0);
     break;
 
+  case ACTID_SEEK_REL_m:
+    if(gGui->numeric.set) {
+      gGui->numeric.arg = -gGui->numeric.arg;
+      gui_seek_relative (gGui->numeric.arg);
+    }
+  break;
+
+  case ACTID_SEEK_REL_p:
+    if(gGui->numeric.set)
+      gui_seek_relative (gGui->numeric.arg);
+    break;
+
   case ACTID_SEEK_REL_m60:
     gui_seek_relative (-60);
     break;
@@ -471,6 +415,22 @@ void gui_execute_action_id(action_id_t action) {
   case ACTID_SEEK_REL_p15:
     gui_seek_relative (15);
     break;
+    
+  case ACTID_SEEK_REL_m30:
+    gui_seek_relative (-30);
+    break;
+    
+  case ACTID_SEEK_REL_m7:
+    gui_seek_relative (-7);
+    break;
+    
+  case ACTID_SEEK_REL_p30:
+    gui_seek_relative (30);
+    break;
+    
+  case ACTID_SEEK_REL_p7:
+    gui_seek_relative (7);
+    break;
 
   case ACTID_MRLBROWSER:
     gui_mrlbrowser_show(NULL, NULL);
@@ -500,6 +460,10 @@ void gui_execute_action_id(action_id_t action) {
     gui_change_speed_playback(NULL, (void*)GUI_NEXT);
     break;
 
+  case ACTID_SPEED_RESET:
+    gui_change_speed_playback(NULL, (void*)GUI_RESET);
+    break;
+
   case ACTID_pVOLUME:
     gui_increase_audio_volume();
     break;
@@ -512,68 +476,6 @@ void gui_execute_action_id(action_id_t action) {
     panel_snapshot(NULL, NULL);
     break;
     
-    /* events for advanced input plugins: */
-
-  case ACTID_EVENT_MENU1:
-    xine_event.type = XINE_EVENT_INPUT_MENU1;
-    xine_send_event(gGui->xine, &xine_event);
-    break;
-
-  case ACTID_EVENT_MENU2:
-    xine_event.type = XINE_EVENT_INPUT_MENU2;
-    xine_send_event(gGui->xine, &xine_event);
-    break;
-
-  case ACTID_EVENT_MENU3:
-    xine_event.type = XINE_EVENT_INPUT_MENU3;
-    xine_send_event(gGui->xine, &xine_event);
-    break;
-
-  case ACTID_EVENT_UP:
-    xine_event.type = XINE_EVENT_INPUT_UP;
-    xine_send_event(gGui->xine, &xine_event);
-    break;
-
-  case ACTID_EVENT_DOWN:
-    xine_event.type = XINE_EVENT_INPUT_DOWN;
-    xine_send_event(gGui->xine, &xine_event);
-    break;
-
-  case ACTID_EVENT_LEFT:
-    xine_event.type = XINE_EVENT_INPUT_LEFT;
-    xine_send_event(gGui->xine, &xine_event);
-    break;
-
-  case ACTID_EVENT_RIGHT:
-    xine_event.type = XINE_EVENT_INPUT_RIGHT;
-    xine_send_event(gGui->xine, &xine_event);
-    break;
-
-  case ACTID_EVENT_NEXT:
-    xine_event.type = XINE_EVENT_INPUT_NEXT;
-    xine_send_event(gGui->xine, &xine_event);
-    break;
-
-  case ACTID_EVENT_PRIOR:
-    xine_event.type = XINE_EVENT_INPUT_PREVIOUS;
-    xine_send_event(gGui->xine, &xine_event);
-    break;
-  
-  case ACTID_EVENT_ANGLE_NEXT:
-    xine_event.type = XINE_EVENT_INPUT_ANGLE_NEXT;
-    xine_send_event(gGui->xine, &xine_event);
-    break;
-
-  case ACTID_EVENT_ANGLE_PRIOR:
-    xine_event.type = XINE_EVENT_INPUT_ANGLE_PREVIOUS;
-    xine_send_event(gGui->xine, &xine_event);
-    break;
-    
-  case ACTID_EVENT_SELECT:
-    xine_event.type = XINE_EVENT_INPUT_SELECT;
-    xine_send_event(gGui->xine, &xine_event);
-    break;
-    
   case ACTID_ZOOM_IN:
     gui_change_zoom(1);
     break;
@@ -585,14 +487,16 @@ void gui_execute_action_id(action_id_t action) {
   case ACTID_ZOOM_RESET:
     gui_reset_zoom();
     break;
-
+    
   case ACTID_GRAB_POINTER:
     if(!gGui->cursor_grabbed) {
       if(!panel_is_visible())
-	XGrabPointer(gGui->display, gGui->video_window, 1, None, GrabModeAsync, GrabModeAsync, gGui->video_window, None, CurrentTime);
+	XGrabPointer(gGui->display, gGui->video_window, 1, None, 
+		     GrabModeAsync, GrabModeAsync, gGui->video_window, None, CurrentTime);
       
       gGui->cursor_grabbed = 1;
-    } else {
+    }
+    else {
       XUngrabPointer(gGui->display, CurrentTime);
       gGui->cursor_grabbed = 0;
     }
@@ -605,10 +509,19 @@ void gui_execute_action_id(action_id_t action) {
   case ACTID_VIEWLOG:
     gui_viewlog_show(NULL, NULL);
     break;
-      
+
+  case ACTID_KBEDIT:
+    gui_kbedit_show(NULL, NULL);
+    break;
+
   default:
     break;
   }
+
+  /* Some sort of function was done given. Clear numeric argument. */
+  gGui->numeric.set = 0;
+  gGui->numeric.arg = 0;
+
 }	    
 
 /*
@@ -638,12 +551,13 @@ void gui_handle_event (XEvent *event, void *data) {
 
     /* printf ("button: %d\n",bevent->button); */
 
-    if ((bevent->button==3) && (bevent->window == gGui->video_window))
+    if ((bevent->button == 3) && (bevent->window == gGui->video_window))
       panel_toggle_visibility (NULL, NULL);
     
   }
   break;
 
+  case ButtonRelease:
   case KeyPress:
     kbindings_handle_kbinding(gGui->kbindings, event);
     break;
@@ -691,9 +605,15 @@ void gui_status_callback (int nStatus) {
       gGui->playlist_cur--;
     }
   }
+
+  if(is_playback_widgets_enabled() && (!gGui->playlist_num) && (!gGui->filename)) {
+    gui_set_current_mrl(NULL);
+    enable_playback_controls(0);
+  }
+
 }
 
-char *gui_next_mrl_callback () {
+char *gui_next_mrl_callback (void) {
 
   if (gGui->playlist_cur >= (gGui->playlist_num-1)) 
     return NULL;
@@ -701,7 +621,7 @@ char *gui_next_mrl_callback () {
   return gGui->playlist[gGui->playlist_cur+1];
 }
 
-void gui_branched_callback () {
+void gui_branched_callback (void) {
 
   if (gGui->playlist_cur < (gGui->playlist_num-1)) {
   
@@ -712,7 +632,6 @@ void gui_branched_callback () {
   }
 }
 
-
 static void gui_find_visual (Visual **visual_return, int *depth_return) {
   XWindowAttributes  attribs;
   XVisualInfo	    *vinfo;
@@ -768,7 +687,7 @@ static void gui_find_visual (Visual **visual_return, int *depth_return) {
 			   VisualIDMask, &vinfo_tmpl, 
 			   &num_visuals);
     if (vinfo == NULL) {
-      printf("gui_main: selected visual %#lx does not exist, trying default visual\n",
+      printf(_("gui_main: selected visual %#lx does not exist, trying default visual\n"),
 	     (long) gGui->prefered_visual_id);
     } else {
       depth = vinfo[0].depth;
@@ -787,7 +706,7 @@ static void gui_find_visual (Visual **visual_return, int *depth_return) {
     if (XMatchVisualInfo(gGui->display, gGui->screen, depth, TrueColor, &vinfo)) {
       visual = vinfo.visual;
     } else {
-      printf ("gui_main: couldn't find true color visual.\n");
+      printf (_("gui_main: couldn't find true color visual.\n"));
 
       depth = DefaultDepth (gGui->display, gGui->screen);
       visual = DefaultVisual (gGui->display, gGui->screen); 
@@ -803,7 +722,7 @@ static void gui_find_visual (Visual **visual_return, int *depth_return) {
 /*
  * Initialize the GUI
  */
-void gui_init (int nfiles, char *filenames[]) {
+void gui_init (int nfiles, char *filenames[], window_attributes_t *window_attribute) {
   int                   i;
   char                 *display_name = ":0.0";
 
@@ -811,7 +730,7 @@ void gui_init (int nfiles, char *filenames[]) {
    * init playlist
    */
 
-  for (i=0; i<nfiles; i++)
+  for (i = 0; i < nfiles; i++)
     gGui->playlist[i] = filenames[i];
 
   gGui->playlist_num = nfiles; 
@@ -831,7 +750,6 @@ void gui_init (int nfiles, char *filenames[]) {
 	    "thread-safe xlib.\n"));
     exit (1);
   } 
-
   
   if(getenv("DISPLAY"))
     display_name = getenv("DISPLAY");
@@ -841,23 +759,50 @@ void gui_init (int nfiles, char *filenames[]) {
     exit(1);
   }
 
-  if (gGui->config->register_bool (gGui->config, "gui.xsynchronize", 0, "synchronized X protocol (debug)", NULL, NULL, NULL)) {
+  if (gGui->config->register_bool (gGui->config, "gui.xsynchronize", 0,
+				   _("synchronized X protocol (debug)"), NULL, NULL, NULL)) {
     XSynchronize (gGui->display, True);
-    fprintf (stderr, "Warning! Synchronized X activated - this is way slow...\n");
+    fprintf (stderr, _("Warning! Synchronized X activated - this is way slow...\n"));
   }
 
-  gGui->layer_above = gGui->config->register_bool (gGui->config, "gui.layer_above", 
-						   1, "use wm layer property to place window on top", 
-						   NULL, NULL, NULL);
+  gGui->layer_above = 
+    gGui->config->register_bool (gGui->config, "gui.layer_above", 1,
+				 _("use wm layer property to place window on top"), 
+				 NULL, NULL, NULL);
+  
+  gGui->snapshot_location = 
+    gGui->config->register_string (gGui->config, "gui.snapshotdir", 
+				   (char *) (xine_get_homedir()),
+				   _("where snapshots will be saved"),
+				   NULL, snapshot_loc_cb, NULL);
+  
+  gGui->ssaver_timeout =
+    gGui->config->register_num (gGui->config, "gui.screensaver_timeout", 10,
+				_("time between two screensaver fake events, 0 to disable"),
+				NULL, ssaver_timeout_cb, NULL);
+  
+  gGui->skip_by_chapter = 
+    gGui->config->register_bool (gGui->config, "gui.skip_by_chapter", 1,
+				 _("play next|previous chapter instead of mrl (dvdnav)"), 
+				 NULL, skip_by_chapter_cb, NULL);
+  
+  gGui->numeric.set = 0;
+  gGui->numeric.arg = 0;
 
-  gGui->snapshot_location = gGui->config->register_string (gGui->config, "gui.snapshotdir", 
-							   (char*) (xine_get_homedir()),
-							   "where snapshots will be saved",
-							   NULL, snapshot_loc_cb, NULL);
   XLockDisplay (gGui->display);
 
   gGui->screen = DefaultScreen(gGui->display);
 
+  /* Some infos */
+  printf(_("XServer Vendor: %s. Release: %d,\n"), 
+	 XServerVendor(gGui->display), XVendorRelease(gGui->display));
+  printf(_("        Protocol Version: %d, Revision: %d,\n"), 
+	 XProtocolVersion(gGui->display), XProtocolRevision(gGui->display));
+  printf(_("        Available Screen(s): %d, using %d\n")
+	 , XScreenCount(gGui->display), gGui->screen);
+  printf(_("        Depth: %d.\n"),
+	 XDisplayPlanes(gGui->display, gGui->screen));
+
   gui_find_visual(&gGui->visual, &gGui->depth);
 
   gui_init_imlib (gGui->visual);
@@ -868,23 +813,23 @@ void gui_init (int nfiles, char *filenames[]) {
    * create and map panel and video window
    */
   xine_pid = getppid();
-
+  
   xitk_init(gGui->display);
-
+  
   init_skins_support();
 
   gGui->kbindings = kbindings_init_kbinding();
 
   gGui->running = 1;
 
-  video_window_init ();
+  video_window_init (window_attribute, 
+		     ((actions_on_start(gGui->actions_on_start, 
+				       ACTID_TOGGLE_WINOUT_VISIBLITY)) ? 1 : 0));
 
   panel_init ();
 
-  disable_screensavers();
 }
 
-
 void gui_init_imlib (Visual *vis) {
   XColor                dummy;
   ImlibInitParams	imlib_init;
@@ -937,7 +882,9 @@ void gui_init_imlib (Visual *vis) {
  *
  */
 void gui_run (void) {
-
+  int i;
+  
+  video_window_change_skins();
   panel_add_autoplay_buttons();
   panel_add_mixer_control();
   panel_update_channel_display () ;
@@ -949,25 +896,33 @@ void gui_run (void) {
     char **autoscan_plugins = xine_get_autoplay_input_plugin_ids(gGui->xine);
     
     int i;
-    for(i=0; autoscan_plugins[i] != NULL; ++i) {
-      if(!strcasecmp(autoscan_plugins[i], gGui->autoscan_plugin)) {
-	int num_mrls;
-	char **autoplay_mrls = xine_get_autoplay_mrls (gGui->xine,
-						       gGui->autoscan_plugin,
-						       &num_mrls);
-	int j;
-	
-	for (j=0; j<num_mrls; j++) {
-	  gGui->playlist[gGui->playlist_num + j] = autoplay_mrls[j];
-	}
-	gGui->playlist_num += j;
-	gGui->playlist_cur = 0;
-	gui_set_current_mrl(gGui->playlist[gGui->playlist_cur]);
+
+    if(autoscan_plugins) {
+
+      for(i=0; autoscan_plugins[i] != NULL; ++i) {
+
+	if(!strcasecmp(autoscan_plugins[i], gGui->autoscan_plugin)) {
+	  int num_mrls;
+	  char **autoplay_mrls = xine_get_autoplay_mrls (gGui->xine,
+							 gGui->autoscan_plugin,
+							 &num_mrls);
+	  int j;
 	  
-      }    
+	  if(autoplay_mrls) {
+	    for (j = 0; j < num_mrls; j++)
+	      gGui->playlist[gGui->playlist_num + j] = autoplay_mrls[j];
+	   
+	    gGui->playlist_num += j;
+	    gGui->playlist_cur = 0;
+	    gui_set_current_mrl(gGui->playlist[gGui->playlist_cur]);
+	  }
+	}    
+      }
     }
   }  
 
+  enable_playback_controls((gGui->playlist_num > 0));
+  
   if(gGui->actions_on_start[0] != ACTID_NOKEY) {
 
     /* Popup setup window if there is no config file */
@@ -982,11 +937,15 @@ void gui_run (void) {
     if(actions_on_start(gGui->actions_on_start, ACTID_TOGGLE_WINOUT_VISIBLITY)) {
       if(!panel_is_visible())
 	gui_execute_action_id(ACTID_TOGGLE_VISIBLITY);
-      gui_execute_action_id(ACTID_TOGGLE_WINOUT_VISIBLITY);
+
+      gGui->vo_driver->gui_data_exchange (gGui->vo_driver, 
+					  GUI_DATA_EX_VIDEOWIN_VISIBLE, 
+					  (int *)0);
+      
     }
 
     /*  The user wants to see in fullscreen mode  */
-    if(actions_on_start(gGui->actions_on_start, ACTID_TOGGLE_FULLSCREEN))
+    for (i = actions_on_start(gGui->actions_on_start, ACTID_TOGGLE_FULLSCREEN); i > 0; i--)
       gui_execute_action_id(ACTID_TOGGLE_FULLSCREEN);
     
     /*  The user request "play on start" */
@@ -1026,12 +985,13 @@ void gui_run (void) {
   
   start_remote_server();
 
+  /* Need for tvmode */
+  video_window_set_fullscreen_mode((video_window_get_fullscreen_mode()));
+
   xitk_run();
 
   gGui->running = 0;
 
   kbindings_save_kbinding(gGui->kbindings);
   kbindings_free_kbinding(&gGui->kbindings);
-
-  reenable_screensavers();
 }
