@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: main.c,v 1.84 2002/04/25 20:03:23 f1rmb Exp $
+ * $Id: main.c,v 1.106 2002/08/03 17:34:40 siggi Exp $
  *
  * xine main for X11
  *
@@ -58,11 +58,14 @@
 #include <xine/video_out_x11.h>
 #include <xine/xineutils.h>
 
-#include "xitk.h"
-
 #include "event.h"
 #include "videowin.h"
 #include "panel.h"
+#include "actions.h"
+#include "i18n.h"
+
+#include "xitk.h"
+#include "errors.h"
 
 #ifdef HAVE_ORBIT 
 #include "../corba/xine-server.h"
@@ -77,10 +80,20 @@
 /*
  * global variables
  */
-gGui_t   *gGui;
+gGui_t    *gGui;
 #ifdef HAVE_LIRC
-int       no_lirc;
+int        no_lirc;
 #endif
+int        unhandled_codec_mode; /* 0 = never, 1 = video, 2 = audio, 3 = always */
+
+
+typedef struct {
+  FILE    *fd;
+  char    *filename;
+  char    *ln;
+  char     buf[256];
+} file_info_t;
+
 
 #define	OPTION_VISUAL		1000
 #define	OPTION_INSTALL_COLORMAP	1001
@@ -97,7 +110,7 @@ static const char *short_options = "?hHgfvn"
 #ifdef DEBUG
  "d:"
 #endif
- "u:a:V:A:p::s:R";
+ "u:a:V:A:p::s:RG:B";
 static struct option long_options[] = {
   {"help"           , no_argument      , 0, 'h'                      },
 #ifdef HAVE_LIRC
@@ -120,16 +133,117 @@ static struct option long_options[] = {
   {"keymap"         , optional_argument, 0,  DISPLAY_KEYMAP          },
   {"network"        , no_argument      , 0, 'n'                      },
   {"root"           , no_argument      , 0, 'R'                      },
+  {"geometry"       , required_argument, 0, 'G'                      },
+  {"borderless"     , no_argument      , 0, 'B'                      },
   {"version"        , no_argument      , 0, 'v'                      },
   {0                , no_argument      , 0,  0                       }
 };
 
+
+/*
+ *    RC file related functions
+ */
+#if DEBUG
+static void _rc_file_check_args(int argc, char **argv) {
+  int i;
+  
+  printf("%s(): argc %d\n", __XINE_FUNCTION__, argc);
+
+  for(i = 0; i < argc; i++)
+    printf("argv[%d] = '%s'\n", i, argv[i]);
+}
+#endif
+static void _rc_file_clean_eol(file_info_t *rcfile) {
+  char *p;
+
+  p = rcfile->ln;
+
+  if(p) {
+    while(*p != '\0') {
+      if(*p == '\n' || *p == '\r') {
+	*p = '\0';
+	break;
+      }
+      
+      p++;
+    }
+    
+    while(p > rcfile->ln) {
+      --p;
+      
+      if(*p == ' ') 
+	*p = '\0';
+      else
+	break;
+    }
+  }
+}
+static int _rc_file_get_next_line(file_info_t *rcfile) {
+
+ __get_next_line:
+  rcfile->ln = fgets(rcfile->buf, 255, rcfile->fd);
+  
+  while(rcfile->ln && (*rcfile->ln == ' ' || *rcfile->ln == '\t')) ++rcfile->ln;
+  
+  if(rcfile->ln) {
+    if(!strncmp(rcfile->ln, "#", 1))
+      goto __get_next_line;
+  }
+  
+  _rc_file_clean_eol(rcfile);
+  
+  if(rcfile->ln && !strlen(rcfile->ln))
+    goto __get_next_line;
+
+  return((rcfile->ln != NULL));
+}
+/*
+ * This function duplicate argv[] to an char** array,, 
+ * try to open and parse a "~/.xine/xinerc" rc file, 
+ * and concatenate found entries to the char **array.
+ * This allow user to always specify a command line argument.
+ */
+static char **build_command_line_args(int argc, char *argv[], int *_argc) {
+  int            i;
+  char          *xinerc = ".xine/xinerc";
+  file_info_t   *rcfile;
+  char        **_argv = NULL;
+  
+  _argv  = (char **) xine_xmalloc(sizeof(char **) * (argc + 1));
+  (*_argc) = argc;
+  for(i = 0; i < argc; i++)
+    _argv[i] = strdup(argv[i]);
+
+  rcfile           = (file_info_t *) xine_xmalloc(sizeof(file_info_t));
+  rcfile->filename = (char *) xine_xmalloc((strlen((xine_get_homedir())) + strlen(xinerc)) + 2);
+  sprintf(rcfile->filename, "%s/%s", (xine_get_homedir()), xinerc);
+  
+  if((rcfile->fd = fopen(rcfile->filename, "r")) != NULL) {
+    
+    while(_rc_file_get_next_line(rcfile)) {
+      _argv = (char **) realloc(_argv, sizeof(char **) * ((*_argc) + 2));
+      _argv[(*_argc)] = strdup(rcfile->ln);
+      (*_argc)++;
+    }
+    
+    fclose(rcfile->fd);
+  }
+
+  _argv[(*_argc)] = NULL;
+  
+  free(rcfile->filename);
+  free(rcfile);
+
+  return _argv;
+}
+
 /*
  *
  */
 void show_version(void) {
 
-  printf("This is xine (X11 gui) - a free video player v%s\n(c) 2000-2002 by G. Bartsch and the xine project team.\n", VERSION);
+  printf(_("This is xine (X11 gui) - a free video player v%s\n"
+	   "(c) 2000-2002 by G. Bartsch and the xine project team.\n"), VERSION);
 }
 
 /*
@@ -205,6 +319,8 @@ void show_usage (void) {
   printf(_("                                 -if no option is given, 'default' is selected.\n"));
   printf(_("  -n, --network                Enable network remote control server.\n"));
   printf(_("  -R, --root                   Use root window as video window.\n"));
+  printf(_("  -G, --geometry <WxH[+X+Y]>   Set output window geometry (X style).\n"));
+  printf(_("  -B, --borderless             Borderless video output window.\n"));
   printf("\n");
   printf(_("examples for valid MRLs (media resource locator):\n"));
   printf(_("  File:  'path/foo.vob'\n"));
@@ -258,7 +374,7 @@ static void load_video_out_driver(char *video_driver_id) {
    * Setting default (configfile stuff need registering before updating, etc...).
    */
   default_driver = gGui->config->register_string (gGui->config, "video.driver", "auto",
-						  "video driver to use",
+						  _("video driver to use"),
 						  NULL, NULL, NULL);
   if (!video_driver_id) {
     /* video output driver auto-probing */
@@ -335,7 +451,7 @@ static ao_driver_t *load_audio_out_driver(char *audio_driver_id) {
    * Setting default (configfile stuff need registering before updating, etc...).
    */
   default_driver = gGui->config->register_string (gGui->config, "audio.driver", "auto",
-						  "audio driver to use",
+						  _("audio driver to use"),
 						  NULL, NULL, NULL);
   
   /*
@@ -351,7 +467,7 @@ static ao_driver_t *load_audio_out_driver(char *audio_driver_id) {
     char **driver_ids = xine_list_audio_output_plugins ();
     int i = 0;
 
-    printf ("main: probing audio drivers...\n");
+    printf (_("main: probing audio drivers...\n"));
     
     while ( driver_ids[i] != NULL ) {
       audio_driver_id = driver_ids[i];
@@ -362,8 +478,7 @@ static ao_driver_t *load_audio_out_driver(char *audio_driver_id) {
 						    driver_ids[i]);
 
       if (audio_driver) {
-	printf ("main: ...worked, using '%s' audio driver.\n",
-		driver_ids[i]);
+	printf (_("main: ...worked, using '%s' audio driver.\n"), driver_ids[i]);
 
 	gGui->config->update_string (gGui->config, "audio.driver", 
 				     audio_driver_id);
@@ -383,7 +498,7 @@ static ao_driver_t *load_audio_out_driver(char *audio_driver_id) {
     /* don't want to load an audio driver ? */
     if (!strncasecmp (audio_driver_id, "NULL", 4)) {
 
-      printf("main: not using any audio driver (as requested).\n");
+      printf(_("main: not using any audio driver (as requested).\n"));
       gGui->config->update_string (gGui->config, "audio.driver", 
 				   "null");
 
@@ -443,40 +558,93 @@ void event_listener (void *user_data, xine_event_t *event) {
     gui_branched_callback ();
     break;
   }      
+ 
+}
+
+/*
+ * Callback of config value change about reporting mode.
+ */
+static void unhandled_codec_mode_cb(void *dummy, cfg_entry_t *entry) {
+  unhandled_codec_mode = entry->num_value;
+}
+
+/*
+ * Callback called on codec reporting.
+ */
+static void codec_reporting(void *user_data, int codec_type,
+			    uint32_t fourcc, char *description, int handled) {
+  char fourcc_txt[10];
   
+  /* store fourcc as text */
+  *(uint32_t *)fourcc_txt = fourcc;
+  fourcc_txt[4] = '\0';
+  
+  /* report error for unknown/unhandled codecs */
+  if( !handled ) {
+    if( codec_type == XINE_CODEC_VIDEO &&
+        (unhandled_codec_mode == 1 || unhandled_codec_mode == 3) ) {
+      
+      /* display fourcc if no description available */
+      if( !description[0] )
+        description = fourcc_txt;
+        
+      xine_error_with_more(_("No video plugin available to decode '%s'."), description);
+    }
+            
+    if( codec_type == XINE_CODEC_AUDIO &&
+        (unhandled_codec_mode == 2 || unhandled_codec_mode == 3) ) {
+      
+      /* display fourcc if no description available */
+      if( !description[0] ) {
+        description = fourcc_txt;
+        
+        /* show waveformattag in hexa */
+        if( fourcc < 0x10000 ) {
+          sprintf(fourcc_txt, "0x%x", fourcc );
+        }  
+      }
+      xine_error_with_more(_("No audio plugin available to decode '%s'."), description);
+    }
+  }
+}
+
+/*
+ * initialize codec reporting stuff.
+ */
+static void init_report_codec(void) {
+  static char *warn_unhandled_codec[] = 
+    { "never", "video only", "audio only", "always", NULL };
+  
+  /*
+   * Register codec reporting
+   */
+  xine_register_report_codec_cb(gGui->xine, codec_reporting, (void *) gGui);
+  
+  unhandled_codec_mode = 
+    gGui->config->register_enum (gGui->config,
+				 "gui.warn_unhandled_codec", 3, warn_unhandled_codec,
+				 _("Display popup window on unhandled codecs"),
+				 NULL, unhandled_codec_mode_cb, NULL);
 }
 
 /*
  *
  */
 int main(int argc, char *argv[]) {
-
   /* command line options will end up in these variables: */
-  int              c = '?', aos = 0;
-  int              option_index = 0;
-  int              audio_channel = -1;
-  int              spu_channel = -1;
-  /*  int              audio_options = 0; FIXME */
-  int		   visual = 0;
-  char            *audio_driver_id = NULL;
-  char            *video_driver_id = NULL;
-  ao_driver_t     *audio_driver = NULL ;
-  sigset_t         vo_mask;
-
-  /* Check xine library version */
-  if(!xine_check_version(0, 9, 9)) {
-    fprintf(stderr, _("Require xine library version 0.9.9, found %d.%d.%d.\n"),
-	    xine_get_major_version(), xine_get_minor_version(),
-	    xine_get_sub_version());
-    exit(1);
-  }
-
-  sigemptyset(&vo_mask);
-  sigaddset(&vo_mask, SIGALRM);
-  if (sigprocmask (SIG_BLOCK,  &vo_mask, NULL)) {
-    printf ("video_out: sigprocmask failed.\n");
-  }
-
+  int                   c = '?', aos = 0;
+  int                   option_index = 0;
+  int                   audio_channel = -1;
+  int                   spu_channel = -1;
+  window_attributes_t   window_attribute;
+  int		        visual = 0;
+  char                 *audio_driver_id = NULL;
+  char                 *video_driver_id = NULL;
+  ao_driver_t          *audio_driver = NULL ;
+  sigset_t              vo_mask;
+  char                **_argv;
+  int                   _argc;
+    
 #ifdef HAVE_SETLOCALE
   xine_set_locale();
   xitk_set_locale();
@@ -486,6 +654,20 @@ int main(int argc, char *argv[]) {
   bindtextdomain(PACKAGE, XITK_LOCALE);
   textdomain(PACKAGE);
   
+  /* Check xine library version */
+  if(!xine_check_version(0, 9, 10)) {
+    fprintf(stderr, _("Require xine library version 0.9.10, found %d.%d.%d.\n"),
+	    xine_get_major_version(), xine_get_minor_version(),
+	    xine_get_sub_version());
+    exit(1);
+  }
+
+  sigemptyset(&vo_mask);
+  sigaddset(&vo_mask, SIGALRM);
+  if (sigprocmask (SIG_BLOCK,  &vo_mask, NULL)) {
+    printf (_("sigprocmask() failed.\n"));
+  }
+
   gGui = (gGui_t *) xine_xmalloc(sizeof(gGui_t));
 
   gGui->debug_level            = 0;
@@ -501,19 +683,28 @@ int main(int argc, char *argv[]) {
 #endif
   gGui->actions_on_start[aos]  = ACTID_NOKEY;
 
+  window_attribute.x     = window_attribute.y      = -8192;
+  window_attribute.width = window_attribute.height = -1;
+  window_attribute.borderless = 0;
+
+  _argv = build_command_line_args(argc, argv, &_argc);
+#if DEBUG
+  _rc_file_check_args(_argc, _argv);
+#endif
+
   /*
    * initialize CORBA server
    */
 #ifdef HAVE_ORBIT
   if (!no_lirc)
-    xine_server_init (&argc, argv);
+    xine_server_init (&_argc, _argv);
 #endif
 
   /*
    * parse command line
    */
   opterr = 0;
-  while((c = getopt_long(argc, argv, short_options,
+  while((c = getopt_long(_argc, _argv, short_options,
 			 long_options, &option_index)) != EOF) {
     switch(c) {
 
@@ -535,7 +726,6 @@ int main(int argc, char *argv[]) {
       if(optarg != NULL) {
 	video_driver_id = xine_xmalloc (strlen (optarg) + 1);
 	strncpy (video_driver_id, optarg, strlen (optarg));
-	printf("video_driver_id = '%s'\n", video_driver_id);
       } else {
 	fprintf (stderr, _("video driver id required for -V option\n"));
 	exit (1);
@@ -642,6 +832,33 @@ int main(int argc, char *argv[]) {
       gGui->use_root_window = 1;
       break;
 
+    case 'G': /* Set geometry */
+      {
+	int width, height, xoff, yoff;
+	
+	if(optarg != NULL) {
+	  if((sscanf(optarg, "%ix%i+%i+%i", &width, &height, &xoff, &yoff)) == 4) {
+	    window_attribute.width  = width;
+	    window_attribute.height = height;
+	    window_attribute.x      = xoff;
+	    window_attribute.y      = yoff;
+	  }
+	  else if((sscanf(optarg, "%ix%i", &width, &height)) == 2) {
+	    window_attribute.width  = width;
+	    window_attribute.height = height;
+	  }
+	  else {
+	    printf(_("Bad geometry '%s', see xine --help\n"), optarg);
+	    exit(1);
+	  }
+	}
+      }
+      break;
+
+    case 'B':
+      window_attribute.borderless = 1;
+      break;
+
     case 'v': /* Display version and exit*/
       show_version();
       exit(1);
@@ -659,8 +876,16 @@ int main(int argc, char *argv[]) {
       exit(1);
     }
   }
-
-  gGui->actions_on_start[aos] = ACTID_NOKEY;
+  
+  /* 
+   * Using root window mode don't allow
+   * window geometry, so, reset those params.
+   */
+  if(gGui->use_root_window) {
+    window_attribute.x     = window_attribute.y      = -8192;
+    window_attribute.width = window_attribute.height = -1;
+    window_attribute.borderless = 0;
+  }
 
   show_banner();
 
@@ -682,13 +907,15 @@ int main(int argc, char *argv[]) {
 
   }
 
+  gGui->actions_on_start[aos] = ACTID_NOKEY;
+
   gGui->config = xine_config_file_init (gGui->configfile);
 
   /*
    * init gui
    */
   
-  gui_init(argc-optind, &argv[optind]);
+  gui_init(_argc - optind, &_argv[optind], &window_attribute);
 
   /*
    * load and init output drivers
@@ -706,9 +933,9 @@ int main(int argc, char *argv[]) {
    * xine init
    */
 
-  //  xine_set_locale();
   gGui->xine = xine_init (gGui->vo_driver, audio_driver,
 			  gGui->config);
+  xine_tvmode_init2(gGui->xine);
 
   xine_select_audio_channel (gGui->xine, audio_channel);
   xine_select_spu_channel (gGui->xine, spu_channel);
@@ -716,10 +943,12 @@ int main(int argc, char *argv[]) {
   /*
    * Register an event listener
    */
-  xine_register_event_listener(gGui->xine, event_listener, gGui);
+  xine_register_event_listener(gGui->xine, event_listener, (void *) gGui);
+
+  init_report_codec();
 
   /*
-   * start CORBA server thread
+   * start CORBA server threadq
    */
 #ifdef HAVE_ORBIT
   if (!no_lirc)
