@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: panel.c,v 1.85 2002/04/27 13:26:38 f1rmb Exp $
+ * $Id: panel.c,v 1.101 2002/08/01 08:38:58 f1rmb Exp $
  *
  * xine panel related stuff
  *
@@ -34,8 +34,6 @@
 #include <xine/xineutils.h>
 #include <zlib.h>
 
-#include "xitk.h"
-
 #include "event.h"
 #include "actions.h"
 #include "control.h"
@@ -43,18 +41,57 @@
 #include "videowin.h"
 #include "panel.h"
 #include "mrl_browser.h"
+#include "setup.h"
+#include "viewlog.h"
 #include "snapshot.h"
 #include "errors.h"
 #include "utils.h"
+#include "i18n.h"
 
-#ifndef __GNUC__
-#define __FUNCTION__    __func__
-#endif
+#include "xitk.h"
 
 extern gGui_t     *gGui;
 
 _panel_t          *panel;
 
+void panel_update_nextprev_tips(void) {
+
+  if(gGui->skip_by_chapter) { 
+    xitk_set_widget_tips(panel->playback_widgets.prev, _("Play previous chapter or mrl")); 
+    xitk_set_widget_tips(panel->playback_widgets.next, _("Play next chapter or mrl")); 
+  }
+  else {
+    xitk_set_widget_tips(panel->playback_widgets.prev, _("Play previous entry from playlist"));
+    xitk_set_widget_tips(panel->playback_widgets.next, _("Play next entry from playlist")); 
+  }
+}
+
+int is_playback_widgets_enabled(void) {
+  return (panel->playback_widgets.enabled == 1);
+}
+
+void enable_playback_controls(int enable) {
+  
+  if(enable != panel->playback_widgets.enabled) {
+    void (*enability)(xitk_widget_t *) = NULL;
+    
+    panel->playback_widgets.enabled = enable;
+    
+    if(enable)
+      enability = xitk_enable_widget;
+    else
+      enability = xitk_disable_widget;
+    
+    enability(panel->playback_widgets.prev);
+    enability(panel->playback_widgets.stop);
+    enability(panel->playback_widgets.play);
+    enability(panel->playback_widgets.pause);
+    enability(panel->playback_widgets.next);
+    enability(panel->playback_widgets.eject);
+    enability(panel->playback_widgets.slider_play);
+  }
+}
+
 /*
  *
  */
@@ -70,6 +107,13 @@ void panel_show_tips(void) {
   mrl_browser_show_tips(panel->tips.enable, panel->tips.timeout);
 }
 
+int panel_get_tips_enable(void) {
+  return panel->tips.enable;
+}
+unsigned long panel_get_tips_timeout(void) {
+  return panel->tips.timeout;
+}
+
 /*
  * Config callbacks.
  */
@@ -129,7 +173,7 @@ void panel_change_skins(void) {
   
   if(!(new_img = Imlib_load_image(gGui->imlib_data,
 				  xitk_skin_get_skin_filename(gGui->skin_config, "BackGround")))) {
-    xine_error("%s(): couldn't find image for background\n", __FUNCTION__);
+    xine_error(_("%s(): couldn't find image for background\n"), __XINE_FUNCTION__);
     exit(-1);
   }
   
@@ -209,17 +253,32 @@ void panel_update_runtime_display(void) {
  * Update slider thread.
  */
 static void *slider_loop(void *dummy) {
+  int screensaver_timer = 0;
   
   pthread_detach(pthread_self());
 
+  /* Wait for xine_init() call */
+  while(gGui->xine == NULL)
+    xine_usec_sleep(100000);
+  
   while(gGui->running) {
+    int status = xine_get_status(gGui->xine);
+    
+    if(status == XINE_PLAY && gGui->ssaver_timeout) {
+      
+      screensaver_timer++;
+      
+      if(screensaver_timer >= gGui->ssaver_timeout) {
+	screensaver_timer = 0;
+	video_window_reset_ssaver();
+      }
+    }
 
     if(panel_is_visible()) {
       if(gGui->xine) {
-	int status = xine_get_status(gGui->xine);
 	
 	if(status == XINE_PLAY) {
-	  xitk_slider_set_pos(panel->widget_list, panel->slider_play, 
+	  xitk_slider_set_pos(panel->widget_list, panel->playback_widgets.slider_play, 
 			      xine_get_current_position(gGui->xine));
 	}
 
@@ -278,6 +337,21 @@ void panel_toggle_visibility (xitk_widget_t *w, void *data) {
     mrl_browser_toggle_visibility(NULL, NULL);
   }
 
+  if(!panel->visible && setup_is_visible()) {}
+  else {
+    setup_toggle_visibility(NULL, NULL);
+  }
+
+  if(!panel->visible && viewlog_is_visible()) {}
+  else {
+    viewlog_toggle_visibility(NULL, NULL);
+  }
+
+  if(!panel->visible && kbedit_is_visible()) {}
+  else {
+    kbedit_toggle_visibility(NULL, NULL);
+  }
+
 
   if (panel->visible) {
     
@@ -316,14 +390,21 @@ void panel_toggle_visibility (xitk_widget_t *w, void *data) {
     if(gGui->cursor_grabbed)
        XUngrabPointer(gGui->display, CurrentTime);
      
+    /* TODO: Currently this is a quick hack
+     * We should rather test whether screen size has changed
+     * and move the panel on screen if it doesn't fit any longer */
+    if (video_window_get_fullscreen_mode () > 1 
 #ifdef HAVE_XF86VIDMODE
+        || gGui->XF86VidMode_fullscreen
+#endif
+       ) {
     /*
      * necessary to place the panel in a visible area (otherwise it might
-     * appear off the video window while switched to a different modline)
+     * appear off the video window while switched to a differenti
+     * modeline or tv mode)
      */
-    if(gGui->XF86VidMode_fullscreen)
-       XMoveWindow(gGui->display, gGui->panel_window, 0, 0);
-#endif
+       XMoveWindow(gGui->display, gGui->panel_window, 40, 40);
+    }
 
     XUnlockDisplay(gGui->display);
   }
@@ -342,7 +423,7 @@ void panel_check_mute(void) {
  */
 void panel_check_pause(void) {
   
-  xitk_checkbox_set_state(panel->checkbox_pause, 
+  xitk_checkbox_set_state(panel->playback_widgets.pause, 
 			  (((xine_get_status(gGui->xine) == XINE_PLAY) && 
 			    (xine_get_speed(gGui->xine) == SPEED_PAUSE)) ? 1 : 0 ), 
 			  gGui->panel_window, panel->widget_list->gc);
@@ -353,7 +434,7 @@ void panel_check_pause(void) {
  * Reset the slider of panel window (set to 0).
  */
 void panel_reset_slider (void) {
-  xitk_slider_reset(panel->widget_list, panel->slider_play);
+  xitk_slider_reset(panel->widget_list, panel->playback_widgets.slider_play);
 }
 
 /*
@@ -412,7 +493,7 @@ void panel_snapshot(xitk_widget_t *w, void *data) {
  */
 static void panel_slider_cb(xitk_widget_t *w, void *data, int pos) {
 
-  if(w == panel->slider_play) {
+  if(w == panel->playback_widgets.slider_play) {
     gui_set_current_position (pos);
     if(xine_get_status(gGui->xine) != XINE_PLAY) {
       panel_reset_slider();
@@ -423,7 +504,8 @@ static void panel_slider_cb(xitk_widget_t *w, void *data, int pos) {
     xine_set_audio_property(gGui->xine, gGui->mixer.volume_mixer, gGui->mixer.volume_level);
   }
   else
-    xine_error("unknown widget slider caller\n");
+    /* FIXME: this error message does not make sense! */
+    xine_error(_("unknown widget slider caller\n"));
 
   panel_check_pause();
 }
@@ -436,6 +518,7 @@ void panel_handle_event(XEvent *event, void *data) {
   switch(event->type) {
 
   case KeyPress:
+  case ButtonRelease:
     gui_handle_event(event, data);
     break;
 
@@ -474,7 +557,7 @@ void panel_add_autoplay_buttons(void) {
     lb.userdata          = NULL;
     xitk_list_append_content (panel->widget_list->l,
 	     (panel->autoplay_plugins[i] =
-	      xitk_labelbutton_create (gGui->skin_config, &lb)));
+	      xitk_labelbutton_create (panel->widget_list, gGui->skin_config, &lb)));
     xitk_set_widget_tips(panel->autoplay_plugins[i], 
 			 xine_get_input_plugin_description(gGui->xine, autoplay_plugins[i]));
 
@@ -540,7 +623,7 @@ void panel_init (void) {
   XSizeHints                hint;
   XWMHints                 *wm_hint;
   XSetWindowAttributes      attr;
-  char                      title[] = {"Xine Panel"}; /* window-title     */
+  char                     *title;
   Atom                      prop, XA_WIN_LAYER;
   MWMHints                  mwmhints;
   XClassHint               *xclasshint;
@@ -554,6 +637,8 @@ void panel_init (void) {
   if (gGui->panel_window)
     return ; /* panel already open  - FIXME: bring to foreground */
 
+  xine_strdupa(title, _("Xine Panel"));
+
   XITK_WIDGET_INIT(&b, gGui->imlib_data);
   XITK_WIDGET_INIT(&cb, gGui->imlib_data);
   XITK_WIDGET_INIT(&lbl, gGui->imlib_data);
@@ -561,6 +646,8 @@ void panel_init (void) {
 
   panel = (_panel_t *) xine_xmalloc(sizeof(_panel_t));
 
+  panel->playback_widgets.enabled = -1;
+
   XLockDisplay (gGui->display);
   
   /*
@@ -569,7 +656,7 @@ void panel_init (void) {
   if (!(panel->bg_image = 
 	Imlib_load_image(gGui->imlib_data,
 			 xitk_skin_get_skin_filename(gGui->skin_config, "BackGround")))) {
-    xine_error("xine-panel: couldn't find image for background\n");
+    xine_error(_("xine-panel: couldn't find image for background\n"));
     exit(-1);
   }
 
@@ -652,7 +739,7 @@ void panel_init (void) {
    */
 
   if((xclasshint = XAllocClassHint()) != NULL) {
-    xclasshint->res_name = "Xine Panel";
+    xclasshint->res_name = title;
     xclasshint->res_class = "Xine";
     XSetClassHint(gGui->display, gGui->panel_window, xclasshint);
   }
@@ -681,8 +768,6 @@ void panel_init (void) {
 
   panel->widget_list                = xitk_widget_list_new();
   panel->widget_list->l             = xitk_list_new ();
-  panel->widget_list->focusedWidget = NULL;
-  panel->widget_list->pressedWidget = NULL;
   panel->widget_list->win           = gGui->panel_window;
   panel->widget_list->gc            = gc;
  
@@ -693,124 +778,144 @@ void panel_init (void) {
   b.skin_element_name = "Prev";
   b.callback          = gui_nextprev;
   b.userdata          = (void *)GUI_PREV;
-  xitk_list_append_content(panel->widget_list->l, (w = xitk_button_create(gGui->skin_config, &b)));
-  xitk_set_widget_tips(w, _("Play previous entry from playlist"));
+  xitk_list_append_content(panel->widget_list->l, 
+   (panel->playback_widgets.prev = xitk_button_create(panel->widget_list, gGui->skin_config, &b)));
 
   /*  Stop button */
   b.skin_element_name = "Stop";
   b.callback          = gui_stop;
   b.userdata          = NULL;
-  xitk_list_append_content(panel->widget_list->l, (w = xitk_button_create(gGui->skin_config, &b)));
-  xitk_set_widget_tips(w, _("Stop playback"));
+  xitk_list_append_content(panel->widget_list->l, 
+   (panel->playback_widgets.stop = xitk_button_create(panel->widget_list, gGui->skin_config, &b)));
+  xitk_set_widget_tips(panel->playback_widgets.stop, _("Stop playback"));
   
   /*  Play button */
   b.skin_element_name = "Play";
   b.callback          = gui_play;
   b.userdata          = NULL;
-  xitk_list_append_content(panel->widget_list->l, (w = xitk_button_create(gGui->skin_config, &b)));
-  xitk_set_widget_tips(w, _("Play selected entry"));
+  xitk_list_append_content(panel->widget_list->l, 
+   (panel->playback_widgets.play = xitk_button_create(panel->widget_list, gGui->skin_config, &b)));
+  xitk_set_widget_tips(panel->playback_widgets.play, _("Play selected entry"));
 
   /*  Pause button */
   cb.skin_element_name = "Pause";
   cb.callback          = gui_pause;
   cb.userdata          = NULL;
   xitk_list_append_content(panel->widget_list->l, 
-			   (panel->checkbox_pause = xitk_checkbox_create(gGui->skin_config, &cb)));
-  xitk_set_widget_tips(panel->checkbox_pause, _("Pause/Resume playback"));
+   (panel->playback_widgets.pause = xitk_checkbox_create(panel->widget_list, gGui->skin_config, &cb)));
+  xitk_set_widget_tips(panel->playback_widgets.pause, _("Pause/Resume playback"));
   
   /*  Next button */
   b.skin_element_name = "Next";
   b.callback          = gui_nextprev;
   b.userdata          = (void *)GUI_NEXT;
-  xitk_list_append_content(panel->widget_list->l, (w = xitk_button_create(gGui->skin_config, &b)));
-  xitk_set_widget_tips(w, _("Play next entry from playlist")); 
+  xitk_list_append_content(panel->widget_list->l, 
+   (panel->playback_widgets.next = xitk_button_create(panel->widget_list, gGui->skin_config, &b)));
 
   /*  Eject button */
   b.skin_element_name = "Eject";
   b.callback          = gui_eject;
   b.userdata          = NULL;
-  xitk_list_append_content(panel->widget_list->l, (w = xitk_button_create(gGui->skin_config, &b)));
-  xitk_set_widget_tips(w, _("Eject current medium"));
+  xitk_list_append_content(panel->widget_list->l, 
+   (panel->playback_widgets.eject = xitk_button_create(panel->widget_list, gGui->skin_config, &b)));
+  xitk_set_widget_tips(panel->playback_widgets.eject, _("Eject current medium"));
 
   /*  Exit button */
   b.skin_element_name = "Exit";
   b.callback          = gui_exit;
   b.userdata          = NULL;
-  xitk_list_append_content(panel->widget_list->l, (w = xitk_button_create(gGui->skin_config, &b)));
+  xitk_list_append_content(panel->widget_list->l,
+			   (w = xitk_button_create(panel->widget_list, gGui->skin_config, &b)));
   xitk_set_widget_tips(w, _("Quit"));
  
   /*  Setup button */
   b.skin_element_name = "Setup";
   b.callback          = gui_setup_show;
   b.userdata          = NULL;
-  xitk_list_append_content(panel->widget_list->l, (w = xitk_button_create(gGui->skin_config, &b)));
+  xitk_list_append_content(panel->widget_list->l, 
+			   (w = xitk_button_create(panel->widget_list, gGui->skin_config, &b)));
   xitk_set_widget_tips(w, _("Setup window"));
 
   /*  Close button */
   b.skin_element_name = "Close";
   b.callback          = panel_toggle_visibility;
   b.userdata          = NULL;
-  xitk_list_append_content(panel->widget_list->l, (w = xitk_button_create(gGui->skin_config, &b)));
+  xitk_list_append_content(panel->widget_list->l,
+			   (w = xitk_button_create(panel->widget_list, gGui->skin_config, &b)));
   xitk_set_widget_tips(w, _("Hide GUI"));
 
   /*  Fullscreen button */
   b.skin_element_name = "FullScreen";
-  b.callback          = gui_toggle_fullscreen;
+  b.callback          = gui_set_fullscreen_mode;
   b.userdata          = NULL;
-  xitk_list_append_content(panel->widget_list->l, (w = xitk_button_create(gGui->skin_config, &b)));
+  xitk_list_append_content(panel->widget_list->l, 
+			   (w = xitk_button_create(panel->widget_list, gGui->skin_config, &b)));
   xitk_set_widget_tips(w, _("Fullscreen/Window mode"));
 
   /*  Next audio channel */
   b.skin_element_name = "AudioNext";
   b.callback          = gui_change_audio_channel;
   b.userdata          = (void *)GUI_NEXT;
-  xitk_list_append_content(panel->widget_list->l, (w = xitk_button_create(gGui->skin_config, &b)));
+  xitk_list_append_content(panel->widget_list->l, 
+			   (w = xitk_button_create(panel->widget_list, gGui->skin_config, &b)));
   xitk_set_widget_tips(w, _("Next audio channel"));
 
   /*  Prev audio channel */
   b.skin_element_name = "AudioPrev";
   b.callback          = gui_change_audio_channel;
   b.userdata          = (void *)GUI_PREV;
-  xitk_list_append_content(panel->widget_list->l, (w = xitk_button_create(gGui->skin_config, &b)));
+  xitk_list_append_content(panel->widget_list->l, 
+			   (w = xitk_button_create(panel->widget_list, gGui->skin_config, &b)));
   xitk_set_widget_tips(w, _("Previous audio channel"));
 
   /*  Prev spuid */
   b.skin_element_name = "SpuNext";
   b.callback          = gui_change_spu_channel;
   b.userdata          = (void *)GUI_NEXT;
-  xitk_list_append_content(panel->widget_list->l, (w = xitk_button_create(gGui->skin_config, &b)));
+  xitk_list_append_content(panel->widget_list->l, 
+			   (w = xitk_button_create(panel->widget_list, gGui->skin_config, &b)));
   xitk_set_widget_tips(w, _("Next SPU channel"));
 
   /*  Next spuid */
   b.skin_element_name = "SpuPrev";
   b.callback          = gui_change_spu_channel;
   b.userdata          = (void *)GUI_PREV;
-  xitk_list_append_content(panel->widget_list->l, (w = xitk_button_create(gGui->skin_config, &b)));
+  xitk_list_append_content(panel->widget_list->l, 
+			   (w = xitk_button_create(panel->widget_list, gGui->skin_config, &b)));
   xitk_set_widget_tips(w, _("Previous SPU channel"));
 
   /*  Label title */
   lbl.skin_element_name = "TitleLabel";
   lbl.label             = gGui->filename;
+  lbl.callback          = NULL;
   xitk_list_append_content (panel->widget_list->l, 
-			   (panel->title_label = xitk_label_create (gGui->skin_config, &lbl)));
+	   (panel->title_label = xitk_label_create (panel->widget_list, gGui->skin_config, &lbl)));
 
   /*  Runtime label */
   lbl.skin_element_name = "TimeLabel";
   lbl.label             = panel->runtime;
+  lbl.callback          = NULL;
   xitk_list_append_content (panel->widget_list->l, 
-			   (panel->runtime_label = xitk_label_create (gGui->skin_config, &lbl)));
+	   (panel->runtime_label = xitk_label_create (panel->widget_list, gGui->skin_config, &lbl)));
+  /* 
+   * Init to default, otherwise if panel is hide
+   * at startup, label is empty 'till it's updated
+   */
+  xitk_label_change_label (panel->widget_list, panel->runtime_label, "00:00:00"); 
 
   /*  Audio channel label */
   lbl.skin_element_name = "AudioLabel";
   lbl.label             = panel->audiochan;
+  lbl.callback          = NULL;
   xitk_list_append_content (panel->widget_list->l, 
-			   (panel->audiochan_label = xitk_label_create (gGui->skin_config, &lbl)));
+	   (panel->audiochan_label = xitk_label_create (panel->widget_list, gGui->skin_config, &lbl)));
 
   /*  Spuid label */
   lbl.skin_element_name = "SpuLabel";
   lbl.label             = panel->spuid;
+  lbl.callback          = NULL;
   xitk_list_append_content (panel->widget_list->l, 
-			   (panel->spuid_label = xitk_label_create (gGui->skin_config, &lbl)));
+	   (panel->spuid_label = xitk_label_create (panel->widget_list, gGui->skin_config, &lbl)));
 
   /*  slider seek */
   sl.skin_element_name = "SliderPlay";
@@ -822,8 +927,9 @@ void panel_init (void) {
   sl.motion_callback   = panel_slider_cb;
   sl.motion_userdata   = NULL;
   xitk_list_append_content (panel->widget_list->l, 
-			   (panel->slider_play = xitk_slider_create(gGui->skin_config, &sl)));
-  xitk_set_widget_tips(panel->slider_play, _("Stream playback position slider"));
+    (panel->playback_widgets.slider_play = xitk_slider_create(panel->widget_list, gGui->skin_config, &sl)));
+  xitk_set_widget_tips(panel->playback_widgets.slider_play, _("Stream playback position slider"));
+  xitk_slider_reset(panel->widget_list, panel->playback_widgets.slider_play);
 
   /* Mixer volume slider */
   sl.skin_element_name = "SliderVol";
@@ -835,7 +941,8 @@ void panel_init (void) {
   sl.motion_callback   = panel_slider_cb;
   sl.motion_userdata   = NULL;
   xitk_list_append_content (panel->widget_list->l, 
-			   (panel->mixer.slider = xitk_slider_create(gGui->skin_config, &sl)));
+	   (panel->mixer.slider = xitk_slider_create(panel->widget_list, gGui->skin_config, &sl)));
+  xitk_slider_reset(panel->widget_list, panel->mixer.slider);
   xitk_disable_widget(panel->mixer.slider);
 
   /*  Mute toggle */
@@ -843,66 +950,73 @@ void panel_init (void) {
   cb.callback          = panel_toggle_audio_mute;
   cb.userdata          = NULL;
   xitk_list_append_content (panel->widget_list->l, 
-			   (panel->mixer.mute = xitk_checkbox_create (gGui->skin_config, &cb)));
+	   (panel->mixer.mute = xitk_checkbox_create (panel->widget_list, gGui->skin_config, &cb)));
   xitk_disable_widget(panel->mixer.mute);
 
   /* Snapshot */
   b.skin_element_name = "Snapshot";
   b.callback          = panel_snapshot;
   b.userdata          = NULL;
-  xitk_list_append_content(panel->widget_list->l, (w = xitk_button_create(gGui->skin_config, &b)));
+  xitk_list_append_content(panel->widget_list->l, 
+			   (w = xitk_button_create(panel->widget_list, gGui->skin_config, &b)));
   xitk_set_widget_tips(w, _("Take a snapshot"));
 
   /*  Playback speed slow */
   b.skin_element_name = "PlaySlow";
   b.callback          = gui_change_speed_playback;
   b.userdata          = (void *)GUI_NEXT;
-  xitk_list_append_content(panel->widget_list->l, (w = xitk_button_create(gGui->skin_config, &b)));
+  xitk_list_append_content(panel->widget_list->l, 
+			   (w = xitk_button_create(panel->widget_list, gGui->skin_config, &b)));
   xitk_set_widget_tips(w, _("Slow motion"));
 
   /*  Playback speed fast */
   b.skin_element_name = "PlayFast";
   b.callback          = gui_change_speed_playback;
   b.userdata          = (void *)GUI_PREV;
-  xitk_list_append_content(panel->widget_list->l, (w = xitk_button_create(gGui->skin_config, &b)));
+  xitk_list_append_content(panel->widget_list->l, 
+			   (w = xitk_button_create(panel->widget_list, gGui->skin_config, &b)));
   xitk_set_widget_tips(w, _("Fast motion"));
   
   /*  Playlist button */
   b.skin_element_name = "PlBtn";
   b.callback          = gui_playlist_show;
   b.userdata          = NULL;
-  xitk_list_append_content(panel->widget_list->l, (w = xitk_button_create(gGui->skin_config, &b)));
+  xitk_list_append_content(panel->widget_list->l, 
+			   (w = xitk_button_create(panel->widget_list, gGui->skin_config, &b)));
   xitk_set_widget_tips(w, _("Playlist"));
   
   /*  Control button */
   b.skin_element_name = "CtlBtn";
   b.callback          = gui_control_show;
   b.userdata          = NULL;
-  xitk_list_append_content(panel->widget_list->l, (w = xitk_button_create(gGui->skin_config, &b)));
+  xitk_list_append_content(panel->widget_list->l, 
+			   (w = xitk_button_create(panel->widget_list, gGui->skin_config, &b)));
   xitk_set_widget_tips(w, _("Control"));
 
   /*  Mrl button */
   b.skin_element_name = "MrlBtn";
   b.callback          = gui_mrlbrowser_show;
   b.userdata          = NULL;
-  xitk_list_append_content(panel->widget_list->l, (w = xitk_button_create(gGui->skin_config, &b)));
+  xitk_list_append_content(panel->widget_list->l, 
+			   (w = xitk_button_create(panel->widget_list, gGui->skin_config, &b)));
   xitk_set_widget_tips(w, _("MRL Browser"));
   
 
   panel->tips.enable = gGui->config->register_bool(gGui->config, "gui.tips_visible", 1,
-						   "gui tips visibility", NULL,
+						   _("gui tips visibility"), NULL,
 						   panel_enable_tips_cb, NULL);
   panel->tips.timeout = gGui->config->register_num(gGui->config, "gui.tips_timeout", 500,
-						   "tips timeout (ms)", NULL, 
+						   _("tips timeout (ms)"), NULL, 
 						   panel_timeout_tips_cb, NULL);
 
+  panel_update_nextprev_tips();
   panel_show_tips();
 
   /* 
    * show panel 
    */
   panel->visible = gGui->config->register_bool (gGui->config, "gui.panel_visible", 1,
-						"gui panel visibility",
+						_("gui panel visibility"),
 						NULL, NULL, NULL);
   
   if(gGui->use_root_window && (!panel->visible))
@@ -942,6 +1056,7 @@ void panel_init (void) {
     
     pthread_create(&panel->slider_thread, &pth_attrs, slider_loop, NULL);
   }
+
 }
 
 void panel_set_title(char *title) {
