@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: playlist.c,v 1.49 2002/04/21 22:17:12 f1rmb Exp $
+ * $Id: playlist.c,v 1.62 2002/08/01 08:38:58 f1rmb Exp $
  *
  * Playlist Editor
  *
@@ -32,23 +32,19 @@
 #include <X11/Xutil.h>
 #include <X11/keysym.h>
 
-#include "xitk.h"
-
 #include "Imlib-light/Imlib.h"
+#include "panel.h"
 #include "event.h"
 #include "actions.h"
 #include "mrl_browser.h"
 #include "errors.h"
+#include "i18n.h"
+
+#include "xitk.h"
 
 #include <xine.h>
 #include <xine/xineutils.h>
 
-#ifndef __GNUC__
-#define __FUNCTION__    __func__
-#endif
-
-#define MAX_LIST 9
-
 extern gGui_t          *gGui;
 
 typedef struct {
@@ -196,13 +192,17 @@ static void pl_on_dbl_click(xitk_widget_t *w, void *data, int selected) {
  */
 static void pl_delete(xitk_widget_t *w, void *data) {
   int i, j;
-
+  
   j = xitk_browser_get_current_selected(playlist->playlist);
   
   if(j >= 0) {
-    for(i = j; i < gGui->playlist_num; i++) {
+
+    if((gGui->playlist_cur == j) && ((xine_get_status(gGui->xine) != XINE_STOP)))
+      gui_stop(NULL, NULL);
+
+    for(i = j; i < gGui->playlist_num; i++)
       gGui->playlist[i] = gGui->playlist[i+1];
-    }
+
     gGui->playlist_num--;
     if(gGui->playlist_cur) gGui->playlist_cur--;
   }
@@ -211,8 +211,16 @@ static void pl_delete(xitk_widget_t *w, void *data) {
   
   if(gGui->playlist_num)
     gui_set_current_mrl(gGui->playlist[gGui->playlist_cur]);
-  else
+  else {
+    
+    if(is_playback_widgets_enabled() && (!gGui->playlist_num))
+      enable_playback_controls(0);
+    
+    if(xine_get_status(gGui->xine) != XINE_STOP)
+      gui_stop(NULL, NULL);
+
     gui_set_current_mrl(NULL);
+  }
 
 }
 
@@ -221,18 +229,21 @@ static void pl_delete(xitk_widget_t *w, void *data) {
  */
 static void pl_delete_all(xitk_widget_t *w, void *data) {
   int i;
-
-  for(i = 0; i < gGui->playlist_num; i++) {
+  
+  for(i = 0; i < gGui->playlist_num; i++)
     gGui->playlist[i] = NULL;
-  }
 
   gGui->playlist_num = 0;
   gGui->playlist_cur = 0;
 
+  if(xine_get_status(gGui->xine) != XINE_STOP)
+    gui_stop(NULL, NULL);
+
   xitk_browser_update_list(playlist->playlist, 
 			   gGui->playlist, gGui->playlist_num, 0);
-  gui_set_current_mrl(NULL);
 
+  gui_set_current_mrl(NULL);
+  enable_playback_controls(0);
 }
 
 /*
@@ -240,6 +251,7 @@ static void pl_delete_all(xitk_widget_t *w, void *data) {
  */
 static void pl_move_updown(xitk_widget_t *w, void *data) {
   int j;
+  int max_entries = xitk_browser_get_num_entries(playlist->playlist);
   
   j = xitk_browser_get_current_selected(playlist->playlist);
   
@@ -263,10 +275,10 @@ static void pl_move_updown(xitk_widget_t *w, void *data) {
       for(i=0; i<gGui->playlist_num; i++)
 	gGui->playlist[i] = tmplist[i];
 
-      if(j > MAX_LIST || (xitk_browser_get_current_start(playlist->playlist) != 0)) {
-	j -= ((MAX_LIST-1)/2);
+      if(j > max_entries || (xitk_browser_get_current_start(playlist->playlist) != 0)) {
+	j -= ((max_entries - 1) / 2);
 	xitk_browser_rebuild_browser(playlist->playlist, j);
-	xitk_browser_set_select(playlist->playlist, ((MAX_LIST-1)/2)-1);
+	xitk_browser_set_select(playlist->playlist, ((max_entries - 1) / 2) - 1);
       }
       else {
 	j = xitk_browser_get_current_selected(playlist->playlist);
@@ -292,7 +304,7 @@ static void pl_move_updown(xitk_widget_t *w, void *data) {
       for(i=0; i<gGui->playlist_num; i++)
 	gGui->playlist[i] = tmplist[i];
 
-      if(j >= (MAX_LIST-1)) {
+      if(j >= (max_entries - 1)) {
 	xitk_browser_rebuild_browser(playlist->playlist, j);
 	xitk_browser_set_select(playlist->playlist, 1);
       }
@@ -464,6 +476,7 @@ void pl_scan_input(xitk_widget_t *w, void *ip) {
 	xitk_browser_update_list(playlist->playlist, 
 				 gGui->playlist, gGui->playlist_num, 0);
     }
+    enable_playback_controls((gGui->playlist_num > 0));
   }
 }
 
@@ -521,6 +534,30 @@ void pl_toggle_visibility (xitk_widget_t *w, void *data) {
   }
 }
 
+/*
+ * Update first displayed entry and input text widget content with
+ * current selected playlist entry.
+ */
+void pl_update_focused_entry(void) {
+
+  if(playlist != NULL) {
+    if(playlist->window) {
+      if(playlist->visible && playlist->running && gGui->playlist_num) {
+	if(!strcmp(gGui->filename, gGui->playlist[gGui->playlist_cur])) {
+	  int max_displayed = xitk_browser_get_num_entries(playlist->playlist);
+	  
+	  if((gGui->playlist_num - gGui->playlist_cur) >= max_displayed)
+	    xitk_browser_update_list(playlist->playlist, 
+				     gGui->playlist, gGui->playlist_num, gGui->playlist_cur);
+	  
+	  xitk_inputtext_change_text(playlist->widget_list, 
+				     playlist->winput, gGui->playlist[gGui->playlist_cur]);
+	}
+      }
+    }
+  }
+}
+
 /*
  * Handle X events here.
  */
@@ -541,34 +578,35 @@ void playlist_handle_event(XEvent *event, void *data) {
   }
   break;
 
+  case ButtonRelease:
+    gui_handle_event(event, data);
+    break;
+
   case KeyPress:
-    if(xitk_is_widget_focused(playlist->winput)) {
-      xitk_send_key_event(playlist->widget_list, playlist->winput, event);
-    }
-    else {
+    if(!playlist)
+      return;
+    
+    mykeyevent = event->xkey;
+    
+    XLockDisplay(gGui->display);
+    len = XLookupString(&mykeyevent, kbuf, sizeof(kbuf), &mykey, NULL);
+    XUnlockDisplay(gGui->display);
+    
+    switch (mykey) {
       
-      mykeyevent = event->xkey;
+    case XK_Down:
+    case XK_Next:
+      xitk_browser_step_up(playlist->playlist, NULL);
+      break;
       
-      XLockDisplay(gGui->display);
-      len = XLookupString(&mykeyevent, kbuf, sizeof(kbuf), &mykey, NULL);
-      XUnlockDisplay(gGui->display);
+    case XK_Up:
+    case XK_Prior:
+      xitk_browser_step_down(playlist->playlist, NULL);
+      break;
       
-      switch (mykey) {
-	
-      case XK_Down:
-      case XK_Next:
-	xitk_browser_step_up(playlist->playlist, NULL);
-	break;
-	
-      case XK_Up:
-      case XK_Prior:
-	xitk_browser_step_down(playlist->playlist, NULL);
-	break;
-	
-      default:
-	gui_handle_event(event, data);
-	break;
-      }
+    default:
+      gui_handle_event(event, data);
+      break;
     }
     break;
 
@@ -606,7 +644,7 @@ void playlist_change_skins(void) {
     
     if(!(new_img = Imlib_load_image(gGui->imlib_data,
 				    xitk_skin_get_skin_filename(gGui->skin_config, "PlBG")))) {
-      xine_error("%s(): couldn't find image for background\n", __FUNCTION__);
+      xine_error(_("%s(): couldn't find image for background\n"), __XINE_FUNCTION__);
       exit(-1);
     }
     
@@ -670,7 +708,7 @@ void playlist_editor(void) {
   XSizeHints                 hint;
   XWMHints                  *wm_hint;
   XSetWindowAttributes       attr;
-  char                       title[] = {"Xine Playlist Editor"};
+  char                      *title;
   Atom                       prop, XA_WIN_LAYER;
   MWMHints                   mwmhints;
   XClassHint                *xclasshint;
@@ -688,6 +726,8 @@ void playlist_editor(void) {
       return;
   }
 
+  xine_strdupa(title, _("Xine Playlist Editor"));
+
   XITK_WIDGET_INIT(&br, gGui->imlib_data);
   XITK_WIDGET_INIT(&lb, gGui->imlib_data);
   XITK_WIDGET_INIT(&lbl, gGui->imlib_data);
@@ -700,7 +740,7 @@ void playlist_editor(void) {
 
   if (!(playlist->bg_image = Imlib_load_image(gGui->imlib_data,
 					      xitk_skin_get_skin_filename(gGui->skin_config, "PlBG")))) {
-    xine_error("xine-playlist: couldn't find image for background\n");
+    xine_error(_("xine-playlist: couldn't find image for background\n"));
     exit(-1);
   }
 
@@ -771,7 +811,7 @@ void playlist_editor(void) {
   /* set xclass */
 
   if((xclasshint = XAllocClassHint()) != NULL) {
-    xclasshint->res_name = "Xine Playlist Editor";
+    xclasshint->res_name = title;//"Xine Playlist Editor";
     xclasshint->res_class = "Xine";
     XSetClassHint(gGui->display, playlist->window, xclasshint);
   }
@@ -794,8 +834,6 @@ void playlist_editor(void) {
    */
   playlist->widget_list                = xitk_widget_list_new();
   playlist->widget_list->l             = xitk_list_new();
-  playlist->widget_list->focusedWidget = NULL;
-  playlist->widget_list->pressedWidget = NULL;
   playlist->widget_list->win           = playlist->window;
   playlist->widget_list->gc            = gc;
 
@@ -806,7 +844,7 @@ void playlist_editor(void) {
   b.callback          = pl_move_updown;
   b.userdata          = (void *)MOVEUP;
   xitk_list_append_content (playlist->widget_list->l, 
-			    (w = xitk_button_create (gGui->skin_config, &b)));
+		    (w = xitk_button_create (playlist->widget_list, gGui->skin_config, &b)));
   xitk_set_widget_tips(w, _("Move up selected MRL"));
 
 
@@ -814,28 +852,28 @@ void playlist_editor(void) {
   b.callback          = pl_move_updown;
   b.userdata          = (void *)MOVEDN;
   xitk_list_append_content (playlist->widget_list->l, 
-			    (w = xitk_button_create (gGui->skin_config, &b)));
+		    (w = xitk_button_create (playlist->widget_list, gGui->skin_config, &b)));
   xitk_set_widget_tips(w, _("Move down selected MRL"));
 
   b.skin_element_name = "PlPlay";
   b.callback          = pl_play;
   b.userdata          = NULL;
   xitk_list_append_content (playlist->widget_list->l, 
-			    (w = xitk_button_create (gGui->skin_config, &b)));
+		    (w = xitk_button_create (playlist->widget_list, gGui->skin_config, &b)));
   xitk_set_widget_tips(w, _("Start playback of selected MRL"));
   
   b.skin_element_name = "PlDelete";
   b.callback          = pl_delete;
   b.userdata          = NULL;
   xitk_list_append_content (playlist->widget_list->l, 
-			    (w = xitk_button_create (gGui->skin_config, &b)));
+		    (w = xitk_button_create (playlist->widget_list, gGui->skin_config, &b)));
   xitk_set_widget_tips(w, _("Delete selected MRL from playlist"));
 
   b.skin_element_name = "PlDeleteAll";
   b.callback          = pl_delete_all;
   b.userdata          = NULL;
   xitk_list_append_content (playlist->widget_list->l, 
-			    (w = xitk_button_create (gGui->skin_config, &b)));
+		    (w = xitk_button_create (playlist->widget_list, gGui->skin_config, &b)));
   xitk_set_widget_tips(w, _("Delete all entries in playlist"));
 
   lb.skin_element_name = "PlAdd";
@@ -845,7 +883,7 @@ void playlist_editor(void) {
   lb.state_callback    = NULL;
   lb.userdata          = NULL;
   xitk_list_append_content (playlist->widget_list->l, 
-			    (w = xitk_labelbutton_create (gGui->skin_config, &lb)));
+		    (w = xitk_labelbutton_create (playlist->widget_list, gGui->skin_config, &lb)));
   xitk_set_widget_tips(w, _("Add one or more entries in playlist"));
     
 
@@ -856,7 +894,7 @@ void playlist_editor(void) {
   lb.state_callback    = NULL;
   lb.userdata          = NULL;
   xitk_list_append_content (playlist->widget_list->l, 
-			    (w = xitk_labelbutton_create (gGui->skin_config, &lb)));
+		    (w = xitk_labelbutton_create (playlist->widget_list, gGui->skin_config, &lb)));
   xitk_set_widget_tips(w, _("Load saved playlist"));
 
   lb.skin_element_name = "PlSave";
@@ -866,7 +904,7 @@ void playlist_editor(void) {
   lb.state_callback    = NULL;
   lb.userdata          = NULL;
   xitk_list_append_content (playlist->widget_list->l, 
-			    (w = xitk_labelbutton_create (gGui->skin_config, &lb)));
+		    (w = xitk_labelbutton_create (playlist->widget_list, gGui->skin_config, &lb)));
   xitk_set_widget_tips(w, _("Save playlist"));
 
   lb.skin_element_name = "PlDismiss";
@@ -876,28 +914,26 @@ void playlist_editor(void) {
   lb.state_callback    = NULL;
   lb.userdata          = NULL;
   xitk_list_append_content (playlist->widget_list->l, 
-			    (w = xitk_labelbutton_create (gGui->skin_config, &lb)));
+		    (w = xitk_labelbutton_create (playlist->widget_list, gGui->skin_config, &lb)));
   xitk_set_widget_tips(w, _("Close playlist window"));
 
   br.arrow_up.skin_element_name    = "PlUp";
   br.slider.skin_element_name      = "SliderPl";
   br.arrow_dn.skin_element_name    = "PlDn";
   br.browser.skin_element_name     = "PlItemBtn";
-  br.browser.max_displayed_entries = 9;
   br.browser.num_entries           = gGui->playlist_num;
   br.browser.entries               = gGui->playlist;
   br.callback                      = handle_selection;
   br.dbl_click_callback            = pl_on_dbl_click;
-  br.dbl_click_time                = DEFAULT_DBL_CLICK_TIME;
   br.parent_wlist                  = playlist->widget_list;
   xitk_list_append_content (playlist->widget_list->l, 
-			   (playlist->playlist = 
-			    xitk_browser_create(gGui->skin_config, &br)));
+   (playlist->playlist = xitk_browser_create(playlist->widget_list, gGui->skin_config, &br)));
 
   lbl.skin_element_name = "AutoPlayLbl";
   lbl.label             = _("Scan for:");
+  lbl.callback          = NULL;
   xitk_list_append_content (playlist->widget_list->l,
-			    xitk_label_create (gGui->skin_config, &lbl));
+			    xitk_label_create (playlist->widget_list, gGui->skin_config, &lbl));
   
   inp.skin_element_name = "PlInputText";
   inp.text              = NULL;
@@ -905,7 +941,7 @@ void playlist_editor(void) {
   inp.callback          = pl_add_input;
   inp.userdata          = NULL;
   xitk_list_append_content (playlist->widget_list->l,
-			   (playlist->winput = xitk_inputtext_create (gGui->skin_config, &inp)));
+   (playlist->winput = xitk_inputtext_create (playlist->widget_list, gGui->skin_config, &inp)));
   xitk_set_widget_tips(playlist->winput, _("Direct MRL entry"));
 
   {
@@ -926,7 +962,7 @@ void playlist_editor(void) {
       lb.userdata          = NULL;
       xitk_list_append_content (playlist->widget_list->l,
 	       (playlist->autoplay_plugins[i] = 
-		xitk_labelbutton_create (gGui->skin_config, &lb)));
+		xitk_labelbutton_create (playlist->widget_list, gGui->skin_config, &lb)));
       xitk_set_widget_tips(playlist->autoplay_plugins[i], 
 			   xine_get_input_plugin_description(gGui->xine, autoplay_plugins[i]));
       
@@ -942,6 +978,8 @@ void playlist_editor(void) {
   xitk_browser_update_list(playlist->playlist, 
 			   gGui->playlist, gGui->playlist_num, 0);
 
+  playlist_show_tips(panel_get_tips_enable(), panel_get_tips_timeout());
+
   XMapRaised(gGui->display, playlist->window); 
   
   XUnlockDisplay (gGui->display);
@@ -956,4 +994,5 @@ void playlist_editor(void) {
 
   playlist->visible = 1;
   playlist->running = 1;
+  pl_update_focused_entry();
 }
