@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: setup.c,v 1.29 2002/04/12 21:38:51 f1rmb Exp $
+ * $Id: setup.c,v 1.40 2002/07/20 11:12:56 f1rmb Exp $
  *
  *
  */
@@ -36,20 +36,22 @@
 #include <xine.h>
 #include <xine/xineutils.h>
 
-#include "xitk.h"
-
 #include "Imlib-light/Imlib.h"
 #include "event.h"
 #include "actions.h"
+#include "viewlog.h"
+#include "kbindings.h"
 #include "skins.h"
 #include "lang.h"
+#include "i18n.h"
+
+#include "xitk.h"
 
 extern gGui_t              *gGui;
 
-static char                *fontname = "-*-helvetica-medium-r-*-*-10-*-*-*-*-*-*-*";
-
-#define WGPREV              1
-#define WGNEXT              2
+static char                *fontname     = "-*-helvetica-medium-r-*-*-10-*-*-*-*-*-*-*";
+static char                *boldfontname = "-*-helvetica-bold-r-*-*-10-*-*-*-*-*-*-*";
+static char                *tabsfontname = "-*-helvetica-bold-r-*-*-12-*-*-*-*-*-*-*";
 
 #define WINDOW_WIDTH        500
 #define WINDOW_HEIGHT       480
@@ -65,12 +67,11 @@ static char                *fontname = "-*-helvetica-medium-r-*-*-10-*-*-*-*-*-*
     xitk_image_t        *image;                                                                 \
     xitk_image_widget_t  im;                                                                    \
     int                  lbearing, rbearing, width, ascent, descent;                            \
-    char                *fontname = "*-lucida-*-r-*-*-10-*-*-*-*-*-*-*";                        \
     xitk_font_t         *fs;                                                                    \
                                                                                                 \
     image = xitk_image_create_image(gGui->imlib_data, FRAME_WIDTH + 1, FRAME_HEIGHT + 1);       \
                                                                                                 \
-    fs = xitk_font_load_font(gGui->display, fontname);                                          \
+    fs = xitk_font_load_font(gGui->display, boldfontname);                                      \
     xitk_font_set_font(fs, setup->widget_list->gc);                                             \
     xitk_font_text_extent(fs, title, strlen(title),                                             \
                           &lbearing, &rbearing, &width, &ascent, &descent);                     \
@@ -84,14 +85,14 @@ static char                *fontname = "-*-helvetica-medium-r-*-*-10-*-*-*-*-*-*
 		   0, 0, image->width, image->height);                                          \
     XUnlockDisplay(gGui->display);                                                              \
                                                                                                 \
-    draw_inner_frame(gGui->imlib_data, image->image, title, fontname,                           \
+    draw_inner_frame(gGui->imlib_data, image->image, title, boldfontname,                       \
                      0, (ascent+descent), FRAME_WIDTH, FRAME_HEIGHT);                           \
 	                                                                                        \
     XITK_WIDGET_INIT(&im, gGui->imlib_data);                                                    \
     im.skin_element_name = NULL;                                                                \
                                                                                                 \
     xitk_list_append_content (setup->widget_list->l,                                            \
-			      (frame = xitk_noskin_image_create(&im, image, x, y)));            \
+	      (frame = xitk_noskin_image_create(setup->widget_list, &im, image, x, y)));        \
                                                                                                 \
     setup->tmp_widgets[setup->num_tmp_widgets] = frame;                                         \
     setup->num_tmp_widgets++;                                                                   \
@@ -150,9 +151,9 @@ static char                *fontname = "-*-helvetica-medium-r-*-*-10-*-*-*-*-*-*
 
 
 typedef struct {
-  xitk_widget_t *frame;
-  xitk_widget_t *label;
-  xitk_widget_t *widget;
+  xitk_widget_t        *frame;
+  xitk_widget_t        *label;
+  xitk_widget_t        *widget;
 } widget_triplet_t;
 
 typedef struct {
@@ -170,7 +171,7 @@ typedef struct {
   xitk_widget_t        *tmp_widgets[200];
   int                   num_tmp_widgets;
 
-  xitk_widget_t        *btnup, *btndn;
+  xitk_widget_t        *slider_wg;
   widget_triplet_t     *wg[100];
   int                   num_wg;
   int                   first_displayed;
@@ -317,6 +318,7 @@ void setup_raise_window(void) {
     }
   }
 }
+
 /*
  * Hide/show the setup panel
  */
@@ -395,11 +397,11 @@ static void setup_paint_widgets(void) {
 
     xitk_get_widget_pos(setup->wg[i]->widget, &wx, &wy);
     /* Inputtext/intbox/combo widgets have special treatments */
-    if(setup->wg[i]->widget->widget_type & WIDGET_TYPE_COMBO) {
+    if((setup->wg[i]->widget->widget_type & WIDGET_GROUP_MASK) & WIDGET_GROUP_COMBO) {
       xitk_set_widget_pos(setup->wg[i]->widget, wx, y - 4);
     }
-    else if(setup->wg[i]->widget->widget_type & WIDGET_TYPE_INPUTTEXT ||
-	    setup->wg[i]->widget->widget_type & WIDGET_TYPE_INTBOX)
+    else if(((setup->wg[i]->widget->widget_type & WIDGET_TYPE_MASK) == WIDGET_TYPE_INPUTTEXT) ||
+    	    ((setup->wg[i]->widget->widget_type & WIDGET_GROUP_MASK) & WIDGET_GROUP_INTBOX))
       xitk_set_widget_pos(setup->wg[i]->widget, wx, y - 5);
     else
       xitk_set_widget_pos(setup->wg[i]->widget, wx, y);
@@ -409,25 +411,6 @@ static void setup_paint_widgets(void) {
     y += (FRAME_HEIGHT>>1) + 2;
   }
 
-  /* Hide Up/Down button only i first displayed triplet is 0 and there no extra widgets */
-  if((setup->first_displayed == 0) && (i >= setup->num_wg)) {
-    xitk_disable_widget(setup->btnup);
-    xitk_hide_widget(setup->widget_list, setup->btnup);
-    xitk_disable_widget(setup->btndn);
-    xitk_hide_widget(setup->widget_list, setup->btndn);
-  }
-
-  if(i<setup->num_wg) {
-    xitk_enable_widget(setup->btnup);
-    xitk_show_widget(setup->widget_list, setup->btnup);
-    xitk_enable_widget(setup->btndn);
-    xitk_show_widget(setup->widget_list, setup->btndn);
-  }
-  /* Disable widget out of window */
-  for(; i < setup->num_wg; i++) {
-    DISABLE_ME(setup->wg[i]);
-  }
-    
   /* Repaint them now */
   xitk_paint_widget_list (setup->widget_list); 
 }
@@ -435,58 +418,48 @@ static void setup_paint_widgets(void) {
 /*
  * Handle X events here.
  */
-void setup_handle_event(XEvent *event, void *data) {
-  XKeyEvent      mykeyevent;
-  KeySym         mykey;
-  char           kbuf[256];
-  int            len;
-
+static void setup_handle_event(XEvent *event, void *data) {
+  
   switch(event->type) {
-
+    
   case KeyPress: {
-    xitk_widget_t *w = xitk_get_focused_widget(setup->widget_list);
-
-    if(w && (w->widget_type & WIDGET_TYPE_INPUTTEXT)) {
-      xitk_send_key_event(setup->widget_list, w, event);
-    }
-    else {
+    XKeyEvent      mykeyevent;
+    KeySym         mykey;
+    char           kbuf[256];
+    int            len;
+    int            modifier;
+    
+    if(xitk_is_widget_enabled(setup->slider_wg)) {
+      
       mykeyevent = event->xkey;
       
+      xitk_get_key_modifier(event, &modifier);
+      
       XLockDisplay(gGui->display);
       len = XLookupString(&mykeyevent, kbuf, sizeof(kbuf), &mykey, NULL);
       XUnlockDisplay(gGui->display);
       
-      switch (mykey) {
+      switch(mykey) {
 	
-      case XK_Return:
-	setup_end(NULL, NULL);
+      case XK_Up:
+	if((modifier & 0xFFFFFFEF) == MODIFIER_NOMOD) {
+	  xitk_slider_make_step(setup->widget_list, setup->slider_wg);
+	  xitk_slider_callback_exec(setup->slider_wg);
+	}
 	break;
 	
-      default:
-	gui_handle_event(event, data);
+      case XK_Down:
+	if((modifier & 0xFFFFFFEF) == MODIFIER_NOMOD) { 
+	  xitk_slider_make_backstep(setup->widget_list, setup->slider_wg);
+	  xitk_slider_callback_exec(setup->slider_wg);
+	}
 	break;
+
       }
     }
   }
   break;
-    
-  case MappingNotify:
-    XLockDisplay(gGui->display);
-    XRefreshKeyboardMapping((XMappingEvent *) event);
-    XUnlockDisplay(gGui->display);
-    break;
-
-  case ConfigureNotify: {
-    xitk_widget_t *w = (xitk_widget_t *) xitk_list_first_content (setup->widget_list->l);
-    
-    while (w) {
-      if(w->widget_type & WIDGET_TYPE_COMBO)
-	xitk_combo_update_pos(w);
-      w = (xitk_widget_t *) xitk_list_next_content (setup->widget_list->l);
-    }
-  }
-  break;
-
+  
   }
 }
 
@@ -510,9 +483,10 @@ static xitk_widget_t *setup_add_label (int x, int y, int w, char *str) {
   lb.gc                  = setup->widget_list->gc;
   lb.skin_element_name   = NULL;
   lb.label               = str;
+  lb.callback            = NULL;
 
   xitk_list_append_content(setup->widget_list->l, 
-			   (label = xitk_noskin_label_create(&lb,
+			   (label = xitk_noskin_label_create(setup->widget_list, &lb,
 							     x, y, w, height, fontname)));
 
   setup->tmp_widgets[setup->num_tmp_widgets] = label;
@@ -544,7 +518,7 @@ static void stringtype_update(xitk_widget_t *w, void *data, char *str) {
   check_entry = gGui->config->lookup_entry(gGui->config, entry->key);
 
   if(check_entry) {
-    if(w->widget_type & WIDGET_TYPE_INPUTTEXT)
+    if((w->widget_type & WIDGET_TYPE_MASK) == WIDGET_TYPE_INPUTTEXT)
       xitk_inputtext_change_text(setup->widget_list, w, check_entry->str_value);
   }
 }
@@ -573,7 +547,7 @@ static widget_triplet_t *setup_add_slider (char *title, char *labelkey,
   sl.motion_callback          = numtype_update;
   sl.motion_userdata          = entry;
   xitk_list_append_content(setup->widget_list->l,
-			   (slider = xitk_noskin_slider_create(&sl,
+			   (slider = xitk_noskin_slider_create(setup->widget_list, &sl,
 							       x, y, 150, 16,
 							       XITK_HSLIDER)));
   xitk_slider_set_pos(setup->widget_list, slider, entry->num_value);
@@ -610,8 +584,8 @@ static widget_triplet_t *setup_add_inputnum(char *title, char *labelkey,
   ib.callback          = numtype_update;
   ib.userdata          = (void *)entry;
   xitk_list_append_content (setup->widget_list->l,
-			    (intbox = 
-			     xitk_noskin_intbox_create(&ib, x, y - 5, 60, 20, &wi, &wbu, &wbd)));
+    (intbox = 
+     xitk_noskin_intbox_create(setup->widget_list, &ib, x, y - 5, 60, 20, &wi, &wbu, &wbd)));
 
   ADD_LABEL(intbox);
   
@@ -651,7 +625,7 @@ static widget_triplet_t *setup_add_inputtext(char *title, char *labelkey,
   inp.userdata          = entry;
   xitk_list_append_content (setup->widget_list->l,
 			   (input = 
-			    xitk_noskin_inputtext_create(&inp,
+			    xitk_noskin_inputtext_create(setup->widget_list, &inp,
 							 x, y - 5, 150, 20,
 							 "Black", "Black", fontname)));
 
@@ -686,7 +660,7 @@ static widget_triplet_t *setup_add_checkbox (char *title, char *labelkey,
 
   xitk_list_append_content (setup->widget_list->l,
 			   (checkbox = 
-			    xitk_noskin_checkbox_create(&cb,
+			    xitk_noskin_checkbox_create(setup->widget_list, &cb,
 							x, y, 10, 10)));
   xitk_checkbox_set_state (checkbox, entry->num_value, xitk_window_get_window(setup->xwin),
 			   setup->widget_list->gc);
@@ -724,7 +698,7 @@ static widget_triplet_t *setup_add_combo (char *title, char *labelkey,
   cmb.userdata          = entry;
   xitk_list_append_content(setup->widget_list->l, 
 			   (combo = 
-			    xitk_noskin_combo_create(&cmb,
+			    xitk_noskin_combo_create(setup->widget_list, &cmb,
 						     x, y - 4, 150, &lw, &bw)));
   xitk_combo_set_select(setup->widget_list, combo, entry->num_value );
 
@@ -763,12 +737,11 @@ static widget_triplet_t *setup_list_browser(int x, int y, char **content, int le
   br.browser.entries               = content;
   br.callback                      = NULL;
   br.dbl_click_callback            = NULL;
-  br.dbl_click_time                = DEFAULT_DBL_CLICK_TIME;
   br.parent_wlist                  = setup->widget_list;
   br.userdata                      = NULL;
   xitk_list_append_content(setup->widget_list->l, 
 			   (browser = 
-			    xitk_noskin_browser_create(&br,
+			    xitk_noskin_browser_create(setup->widget_list, &br,
 						       setup->widget_list->gc, 
 						       x, y,
 						       (WINDOW_WIDTH - 50) - 16,
@@ -797,7 +770,10 @@ static void setup_section_widgets(int s) {
   int                  len;
   char                *section;
   char                *labelkey;
+  int                  slidmax = 1;
 
+  xitk_disable_widget(setup->slider_wg);
+  
   /* Selected tab is one of help sections */
   if(s >= (setup->num_sections - 3)) {
     if(s == (setup->num_sections - 3)) {
@@ -867,6 +843,15 @@ static void setup_section_widgets(int s) {
       entry = entry->next;
     }
 
+    xitk_enable_widget(setup->slider_wg);
+
+    if(setup->num_wg > MAX_DISPLAY_WIDGETS)
+      slidmax = setup->num_wg - MAX_DISPLAY_WIDGETS;
+    else
+      slidmax = setup->num_wg;
+
+    xitk_slider_set_max(setup->slider_wg, slidmax);
+    xitk_slider_set_pos(setup->widget_list, setup->slider_wg, slidmax);
   }
 }
 
@@ -989,9 +974,9 @@ static void setup_sections (void) {
   tab.callback          = setup_change_section;
   tab.userdata          = NULL;
   xitk_list_append_content (setup->widget_list->l,
-			    (setup->tabs = 
-			     xitk_noskin_tabs_create(&tab, 20, 25, WINDOW_WIDTH - 40)));
-
+    (setup->tabs = 
+     xitk_noskin_tabs_create(setup->widget_list, &tab, 20, 24, WINDOW_WIDTH - 40, tabsfontname)));
+  
   bg = xitk_image_create_pixmap(gGui->imlib_data, WINDOW_WIDTH, WINDOW_HEIGHT);
   
   XLockDisplay(gGui->display);
@@ -1024,23 +1009,14 @@ static void setup_end(xitk_widget_t *w, void *data) {
 /*
  *
  */
-static void setup_nextprev_wg(xitk_widget_t *w, void *data) {
+static void setup_nextprev_wg(xitk_widget_t *w, void *data, int pos) {
+  int rpos = (xitk_slider_get_max(setup->slider_wg)) - pos;
 
-  switch((int)data) {
-
-  case WGPREV:
-    if(setup->first_displayed)
-      setup->first_displayed--;
-    break;
-
-  case WGNEXT:
-    if((setup->first_displayed + MAX_DISPLAY_WIDGETS) < setup->num_wg)
-      setup->first_displayed++;
-    break;
+  if(rpos != setup->first_displayed) {
+    setup->first_displayed = rpos;
+    setup_clear_tab();
+    setup_paint_widgets();
   }
-
-  setup_clear_tab();
-  setup_paint_widgets();
 }
 
 /*
@@ -1112,7 +1088,7 @@ void setup_panel(void) {
   GC                         gc;
   xitk_labelbutton_widget_t  lb;
   xitk_button_widget_t       b;
-  char                      *fontname = "*-lucida-*-r-*-*-10-*-*-*-*-*-*-*";
+  xitk_slider_widget_t       sl;
   int                        x, y;
 
   /* this shouldn't happen */
@@ -1143,42 +1119,28 @@ void setup_panel(void) {
 
   setup->widget_list                = xitk_widget_list_new();
   setup->widget_list->l             = xitk_list_new ();
-  setup->widget_list->focusedWidget = NULL;
-  setup->widget_list->pressedWidget = NULL;
   setup->widget_list->win           = (xitk_window_get_window(setup->xwin));
   setup->widget_list->gc            = gc;
 
 
   XITK_WIDGET_INIT(&b, gGui->imlib_data);
+  XITK_WIDGET_INIT(&sl, gGui->imlib_data);
 
   {
-    xitk_image_t  *wimage;
-    
-    b.skin_element_name = NULL;
-    b.callback          = setup_nextprev_wg;
-    b.userdata          = (void *)WGPREV;
+
+    sl.min                      = 0;
+    sl.max                      = 1;
+    sl.step                     = 1;
+    sl.skin_element_name        = NULL;
+    sl.callback                 = NULL;
+    sl.userdata                 = NULL;
+    sl.motion_callback          = setup_nextprev_wg;
+    sl.motion_userdata          = NULL;
     xitk_list_append_content(setup->widget_list->l,
-	     (setup->btnup = xitk_noskin_button_create(&b, (WINDOW_WIDTH - 40) + 2, 53, 16, 16)));
-    
-    wimage = xitk_get_widget_foreground_skin(setup->btnup);
-    if(wimage)
-      draw_arrow_up(gGui->imlib_data, wimage);
-    
-    
-    b.skin_element_name = NULL;
-    b.callback          = setup_nextprev_wg;
-    b.userdata          = (void *)WGNEXT;
-    xitk_list_append_content(setup->widget_list->l,
-	     (setup->btndn = xitk_noskin_button_create(&b, (WINDOW_WIDTH - 40) + 2, 69, 16, 16)));
-    
-    wimage = xitk_get_widget_foreground_skin(setup->btndn);
-    if(wimage)
-      draw_arrow_down(gGui->imlib_data, wimage);
-    
-    xitk_disable_widget(setup->btnup);
-    xitk_hide_widget(setup->widget_list, setup->btnup);
-    xitk_disable_widget(setup->btndn);
-    xitk_hide_widget(setup->widget_list, setup->btndn);
+     (setup->slider_wg = xitk_noskin_slider_create(setup->widget_list, &sl,
+						   (WINDOW_WIDTH - 41), 70, 
+						   16, (WINDOW_HEIGHT - 140), XITK_VSLIDER)));
+    xitk_slider_set_pos(setup->widget_list, setup->slider_wg, 1);
   }
 
   setup_sections();
@@ -1186,6 +1148,30 @@ void setup_panel(void) {
 
   XITK_WIDGET_INIT(&lb, gGui->imlib_data);
 
+  x = (WINDOW_WIDTH - (3 * 100))>>2;
+
+  lb.button_type       = CLICK_BUTTON;
+  lb.label             = _("Logs");
+  lb.align             = LABEL_ALIGN_CENTER;
+  lb.callback          = gui_viewlog_show; 
+  lb.state_callback    = NULL;
+  lb.userdata          = NULL;
+  lb.skin_element_name = NULL;
+  xitk_list_append_content(setup->widget_list->l, 
+	   xitk_noskin_labelbutton_create(setup->widget_list, &lb, x, WINDOW_HEIGHT - 40, 100, 23,
+					  "Black", "Black", "White", tabsfontname));
+
+  lb.button_type       = CLICK_BUTTON;
+  lb.label             = _("Key Bindings");
+  lb.align             = LABEL_ALIGN_CENTER;
+  lb.callback          = gui_kbedit_show; 
+  lb.state_callback    = NULL;
+  lb.userdata          = NULL;
+  lb.skin_element_name = NULL;
+  xitk_list_append_content(setup->widget_list->l, 
+   xitk_noskin_labelbutton_create(setup->widget_list, &lb, (x * 2) + 100, WINDOW_HEIGHT - 40, 100, 23,
+					  "Black", "Black", "White", tabsfontname));
+
   lb.button_type       = CLICK_BUTTON;
   lb.label             = _("Close");
   lb.align             = LABEL_ALIGN_CENTER;
@@ -1194,10 +1180,8 @@ void setup_panel(void) {
   lb.userdata          = NULL;
   lb.skin_element_name = NULL;
   xitk_list_append_content(setup->widget_list->l, 
-	   xitk_noskin_labelbutton_create(&lb,
-					  (WINDOW_WIDTH>>1) - 50, WINDOW_HEIGHT - 40,
-					  100, 23,
-					  "Black", "Black", "White", fontname));
+   xitk_noskin_labelbutton_create(setup->widget_list, &lb, (x * 3) + (100 * 2), WINDOW_HEIGHT - 40, 100, 23,
+					  "Black", "Black", "White", tabsfontname));
   
   XMapRaised(gGui->display, xitk_window_get_window(setup->xwin));
 
