@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: skins.c,v 1.16 2002/04/27 13:26:39 f1rmb Exp $
+ * $Id: skins.c,v 1.24 2002/08/03 20:58:58 siggi Exp $
  *
  */
 #ifdef HAVE_CONFIG_H
@@ -35,24 +35,26 @@
 #include "event.h"
 #include "skins.h"
 #include "errors.h"
-
-#include "xitk.h"
+#include "i18n.h"
 
 #include <xine.h>
 #include <xine/xineutils.h>
 
+#include "xitk.h"
+
 /*
 #define SKIN_DEBUG 1
 */
 
 #define DEFAULT_SKIN        "xinetic"
-#define SKIN_IFACE_VERSION  1
+#define SKIN_IFACE_VERSION  2
 
 extern gGui_t             *gGui;
 
 static skins_locations_t **skins_avail = NULL;
 static int                 skins_avail_num = 0;
 static int                 change_config_entry;
+static char               **skin_names;
 
 /*
  * Fill **skins_avail with available skins from path.
@@ -65,8 +67,9 @@ static void get_available_skins_from(char *path) {
     return;
 
   if((pdir = opendir(path)) != NULL) {
-    struct stat   pstat;
+    struct stat   pstat, sstat;
     char          fullfilename[XITK_PATH_MAX + XITK_NAME_MAX + 1];
+    char          skcfgname[XITK_PATH_MAX + XITK_NAME_MAX + 1];
     
     while((pdirent = readdir(pdir)) != NULL) {
       memset(&fullfilename, 0, sizeof(fullfilename));
@@ -78,13 +81,27 @@ static void get_available_skins_from(char *path) {
 	 && (!(strlen(pdirent->d_name) == 1 && pdirent->d_name[0] == '.' )
 	     && !(strlen(pdirent->d_name) == 2 
 		  && (pdirent->d_name[0] == '.' && pdirent->d_name[1] == '.')))) {
-	skins_avail = (skins_locations_t **) 
-	  realloc(skins_avail, (skins_avail_num + 2) * sizeof(skins_locations_t*));
-	skins_avail[skins_avail_num] = (skins_locations_t *) xine_xmalloc(sizeof(skins_locations_t));
-	
-	skins_avail[skins_avail_num]->pathname = strdup(path);
-	skins_avail[skins_avail_num]->skin = strdup(pdirent->d_name);
-	skins_avail_num++;
+
+	/*
+	 * Check if a skinconfig file exist
+	 */
+	memset(&skcfgname, 0, sizeof(skcfgname));
+	sprintf(skcfgname, "%s/%s", fullfilename, "skinconfig");
+	if(((stat(skcfgname, &sstat)) > -1) && (S_ISREG(sstat.st_mode))) {
+	  
+	  skins_avail = (skins_locations_t **) realloc(skins_avail, 
+						       (skins_avail_num + 2) * sizeof(skins_locations_t*));
+	  skins_avail[skins_avail_num] = (skins_locations_t *) xine_xmalloc(sizeof(skins_locations_t));
+	  
+	  skins_avail[skins_avail_num]->pathname = strdup(path);
+	  skins_avail[skins_avail_num]->skin = strdup(pdirent->d_name);
+	  skins_avail_num++;
+	  
+	}
+	else {
+	  fprintf(stderr, "skinconfig file in '%s' is missing: skin '%s' skiped.\n",
+		  fullfilename, pdirent->d_name);
+	}
       }
     }
     closedir(pdir);
@@ -121,7 +138,7 @@ char *skin_get_skindir(void) {
   entry = gGui->config->lookup_entry(gGui->config, "gui.skin");
 
   if(entry)
-    skin = entry->str_value;
+    skin = (char *) skins_avail[entry->num_value]->skin;
   else
     skin = DEFAULT_SKIN;
   
@@ -143,7 +160,7 @@ char *skin_get_configfile(void) {
   entry = gGui->config->lookup_entry(gGui->config, "gui.skin");
 
   if(entry)
-    skin = entry->str_value;
+    skin = (char *) skins_avail[entry->num_value]->skin;
   else
     skin = DEFAULT_SKIN;
   
@@ -167,6 +184,20 @@ int get_available_skins_num(void) {
   return skins_avail_num;
 }
 
+/*
+ *
+ */
+int get_skin_offset(char *skin) {
+  int i = 0;
+  
+  while(skins_avail[i] != NULL) {
+    if(!strcasecmp(skin, skins_avail[i]->skin))
+      return i;
+    i++;
+  }
+  return -1;
+}
+
 /*
  * Try to find desired skin in skins_avail, then return
  * a pointer to the right entry.
@@ -185,12 +216,9 @@ skins_locations_t *get_skin_location(char *skin) {
 /*
  * Select a new skin (used in control skin list).
  */
-void select_new_skin(skins_locations_t *sk) {
+void select_new_skin(int selected) {
   
-  if(!sk)
-    return;
-  
-  gGui->config->update_string(gGui->config, "gui.skin", (char *)sk->skin);
+  gGui->config->update_num(gGui->config, "gui.skin", selected);
 }
 
 /*
@@ -202,50 +230,61 @@ void change_skin(skins_locations_t *sk) {
   char                *old_skin;
   skins_locations_t   *sks = sk;
   cfg_entry_t         *entry;
-  int                  twice = 0;
+  int                  twice = 0, twice_load = 0;
 
   if(!sk)
     return;
   
   entry = gGui->config->lookup_entry(gGui->config, "gui.skin");
   if(entry)
-    old_skin = entry->str_value;
+    old_skin = (char *) skins_avail[entry->num_value]->skin;
   else
     old_skin = DEFAULT_SKIN;
   
   xitk_skin_unload_config(gGui->skin_config);
-  //  skonfig = gGui->skin_config;
 
   gGui->skin_config = xitk_skin_init_config();
 
-  if(change_config_entry) {
-    gGui->config->update_string(gGui->config, "gui.skin", (char *)sk->skin);
-  }
+  if(change_config_entry)
+    gGui->config->update_num(gGui->config, "gui.skin", (get_skin_offset((char * )sk->skin)));
 
  __reload_skin:
   memset(&buf, 0, sizeof(buf));
   sprintf(buf, "%s/%s", sks->pathname, sks->skin);
-  
+
   if(!xitk_skin_load_config(gGui->skin_config, buf, "skinconfig")) {
-    xine_error(_("Failed to load %s/%s. Reload old skin '%s'.\n"), buf, "skinconfig", old_skin);
-    gGui->config->update_string(gGui->config, "gui.skin", old_skin);
-    sks = get_skin_location(old_skin);
-    if(!twice) {
-      twice++;
+    skins_locations_t   *osks;
+
+    gGui->config->update_num(gGui->config, "gui.skin", (get_skin_offset(old_skin)));
+    osks = get_skin_location(old_skin);
+    if((!strcmp(osks->pathname, sks->pathname)) && (!strcmp(osks->skin, sks->skin))) {
+      xine_error(_("Failed to load %s/%s. Load fallback skin %s\n"), 
+		 buf, "skinconfig", DEFAULT_SKIN);
+      gGui->config->update_num(gGui->config, "gui.skin", (get_skin_offset(DEFAULT_SKIN)));
+      sks = get_skin_location(DEFAULT_SKIN);
+    }
+    else {
+      xine_error(_("Failed to load %s/%s. Reload old skin '%s'.\n"), buf, "skinconfig", old_skin);
+      sks = osks;
+    }
+    
+    if(!twice_load) {
+      twice_load++;
       goto __reload_skin;
     }
     else {
       exit(-1);
     }
   }
-  
+  twice_load = 0;
+
   /* Check skin version */
   if(xitk_skin_check_version(gGui->skin_config, SKIN_IFACE_VERSION) < 1) {
     xitk_skin_unload_config(gGui->skin_config);
     gGui->skin_config = xitk_skin_init_config();
     xine_error(_("Failed to load %s, wrong version. Load fallback skin '%s'.\n"), 
 	       buf, DEFAULT_SKIN);
-    gGui->config->update_string(gGui->config, "gui.skin", DEFAULT_SKIN);
+    gGui->config->update_num(gGui->config, "gui.skin", (get_skin_offset(DEFAULT_SKIN)));
     sks = get_skin_location(DEFAULT_SKIN);
     twice++;
     goto __reload_skin;
@@ -278,17 +317,16 @@ void change_skin(skins_locations_t *sk) {
  */
 static void skin_change_cb(void *data, cfg_entry_t *cfg) {
   skins_locations_t   *sk;
-  char                *new_skin = cfg->str_value;
   
   if(skins_avail == NULL || change_config_entry == 0)
     return;
   
   /* First, try to see if the skin exist somewhere */
-  sk = get_skin_location(new_skin);
+  sk = skins_avail[cfg->num_value];
   if(!sk) {
-    xine_error(_("Ooch, skin '%s' not found, use fallback '%s'.\n"), new_skin, DEFAULT_SKIN);
+    xine_error(_("Ooch, skin not found, use fallback '%s'.\n"), DEFAULT_SKIN);
     sk = get_skin_location(DEFAULT_SKIN);
-    gGui->config->update_string(gGui->config, "gui.skin", DEFAULT_SKIN);
+    gGui->config->update_num(gGui->config, "gui.skin", (get_skin_offset(DEFAULT_SKIN)));
   }
 
   change_config_entry = 0;
@@ -302,9 +340,9 @@ static void skin_change_cb(void *data, cfg_entry_t *cfg) {
  */
 void init_skins_support(void) {
   skins_locations_t   *sk;
-  char                *skin;
-  char                buf[XITK_PATH_MAX + XITK_NAME_MAX + 1];
-  int                 twice = 0;
+  char                 buf[XITK_PATH_MAX + XITK_NAME_MAX + 1];
+  int                  twice = 0, twice_load = 0;
+  int                  skin_num, i;
     
   change_config_entry = 0;
 
@@ -312,21 +350,26 @@ void init_skins_support(void) {
   
   looking_for_available_skins();
   
-  if(skins_avail == NULL) {
+  if((skins_avail == NULL) || (skins_avail_num == 0)) {
     fprintf(stderr, _("No available skin found. Say goodbye.\n"));
     exit(-1);
   }
   
-  skin = gGui->config->register_string (gGui->config, "gui.skin", DEFAULT_SKIN,
-					"gui skin theme", 
-					NULL, skin_change_cb, NULL);
-  sk = get_skin_location(skin);
+  skin_names = (char **) xine_xmalloc(sizeof(char *) * (skins_avail_num + 1));
+  for(i = 0; i < skins_avail_num; i++)
+    skin_names[i] = strdup(skins_avail[i]->skin);
+  skin_names[skins_avail_num] = NULL;
+
+  skin_num = gGui->config->register_enum (gGui->config, "gui.skin", 
+					  (get_skin_offset(DEFAULT_SKIN)), skin_names,
+					  _("gui skin theme"), NULL, skin_change_cb, NULL);
   
-  memset(&buf, 0, sizeof(buf));
+  sk = (skin_num < skins_avail_num) ? skins_avail[skin_num] : NULL;
   
   if(!sk) {
-    xine_error(_("Ooch, skin '%s' not found, use fallback '%s'.\n"), skin, DEFAULT_SKIN);
-    gGui->config->update_string(gGui->config, "gui.skin", DEFAULT_SKIN);
+    xine_error(_("Ooch, skin '%s' not found, use fallback '%s'.\n"), skins_avail[skin_num], 
+	       (skins_avail[(get_skin_offset(DEFAULT_SKIN))]->skin));
+    gGui->config->update_num(gGui->config, "gui.skin", (get_skin_offset(DEFAULT_SKIN)));
     sk = get_skin_location(DEFAULT_SKIN);
     if(!sk) {
       fprintf(stderr, _("Failed to load fallback skin. Check your installed skins. Exiting.\n"));
@@ -334,22 +377,33 @@ void init_skins_support(void) {
     }
   } 
   
-  sprintf(buf, "%s/%s", sk->pathname, sk->skin);
-  
  __reload_skin:
+  memset(&buf, 0, XITK_PATH_MAX + XITK_NAME_MAX);
+  sprintf(buf, "%s/%s", sk->pathname, sk->skin);
+
   if(!xitk_skin_load_config(gGui->skin_config, buf, "skinconfig")) {
-    fprintf(stderr, _("Failed to load %s/%s. Exiting.\n"), buf, "skinconfig");
-    xitk_skin_free_config(gGui->skin_config);
-    exit(-1);
+    if(!twice_load) {
+      twice_load++;
+      fprintf(stderr, _("Failed to load %s/%s. Load fallback skin %s\n"), 
+	      buf, "skinconfig", DEFAULT_SKIN);
+      gGui->config->update_num(gGui->config, "gui.skin", (get_skin_offset(DEFAULT_SKIN)));
+      sk = get_skin_location(DEFAULT_SKIN);
+      goto __reload_skin;
+    }
+    else {
+      fprintf(stderr, _("Failed to load %s/%s. Exiting.\n"), buf, "skinconfig");
+      exit(-1);
+    }
   }
+  twice_load = 0;
 
   /* Check skin version */
   if(xitk_skin_check_version(gGui->skin_config, SKIN_IFACE_VERSION) < 1) {
     xitk_skin_unload_config(gGui->skin_config);
     gGui->skin_config = xitk_skin_init_config();
     fprintf(stderr, _("Failed to load %s skin, wrong version. Load fallback skin '%s'.\n"), 
-	    buf, DEFAULT_SKIN);
-    gGui->config->update_string(gGui->config, "gui.skin", DEFAULT_SKIN);
+	    buf, (skins_avail[(get_skin_offset(DEFAULT_SKIN))]->skin));
+    gGui->config->update_num(gGui->config, "gui.skin", (get_skin_offset(DEFAULT_SKIN)));
     sk = get_skin_location(DEFAULT_SKIN);
     sprintf(buf, "%s/%s", sk->pathname, sk->skin);
     if(!twice) {
