@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: skins.h,v 1.3 2002/03/21 00:22:52 f1rmb Exp $
+ * $Id: skins.h,v 1.4 2002/05/05 09:46:09 f1rmb Exp $
  *
  */
 
@@ -34,7 +34,7 @@ char *gui_get_configfile(void);
 skins_locations_t **get_available_skins(void);
 int get_available_skins_num(void);
 void init_skins_support(void);
-void select_new_skin(skins_locations_t *sk);
+void select_new_skin(int selected);
 void change_skin(skins_locations_t *sk);
 
 #endif
