@@ -15,7 +15,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: snapshot.c,v 1.15 2002/04/27 13:26:40 f1rmb Exp $
+ * $Id: snapshot.c,v 1.17 2002/06/10 23:27:56 f1rmb Exp $
  *
  * Image snapshot related stuff
  *
@@ -67,9 +67,11 @@
 #include <xine.h>
 #include <xine/xineutils.h>
 
-#include "xitk.h"
 #include "event.h"
 #include "errors.h"
+#include "i18n.h"
+
+#include "xitk.h"
 
 #define PIXSZ 3
 #define BIT_DEPTH 8
@@ -811,12 +813,12 @@ static void yv12_2_rgb( struct prvt_image_s *image )
 
 static void user_error_fn(png_structp png_ptr, png_const_charp error_msg)
 {
-  xine_error("Error: %s\n", error_msg);
+  xine_error(_("Error: %s\n"), error_msg);
 }
 
 static void user_warning_fn(png_structp png_ptr, png_const_charp warning_msg)
 {
-  xine_error("Warning: %s\n", warning_msg);
+  xine_error(_("Warning: %s\n"), warning_msg);
 }
 
 /*
@@ -866,7 +868,7 @@ void create_snapshot (void)
 
   /* the dxr3 driver does not allocate yuv buffers */
   if (! image->y) { /* image->u and image->v are always 0 for YUY2 */
-    printf("snapshot.c: not supported for this video out driver\n");
+    printf(_("snapshot.c: not supported for this video out driver\n"));
     prvt_image_free( &image );
     return;
   }
