@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2001 the xine project
+ * Copyright (C) 2000-2002 the xine project
  * 
  * This file is part of xine, a unix video player.
  * 
@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: snapshot.h,v 1.2 2001/10/21 00:28:54 f1rmb Exp $
+ * $Id: snapshot.h,v 1.4 2002/04/27 13:26:41 f1rmb Exp $
  *
  * Image snapshot related stuff
  *
@@ -25,6 +25,6 @@
 #ifndef SNAPSHOT_H
 #define SNAPSHOT_H
 
-void create_snapshot ( gGui_t *gGui );
+void create_snapshot (void);
 
 #endif
