@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: videowin.h,v 1.20 2002/04/27 13:26:44 f1rmb Exp $
+ * $Id: videowin.h,v 1.24 2002/07/13 10:32:32 f1rmb Exp $
  *
  * video window handling functions
  */
@@ -25,7 +25,17 @@
 #ifndef VIDEOWIN_H
 #define VIDEOWIN_H
 
-void video_window_init (void);
+typedef struct {
+  int   x;
+  int   y;
+  int   width;
+  int   height;
+  int   borderless;
+} window_attributes_t;
+
+void video_window_init (window_attributes_t *window_attribute, int hide_on_start);
+
+void video_window_exit (void);
 
 void video_window_select_visual (void);
 
@@ -40,8 +50,8 @@ void video_window_dest_size_cb (void *this,
 				int *dest_width, int *dest_height);
 
 /* set/check fullscreen mode */
-void video_window_set_fullscreen (int req_fullscreen);
-int video_window_is_fullscreen (void);
+void video_window_set_fullscreen_mode (int req_fullscreen);
+int video_window_get_fullscreen_mode (void);
 
 /* hide/show cursor in video window*/
 void video_window_set_cursor_visibility(int show_cursor);
@@ -57,4 +67,6 @@ float video_window_get_mag (void);
 
 void video_window_change_skins(void);
 
+void video_window_reset_ssaver(void);
+
 #endif
