@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: viewlog.c,v 1.14 2002/04/12 21:38:51 f1rmb Exp $
+ * $Id: viewlog.c,v 1.23 2002/07/17 20:29:30 f1rmb Exp $
  *
  *
  */
@@ -36,13 +36,14 @@
 #include <xine.h>
 #include <xine/xineutils.h>
 
-#include "xitk.h"
-
 #include "Imlib-light/Imlib.h"
 #include "event.h"
 #include "actions.h"
 #include "skins.h"
 #include "lang.h"
+#include "i18n.h"
+
+#include "xitk.h"
 
 /*
 #define DEBUG_VIEWLOG
@@ -50,8 +51,8 @@
 
 extern gGui_t              *gGui;
 
-//static char                *br_fontname = "-*-helvetica-medium-r-*-*-10-*-*-*-*-*-*-*";
 static char                *br_fontname = "-misc-fixed-medium-r-normal-*-10-*-*-*-*-*-*-*";
+static char                *tabsfontname = "-*-helvetica-bold-r-*-*-12-*-*-*-*-*-*-*";
 
 #define WINDOW_WIDTH        580
 #define WINDOW_HEIGHT       480
@@ -68,6 +69,7 @@ typedef struct {
 
   char                **log;
   int                   log_entries;
+  int                   real_num_entries;
   
   xitk_widget_t        *browser_widget;
 
@@ -198,42 +200,12 @@ void viewlog_toggle_visibility (xitk_widget_t *w, void *data) {
 /*
  * Handle X events here.
  */
-void viewlog_handle_event(XEvent *event, void *data) {
+static void viewlog_handle_event(XEvent *event, void *data) {
 
+  /*
   switch(event->type) {
-
-  case KeyPress: {
-    XKeyEvent      mykeyevent;
-    KeySym         mykey;
-    char           kbuf[256];
-    int            len;
-    
-    mykeyevent = event->xkey;
-    
-    XLockDisplay(gGui->display);
-    len = XLookupString(&mykeyevent, kbuf, sizeof(kbuf), &mykey, NULL);
-    XUnlockDisplay(gGui->display);
-    
-    switch (mykey) {
-      
-    case XK_Return:
-      viewlog_end(NULL, NULL);
-      break;
-      
-    default:
-      gui_handle_event(event, data);
-      break;
-    }
-  }
-  break;
-  
-  case MappingNotify:
-    XLockDisplay(gGui->display);
-    XRefreshKeyboardMapping((XMappingEvent *) event);
-    XUnlockDisplay(gGui->display);
-    break;
-    
   }
+  */
 }
 
 /*
@@ -278,7 +250,7 @@ static void viewlog_change_section(xitk_widget_t *wx, void *data, int section) {
   }
   
   /* Compute log entries */
-  viewlog->log_entries = k = 0;
+  viewlog->log_entries = viewlog->real_num_entries = k = 0;
   
   if(log) {
     
@@ -311,13 +283,14 @@ static void viewlog_change_section(xitk_widget_t *wx, void *data, int section) {
 	    if(strlen(buf)) {
 	      viewlog->log = (char **) realloc(viewlog->log, sizeof(char **) * ((j + 1) + 1));
 	      viewlog->log[j++] = strdup(buf);
-	      //	      printf("added line '%s'\n", viewlog->log[j-1]);
+	      viewlog->real_num_entries++;
+	      // printf("added line '%s'\n", viewlog->log[j-1]);
 	    }
 	    memset(&buf, 0, sizeof(buf));
 	    break;
 	    
 	  default:
-	    //	    printf("- %c", *p);
+	    // printf("- %c", *p);
 	    sprintf(buf, "%s%c", buf, *p);
 	    break;
 	  }
@@ -358,7 +331,7 @@ static void viewlog_change_section(xitk_widget_t *wx, void *data, int section) {
 			  xitk_tabs_get_current_tab_selected(viewlog->tabs));
 #endif
   
-  xitk_browser_update_list(viewlog->browser_widget, viewlog->log, viewlog->log_entries, 0);
+  xitk_browser_update_list(viewlog->browser_widget, viewlog->log, viewlog->real_num_entries, 0);
   
   viewlog_clear_tab();
   viewlog_paint_widgets();
@@ -399,8 +372,8 @@ static void viewlog_create_tabs(void) {
   tab.callback          = viewlog_change_section;
   tab.userdata          = NULL;
   xitk_list_append_content (viewlog->widget_list->l,
-			    (viewlog->tabs = 
-			     xitk_noskin_tabs_create(&tab, 20, 25, WINDOW_WIDTH - 40)));
+    (viewlog->tabs = 
+     xitk_noskin_tabs_create(viewlog->widget_list, &tab, 20, 24, WINDOW_WIDTH - 40, tabsfontname)));
 
   bg = xitk_image_create_pixmap(gGui->imlib_data, WINDOW_WIDTH, WINDOW_HEIGHT);
   
@@ -416,7 +389,7 @@ static void viewlog_create_tabs(void) {
   XLockDisplay(gGui->display);
   XFreePixmap(gGui->display, bg);
   XUnlockDisplay(gGui->display);
-
+  
   viewlog_change_section(NULL, NULL, 0);
 }
 
@@ -435,7 +408,6 @@ void viewlog_window(void) {
   GC                         gc;
   xitk_labelbutton_widget_t  lb;
   xitk_browser_widget_t      br;
-  char                      *fontname = "*-lucida-*-r-*-*-10-*-*-*-*-*-*-*";
   int                        x, y;
   
   /* this shouldn't happen */
@@ -462,8 +434,6 @@ void viewlog_window(void) {
 
   viewlog->widget_list                = xitk_widget_list_new();
   viewlog->widget_list->l             = xitk_list_new ();
-  viewlog->widget_list->focusedWidget = NULL;
-  viewlog->widget_list->pressedWidget = NULL;
   viewlog->widget_list->win           = (xitk_window_get_window(viewlog->xwin));
   viewlog->widget_list->gc            = gc;
 
@@ -481,24 +451,23 @@ void viewlog_window(void) {
   br.browser.entries               = viewlog->log;
   br.callback                      = NULL;
   br.dbl_click_callback            = NULL;
-  br.dbl_click_time                = DEFAULT_DBL_CLICK_TIME;
   br.parent_wlist                  = viewlog->widget_list;
   br.userdata                      = NULL;
   xitk_list_append_content(viewlog->widget_list->l, 
 			   (viewlog->browser_widget = 
-			    xitk_noskin_browser_create(&br,
+			    xitk_noskin_browser_create(viewlog->widget_list, &br,
 						       viewlog->widget_list->gc, 25, 58, 
 						       WINDOW_WIDTH - (50 + 16), 20,
 						       16, br_fontname)));
 
   xitk_browser_set_alignment(viewlog->browser_widget, LABEL_ALIGN_LEFT);
-  xitk_browser_update_list(viewlog->browser_widget, viewlog->log, viewlog->log_entries, 0);
+  xitk_browser_update_list(viewlog->browser_widget, viewlog->log, viewlog->real_num_entries, 0);
   
   viewlog_paint_widgets();
   
   XITK_WIDGET_INIT(&lb, gGui->imlib_data);
 
-  x = (WINDOW_WIDTH - (100 * 2)) / 3;
+  x = ((WINDOW_WIDTH / 2) - 100) / 2;
   y = WINDOW_HEIGHT - 40;
   
   lb.button_type       = CLICK_BUTTON;
@@ -509,11 +478,11 @@ void viewlog_window(void) {
   lb.userdata          = NULL;
   lb.skin_element_name = NULL;
   xitk_list_append_content(viewlog->widget_list->l, 
-	   xitk_noskin_labelbutton_create(&lb,
+	   xitk_noskin_labelbutton_create(viewlog->widget_list, &lb,
 					  x, y, 100, 23,
-					  "Black", "Black", "White", fontname));
+					  "Black", "Black", "White", tabsfontname));
 
-  x += x * 2;
+  x += (WINDOW_WIDTH / 2);
 
   lb.button_type       = CLICK_BUTTON;
   lb.label             = _("Close");
@@ -523,9 +492,9 @@ void viewlog_window(void) {
   lb.userdata          = NULL;
   lb.skin_element_name = NULL;
   xitk_list_append_content(viewlog->widget_list->l, 
-	   xitk_noskin_labelbutton_create(&lb,
+	   xitk_noskin_labelbutton_create(viewlog->widget_list, &lb,
 					  x, y, 100, 23,
-					  "Black", "Black", "White", fontname));
+					  "Black", "Black", "White", tabsfontname));
 
   
   XUnlockDisplay (gGui->display);
