@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: _config.h,v 1.3 2002/04/21 22:17:12 f1rmb Exp $
+ * $Id: _config.h,v 1.5 2002/05/03 12:06:17 f1rmb Exp $
  *
  *
  */
@@ -44,10 +44,13 @@ typedef struct {
     int             background;
     int             focus;
     int             select;
+    int             warn_foreground;
+    int             warn_background;
   } colors;
 
   struct {
     unsigned long   label_anim;
+    long int        dbl_click;
   } timers;
 
 } xitk_config_t;
@@ -63,5 +66,8 @@ int xitk_config_get_background_color(xitk_config_t *xtcf);
 int xitk_config_get_focus_color(xitk_config_t *xtcf);
 int xitk_config_get_select_color(xitk_config_t *xtcf);
 unsigned long xitk_config_get_timer_label_animation(xitk_config_t *xtcf);
+long xitk_config_get_timer_dbl_click(xitk_config_t *xtcf);
+unsigned long xitk_config_get_warning_foreground(xitk_config_t *xtcf);
+unsigned long xitk_config_get_warning_background(xitk_config_t *xtcf);
 
 #endif
