@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: _xitk.h,v 1.35 2002/04/21 22:17:12 f1rmb Exp $
+ * $Id: _xitk.h,v 1.42 2002/06/19 00:33:55 f1rmb Exp $
  *
  *
  */
@@ -274,6 +274,8 @@ typedef struct {
   GC                        gc;
   char                     *label;
   char                     *skin_element_name;
+  xitk_simple_callback_t    callback;
+  void                     *userdata;
 } xitk_label_widget_t;
 
 #define XITK_VSLIDER 1
@@ -316,7 +318,6 @@ typedef struct {
     char                  **entries;
   } browser;
   
-  int                       dbl_click_time;
   xitk_state_callback_t     dbl_click_callback;
 
   /* Callback on selection function */
@@ -436,6 +437,10 @@ typedef struct {
   xine_t                   *xine;
 
   xitk_browser_widget_t     browser;
+  
+  struct {
+    char                    *skin_element_name;
+  } combo;
 
 } xitk_mrlbrowser_widget_t;
 #endif
@@ -480,7 +485,15 @@ int xitk_get_white_color(void);
 int xitk_get_background_color(void);
 int xitk_get_focus_color(void);
 int xitk_get_select_color(void);
+/*
+ * copy src to dest and substitute special chars. dest should have 
+ * enought space to store chars.
+ */
+void xitk_subst_special_chars(char *, char *);
 unsigned long xitk_get_timer_label_animation(void);
+long int xitk_get_timer_dbl_click(void);
+unsigned long xitk_get_warning_foreground(void);
+unsigned long xitk_get_warning_background(void);
 
 int xitk_skin_get_visibility(xitk_skin_config_t *, const char *);
 int xitk_skin_get_enability(xitk_skin_config_t *, const char *);
@@ -500,6 +513,8 @@ char *xitk_skin_get_skin_filename(xitk_skin_config_t *, const char *);
 char *xitk_skin_get_slider_skin_filename(xitk_skin_config_t *, const char *);
 int xitk_skin_get_slider_type(xitk_skin_config_t *, const char *);
 int xitk_skin_get_slider_radius(xitk_skin_config_t *, const char *);
+char *xitk_skin_get_logo(xitk_skin_config_t *);
+int xitk_skin_get_browser_entries(xitk_skin_config_t *, const char *);
 void xitk_skin_lock(xitk_skin_config_t *);
 void xitk_skin_unlock(xitk_skin_config_t *);
 
