@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: browser.c,v 1.25 2002/04/18 21:14:35 f1rmb Exp $
+ * $Id: browser.c,v 1.37 2002/07/17 20:29:31 f1rmb Exp $
  *
  */
 #ifdef HAVE_CONFIG_H
@@ -37,6 +37,7 @@
 #include "button.h"
 #include "labelbutton.h"
 #include "slider.h"
+#include "inputtext.h"
 
 #include "_xitk.h"
 
@@ -51,15 +52,20 @@ typedef struct {
 #define WBDN    2  /*  Position of button down in item_tree  */
 #define WBSTART 3  /*  Position of first item button in item_tree */
 
+#define EXTRA_BTNS 50
+
 static void notify_change_skin(xitk_widget_list_t *, xitk_widget_t *, xitk_skin_config_t *);
 /*
  *
  */
 static void paint(xitk_widget_t *w, Window win, GC gc) {
-  browser_private_data_t *private_data = (browser_private_data_t *) w->private_data;
+  browser_private_data_t *private_data;
   
-  if(w->widget_type & WIDGET_TYPE_BROWSER) {
+  if(w && (((w->widget_type & WIDGET_GROUP_MASK) & WIDGET_GROUP_BROWSER) &&
+	   (w->widget_type & WIDGET_GROUP_WIDGET))) {
     int i;
+    
+    private_data = (browser_private_data_t *) w->private_data;
 
     if((w->visible == 1)) {
       if(!private_data->skin_element_name) {
@@ -80,7 +86,7 @@ static void paint(xitk_widget_t *w, Window win, GC gc) {
 	xitk_show_widget(private_data->parent_wlist, private_data->item_tree[WSLID]);
 	xitk_show_widget(private_data->parent_wlist, private_data->item_tree[WBDN]);
 	
-	for(i = WBSTART; i < private_data->max_length+WBSTART; i++) {
+	for(i = WBSTART; i < (private_data->max_length+WBSTART); i++) {
 	  (void) xitk_set_widget_pos(private_data->item_tree[i], x, iy);
 	  xitk_show_widget(private_data->parent_wlist, private_data->item_tree[i]);
 	  iy += itemh;
@@ -88,7 +94,7 @@ static void paint(xitk_widget_t *w, Window win, GC gc) {
       }
     }
     else {
-      for(i = 0; i < private_data->max_length+WBSTART; i++) {
+      for(i = 0; i < (private_data->max_length+WBSTART); i++) {
 	xitk_hide_widget(private_data->parent_wlist, private_data->item_tree[i]);
       }
     }
@@ -100,9 +106,13 @@ static void paint(xitk_widget_t *w, Window win, GC gc) {
  */
 static void notify_change_skin(xitk_widget_list_t *wl, 
 			       xitk_widget_t *w, xitk_skin_config_t *skonfig) {
-  browser_private_data_t *private_data = (browser_private_data_t *) w->private_data;
+  browser_private_data_t *private_data;
   
-  if(w->widget_type & WIDGET_TYPE_BROWSER) {
+  if(w && (((w->widget_type & WIDGET_GROUP_MASK) & WIDGET_GROUP_BROWSER) &&
+	   (w->widget_type & WIDGET_GROUP_WIDGET))) {
+    
+    private_data = (browser_private_data_t *) w->private_data;
+    
     xitk_skin_lock(skonfig);
     
     if(private_data->skin_element_name) {
@@ -112,27 +122,52 @@ static void notify_change_skin(xitk_widget_list_t *wl,
       y          = xitk_skin_get_coord_y(skonfig, private_data->skin_element_name);
       w->visible = (xitk_skin_get_visibility(skonfig, private_data->skin_element_name)) ? 1 : -1;
       w->enable  = xitk_skin_get_enability(skonfig, private_data->skin_element_name);
+
+      private_data->max_length = xitk_skin_get_browser_entries(skonfig, 
+							       private_data->skin_element_name);
       
-      for(i = WBSTART; i < private_data->max_length + WBSTART; i++) {
-	
-	(void) xitk_set_widget_pos(private_data->item_tree[i], x, y);
-	
-	y += xitk_get_widget_height(private_data->item_tree[i]) + 1;
-	
+      for(i = WBSTART; i < (EXTRA_BTNS + WBSTART); i++) {
+	xitk_disable_widget(private_data->item_tree[i]);
+	xitk_hide_widget(private_data->parent_wlist, private_data->item_tree[i]);
       }
+
+      for(i = WBSTART; i < private_data->max_length + WBSTART; i++) {
+	(void) xitk_set_widget_pos(private_data->item_tree[i], x, y);
+	y += xitk_get_widget_height(private_data->item_tree[i]) + 1;
+	xitk_enable_widget(private_data->item_tree[i]);
+	xitk_show_widget(private_data->parent_wlist, private_data->item_tree[i]);
+      }
+
+      xitk_browser_rebuild_browser(w, 0);
     }
     xitk_skin_unlock(skonfig);
   }
 }
 
+/**
+ * Return the number of displayed entries
+ */
+int xitk_browser_get_num_entries(xitk_widget_t *w) {
+  browser_private_data_t *private_data;
+
+  if(w && (((w->widget_type & WIDGET_GROUP_MASK) & WIDGET_GROUP_BROWSER) &&
+	   (w->widget_type & WIDGET_GROUP_WIDGET))) {
+    private_data = (browser_private_data_t *) w->private_data;
+    return private_data->max_length;
+  }
+
+  return -1;
+}
+
 /**
  * Return the real number of first displayed in list
  */
 int xitk_browser_get_current_start(xitk_widget_t *w) {
-  browser_private_data_t *private_data = 
-    (browser_private_data_t *) w->private_data;
+  browser_private_data_t *private_data;
 
-  if(w->widget_type & WIDGET_TYPE_BROWSER) {
+  if(w && (((w->widget_type & WIDGET_GROUP_MASK) & WIDGET_GROUP_BROWSER) &&
+	   (w->widget_type & WIDGET_GROUP_WIDGET))) {
+    private_data = (browser_private_data_t *) w->private_data;
     return private_data->current_start;
   }
 
@@ -143,11 +178,14 @@ int xitk_browser_get_current_start(xitk_widget_t *w) {
  * Change browser labels alignment
  */
 void xitk_browser_set_alignment(xitk_widget_t *w, int align) {
-  browser_private_data_t *private_data = 
-    (browser_private_data_t *) w->private_data;
+  browser_private_data_t *private_data;
   int i;
   
-  if(w->widget_type & WIDGET_TYPE_BROWSER) {
+  if(w && (((w->widget_type & WIDGET_GROUP_MASK) & WIDGET_GROUP_BROWSER) &&
+	   (w->widget_type & WIDGET_GROUP_WIDGET))) {
+    
+    private_data = (browser_private_data_t *) w->private_data;
+    
     for(i = 0; i < private_data->max_length; i++) {
       xitk_labelbutton_set_alignment(private_data->item_tree[i+WBSTART], align);
     }
@@ -158,11 +196,14 @@ void xitk_browser_set_alignment(xitk_widget_t *w, int align) {
  * Return the current selected button (if not, return -1)
  */
 int xitk_browser_get_current_selected(xitk_widget_t *w) {
-  browser_private_data_t *private_data = 
-    (browser_private_data_t *) w->private_data;
-  int i;
+  browser_private_data_t *private_data;
+  int                     i;
   
-  if(w->widget_type & WIDGET_TYPE_BROWSER) {
+  if(w && (((w->widget_type & WIDGET_GROUP_MASK) & WIDGET_GROUP_BROWSER) &&
+	   (w->widget_type & WIDGET_GROUP_WIDGET))) {
+    
+    private_data = (browser_private_data_t *) w->private_data;
+    
     for(i = 0; i < private_data->max_length; i++) {
       if(xitk_labelbutton_get_state(private_data->item_tree[i+WBSTART]))
 	return (private_data->current_start + i);
@@ -176,11 +217,14 @@ int xitk_browser_get_current_selected(xitk_widget_t *w) {
  * Release all enabled buttons
  */
 void xitk_browser_release_all_buttons(xitk_widget_t *w) {
-  browser_private_data_t *private_data = 
-    (browser_private_data_t *) w->private_data;
-  int i;
+  browser_private_data_t *private_data;
+  int                     i;
   
-  if(w->widget_type & WIDGET_TYPE_BROWSER) {
+  if(w && (((w->widget_type & WIDGET_GROUP_MASK) & WIDGET_GROUP_BROWSER) &&
+	   (w->widget_type & WIDGET_GROUP_WIDGET))) {
+    
+    private_data = (browser_private_data_t *) w->private_data;
+    
     for(i = 0; i < private_data->max_length; i++) {
       xitk_labelbutton_set_state(private_data->item_tree[i+WBSTART], 0, 
 				 private_data->parent_wlist->win,
@@ -193,10 +237,13 @@ void xitk_browser_release_all_buttons(xitk_widget_t *w) {
  * Select the item 'select' in list
  */
 void xitk_browser_set_select(xitk_widget_t *w, int select) {
-  browser_private_data_t *private_data = 
-    (browser_private_data_t *) w->private_data;
+  browser_private_data_t *private_data;
   
-  if(w->widget_type & WIDGET_TYPE_BROWSER) {
+  if(w && (((w->widget_type & WIDGET_GROUP_MASK) & WIDGET_GROUP_BROWSER) &&
+	   (w->widget_type & WIDGET_GROUP_WIDGET))) {
+    
+    private_data = (browser_private_data_t *) w->private_data;
+    
     xitk_browser_release_all_buttons(w);
     
     xitk_labelbutton_set_state(private_data->item_tree[select+WBSTART], 1, 
@@ -208,39 +255,48 @@ void xitk_browser_set_select(xitk_widget_t *w, int select) {
  * Redraw buttons/slider
  */
 void xitk_browser_rebuild_browser(xitk_widget_t *w, int start) {
-  browser_private_data_t *private_data = 
-    (browser_private_data_t *) w->private_data;
-  xitk_widget_list_t* wl;
-  int i, j;
+  browser_private_data_t *private_data;
+  xitk_widget_list_t     *wl;
+  int                     i, j, max;
 
-  if(w->widget_type & WIDGET_TYPE_BROWSER) {
+  if(w && (((w->widget_type & WIDGET_GROUP_MASK) & WIDGET_GROUP_BROWSER) &&
+	   (w->widget_type & WIDGET_GROUP_WIDGET))) {
+    
+    private_data = (browser_private_data_t *) w->private_data;
+    
     xitk_browser_release_all_buttons(w);
-    
-    if(start >= 0)
-      private_data->current_start = start;
-    
+
     wl = (xitk_widget_list_t*) xitk_xmalloc(sizeof(xitk_widget_list_t));
     wl->win = private_data->parent_wlist->win;
     wl->gc = private_data->parent_wlist->gc;
     
+    j = (private_data->list_length > (private_data->max_length - 1) ? 
+	 (private_data->list_length - 1) : 1);
+
+    if(j <= (private_data->max_length - 1))
+      max = 0;
+    else if(j > (private_data->max_length - 1))
+      max = j - (private_data->max_length - 1);
+    
+    if((start >= 0) && max)
+      private_data->current_start = start;
+
     for(i = 0; i < private_data->max_length; i++) {
-      if (((private_data->current_start+i)<private_data->list_length) 
-	  && (private_data->content[private_data->current_start+i] != NULL)) {
-	xitk_labelbutton_change_label(wl, private_data->item_tree[i+WBSTART], 
-				      private_data->content[private_data->
-							   current_start+i]);
+      if (((private_data->current_start + i) < private_data->list_length) 
+	  && (private_data->content[private_data->current_start + i] != NULL)) {
+	xitk_labelbutton_change_label(wl, private_data->item_tree[i + WBSTART], 
+				      private_data->content[private_data->current_start + i]);
       }
       else {
 	xitk_labelbutton_change_label(wl, private_data->item_tree[i+WBSTART], "");
       }
     }
-    j = (private_data->list_length > (private_data->max_length-1) ? 
-	 (private_data->list_length-1) : 1);
-    xitk_slider_set_max(private_data->item_tree[WSLID], j);
+
+    xitk_slider_set_max(private_data->item_tree[WSLID], max);
     
     if(start == 0)
       xitk_slider_set_to_max(wl, private_data->item_tree[WSLID]);
-    else if(j>1)
+    else if(max)
       xitk_slider_set_pos(wl, private_data->item_tree[WSLID], 
 			  (xitk_slider_get_max(private_data->item_tree[WSLID]) 
 			   - xitk_slider_get_min(private_data->item_tree[WSLID]) 
@@ -255,15 +311,15 @@ void xitk_browser_rebuild_browser(xitk_widget_t *w, int start) {
  */
 void xitk_browser_update_list(xitk_widget_t *w, char **list, int len, int start) {
   browser_private_data_t *private_data;
-
-  if(w) {
-    if(w->widget_type & WIDGET_TYPE_BROWSER) {
-      private_data = (browser_private_data_t *) w->private_data;
-      private_data->content = list;
-      private_data->list_length = len;
-      private_data->current_start = start;
-      xitk_browser_rebuild_browser(w, start);
-    }
+  
+  if(w && (((w->widget_type & WIDGET_GROUP_MASK) & WIDGET_GROUP_BROWSER) &&
+	   (w->widget_type & WIDGET_GROUP_WIDGET))) {
+    
+    private_data = (browser_private_data_t *) w->private_data;
+    private_data->content = list;
+    private_data->list_length = len;
+    private_data->current_start = start;
+    xitk_browser_rebuild_browser(w, start);
   }
 }  
 
@@ -271,12 +327,13 @@ void xitk_browser_update_list(xitk_widget_t *w, char **list, int len, int start)
  * Handle slider movments
  */
 static void browser_slidmove(xitk_widget_t *w, void *data, int pos) {
-  browser_private_data_t *private_data = 
-    (browser_private_data_t *) ((xitk_widget_t*)data)->private_data;
-  int realpos;
+  browser_private_data_t *private_data;
+  int                     realpos;
   
-  
-  if(w->widget_type & (WIDGET_TYPE_SLIDER | WIDGET_TYPE_BUTTON | WIDGET_TYPE_BROWSER)) {
+  if(w && ((w->widget_type & WIDGET_GROUP_MASK) & WIDGET_GROUP_BROWSER)) {
+
+    private_data = (browser_private_data_t *) ((xitk_widget_t*)data)->private_data;
+    
     if(xitk_browser_get_current_selected(((xitk_widget_t*)data)) > -1)
       xitk_browser_release_all_buttons(((xitk_widget_t*)data));
     
@@ -296,11 +353,12 @@ static void browser_slidmove(xitk_widget_t *w, void *data, int pos) {
  * slide up
  */
 static void browser_up(xitk_widget_t *w, void *data) {
-  browser_private_data_t *private_data = 
-    (browser_private_data_t *) ((xitk_widget_t*)data)->private_data;
-  xitk_widget_list_t* wl;
+  browser_private_data_t *private_data;
+  xitk_widget_list_t     *wl;
+  
+  if(w && ((w->widget_type & WIDGET_TYPE_MASK) == WIDGET_TYPE_BUTTON)) {
+    private_data = (browser_private_data_t *) ((xitk_widget_t*)data)->private_data;
 
-  if(w->widget_type & WIDGET_TYPE_BUTTON) {
     wl = (xitk_widget_list_t*) xitk_xmalloc(sizeof(xitk_widget_list_t));
     wl->win = private_data->parent_wlist->win;
     wl->gc = private_data->parent_wlist->gc;
@@ -316,11 +374,12 @@ static void browser_up(xitk_widget_t *w, void *data) {
  * slide down
  */
 static void browser_down(xitk_widget_t *w, void *data) {
-  browser_private_data_t *private_data = 
-    (browser_private_data_t *) ((xitk_widget_t*)data)->private_data;
-  xitk_widget_list_t *wl;
+  browser_private_data_t *private_data;
+  xitk_widget_list_t     *wl;
 
-  if(w->widget_type & WIDGET_TYPE_BUTTON) {
+  if(w && ((w->widget_type & WIDGET_TYPE_MASK) == WIDGET_TYPE_BUTTON)) {
+    private_data = (browser_private_data_t *) ((xitk_widget_t*)data)->private_data;
+    
     wl = (xitk_widget_list_t*) xitk_xmalloc(sizeof(xitk_widget_list_t));
     wl->win = private_data->parent_wlist->win;
     wl->gc = private_data->parent_wlist->gc;
@@ -336,11 +395,13 @@ static void browser_down(xitk_widget_t *w, void *data) {
  * slide up (extern).
  */
 void xitk_browser_step_up(xitk_widget_t *w, void *data) {
-  browser_private_data_t *private_data = 
-    (browser_private_data_t *) w->private_data;
-  xitk_widget_list_t *wl;
+  browser_private_data_t *private_data;
+  xitk_widget_list_t     *wl;
   
-  if(w->widget_type & WIDGET_TYPE_BROWSER) {
+  if(w && (((w->widget_type & WIDGET_GROUP_MASK) & WIDGET_GROUP_BROWSER) &&
+	   (w->widget_type & WIDGET_GROUP_WIDGET))) {
+    private_data = (browser_private_data_t *) w->private_data;
+    
     wl = (xitk_widget_list_t*) xitk_xmalloc(sizeof(xitk_widget_list_t));
     wl->win = private_data->parent_wlist->win;
     wl->gc = private_data->parent_wlist->gc;
@@ -356,11 +417,14 @@ void xitk_browser_step_up(xitk_widget_t *w, void *data) {
  * slide Down (extern).
  */
 void xitk_browser_step_down(xitk_widget_t *w, void *data) {
-  browser_private_data_t *private_data = 
-    (browser_private_data_t *) w->private_data;
-  xitk_widget_list_t* wl;
+  browser_private_data_t *private_data;
+  xitk_widget_list_t     *wl;
   
-  if(w->widget_type & WIDGET_TYPE_BROWSER) {
+  if(w && (((w->widget_type & WIDGET_GROUP_MASK) & WIDGET_GROUP_BROWSER) &&
+	   (w->widget_type & WIDGET_GROUP_WIDGET))) {
+    
+    private_data = (browser_private_data_t *) w->private_data;
+    
     wl = (xitk_widget_list_t*) xitk_xmalloc(sizeof(xitk_widget_list_t));
     wl->win = private_data->parent_wlist->win;
     wl->gc = private_data->parent_wlist->gc;
@@ -377,11 +441,13 @@ void xitk_browser_step_down(xitk_widget_t *w, void *data) {
  * Handle list selections
  */
 static void browser_select(xitk_widget_t *w, void *data, int state) {
-  int i, btn_selected;
-  browser_private_data_t *private_data = 
-    (browser_private_data_t*) ((btnlist_t*)data)->itemlist->private_data;
-
-  if(w->widget_type & WIDGET_TYPE_LABELBUTTON) { 
+  browser_private_data_t *private_data;
+  int                     i, btn_selected;
+  
+  if(w && ((w->widget_type & WIDGET_TYPE_MASK) == WIDGET_TYPE_LABELBUTTON)) { 
+    
+    private_data = (browser_private_data_t*) ((btnlist_t*)data)->itemlist->private_data;
+    
     if(private_data->dbl_click_callback || private_data->callback) {
       if(!strcasecmp((xitk_labelbutton_get_label(private_data->
 						 item_tree[(int)((btnlist_t*)data)->
@@ -404,6 +470,8 @@ static void browser_select(xitk_widget_t *w, void *data, int state) {
       
       if((i = xitk_browser_get_current_selected(((btnlist_t*)data)->itemlist)) > -1) {
 	// Callback call
+	private_data->jumped = i;
+
 	if(private_data->callback)
 	  private_data->callback(((btnlist_t*)data)->itemlist, (void*)(i));
       }
@@ -481,10 +549,126 @@ static void browser_select(xitk_widget_t *w, void *data, int state) {
   }
 }
 
+/*
+ *
+ */
+xitk_widget_t *xitk_browser_get_browser(xitk_widget_list_t *wl, xitk_widget_t *w) {
+  xitk_widget_t *widget = NULL;
+  
+  if(wl && w && ((w->widget_type & WIDGET_GROUP_MASK) & WIDGET_GROUP_BROWSER)) {
+
+    widget = (xitk_widget_t *) xitk_list_first_content(wl->l);
+    while(widget && (widget != w)) {
+      widget = (xitk_widget_t *) xitk_list_next_content(wl->l);
+    }
+    while(widget && (!(((widget->widget_type & WIDGET_GROUP_MASK) & WIDGET_GROUP_BROWSER) &&
+		       (widget->widget_type & WIDGET_GROUP_WIDGET)))) {
+      widget = (xitk_widget_t *) xitk_list_next_content(wl->l);
+    }
+  }
+
+  return widget;
+}
+
+/*
+ * Jump to entry in list which match with the alphanum char key.
+ */
+void xitk_browser_warp_jump(xitk_widget_t *w, char *key, int modifier) {
+  browser_private_data_t *private_data;
+
+  if(w && (((w->widget_type & WIDGET_GROUP_MASK) & WIDGET_GROUP_BROWSER) &&
+	   (w->widget_type & WIDGET_GROUP_WIDGET))) {
+
+    private_data = (browser_private_data_t *) w->private_data;
+    
+    if(((modifier & 0xFFFFFFEF) == MODIFIER_NOMOD) || (modifier & MODIFIER_SHIFT)) {
+      int jumped;
+      int start, end, mark;
+      int focus;
+
+      /* Jump wasn't happend yet */
+      if(private_data->jumped == -1)
+	jumped = 0;
+      else {
+	if((private_data->jumped + 1) < private_data->list_length)
+	  jumped = private_data->jumped + 1;
+	else
+	  jumped = 0;
+      }
+
+      start = jumped;
+      end = ((start - 1) > 0) ? (start - 1) : 0;
+      mark = private_data->list_length;
+      
+      for(; start < mark; start++) {
+	
+	if(!strncasecmp(private_data->content[start], key, strlen(key))) {
+	  
+	  private_data->jumped = start;
+
+	  focus = 0;
+	  if((private_data->list_length - start) <= private_data->max_length) {
+
+	    if(private_data->list_length >= private_data->max_length) {
+	      focus = private_data->max_length - (private_data->list_length - start);
+	      start = (private_data->list_length - private_data->max_length);
+	    }
+	    else
+	      focus = start;
+
+	  }
+	  
+	  xitk_browser_rebuild_browser(w, start);
+
+	  if(private_data->parent_wlist->widget_focused) {
+	    
+	    if(private_data->parent_wlist->widget_focused->notify_focus && 
+	       private_data->parent_wlist->widget_focused->enable == WIDGET_ENABLE) {
+	      
+	      (void) (private_data->parent_wlist->widget_focused->notify_focus) (private_data->parent_wlist, private_data->parent_wlist->widget_focused, FOCUS_LOST);
+	      
+	      private_data->parent_wlist->widget_focused->have_focus = FOCUS_LOST;
+	    }
+	    
+	    if(private_data->parent_wlist->widget_focused->paint)
+	      (private_data->parent_wlist->widget_focused->paint) (private_data->parent_wlist->widget_focused, private_data->parent_wlist->win, private_data->parent_wlist->gc);
+
+	  }
+	  
+	  xitk_browser_set_select(w, focus);
+	  
+	  private_data->parent_wlist->widget_focused = private_data->item_tree[WBSTART + focus];
+	  
+	  if(private_data->parent_wlist->widget_focused->notify_focus && private_data->parent_wlist->widget_focused->enable == WIDGET_ENABLE) {
+	    
+	    (void) (private_data->parent_wlist->widget_focused->notify_focus) (private_data->parent_wlist, private_data->parent_wlist->widget_focused, FOCUS_RECEIVED);
+	    
+	    private_data->parent_wlist->widget_focused->have_focus = FOCUS_RECEIVED;
+	    
+	  }
+	  
+	  if(private_data->parent_wlist->widget_focused->paint)
+	    (private_data->parent_wlist->widget_focused->paint) (private_data->parent_wlist->widget_focused, private_data->parent_wlist->win, private_data->parent_wlist->gc);
+	  
+	  return;
+	}
+
+	if(((start + 1) == mark) && end) {
+	  mark = end;
+	  start = end = -1;
+	}
+      }
+
+      private_data->jumped = -1;
+    }
+  }
+}
+
 /**
  * Create the list browser
  */
-static xitk_widget_t *_xitk_browser_create(xitk_skin_config_t *skonfig, xitk_browser_widget_t *br,
+static xitk_widget_t *_xitk_browser_create(xitk_widget_list_t *wl,
+					   xitk_skin_config_t *skonfig, xitk_browser_widget_t *br,
 					   int x, int y, int width, int height,
 					   char *skin_element_name,
 					   xitk_widget_t *mywidget,
@@ -504,14 +688,12 @@ static xitk_widget_t *_xitk_browser_create(xitk_skin_config_t *skonfig, xitk_bro
   private_data->bWidget              = mywidget;
   private_data->imlibdata            = br->imlibdata;
   private_data->skin_element_name    = (skin_element_name == NULL) ? NULL : strdup(br->browser.skin_element_name);
+  private_data->jumped               = -1;
   private_data->content              = br->browser.entries;
   private_data->list_length          = br->browser.num_entries;
   private_data->max_length           = br->browser.max_displayed_entries;
 
-  if(br->dbl_click_time)
-    private_data->dbl_click_time     = br->dbl_click_time;
-  else
-    private_data->dbl_click_time     = DEFAULT_DBL_CLICK_TIME;
+  private_data->dbl_click_time       = xitk_get_timer_dbl_click();
 
   if(br->dbl_click_callback)
     private_data->dbl_click_callback = br->dbl_click_callback;
@@ -528,6 +710,8 @@ static xitk_widget_t *_xitk_browser_create(xitk_skin_config_t *skonfig, xitk_bro
   
   mywidget->private_data               = private_data;
 
+  mywidget->widget_list                = wl;
+
   mywidget->enable                     = enable;
   mywidget->running                    = 1;
   mywidget->visible                    = visible;
@@ -539,7 +723,7 @@ static xitk_widget_t *_xitk_browser_create(xitk_skin_config_t *skonfig, xitk_bro
   mywidget->width                      = width;
   mywidget->height                     = height;
   
-  mywidget->widget_type                = WIDGET_TYPE_BROWSER | WIDGET_TYPE_GROUP;
+  mywidget->widget_type                = WIDGET_GROUP | WIDGET_GROUP_WIDGET | WIDGET_GROUP_BROWSER;
   mywidget->paint                      = paint;
   mywidget->notify_click               = NULL;
   mywidget->notify_focus               = NULL;
@@ -558,7 +742,8 @@ static xitk_widget_t *_xitk_browser_create(xitk_skin_config_t *skonfig, xitk_bro
 /**
  * Create the list browser
  */
-xitk_widget_t *xitk_browser_create(xitk_skin_config_t *skonfig, xitk_browser_widget_t *br) {
+xitk_widget_t *xitk_browser_create(xitk_widget_list_t *wl,
+				   xitk_skin_config_t *skonfig, xitk_browser_widget_t *br) {
   xitk_widget_t                 *mywidget;
   browser_private_data_t        *private_data;
   xitk_button_widget_t           b;
@@ -575,30 +760,9 @@ xitk_widget_t *xitk_browser_create(xitk_skin_config_t *skonfig, xitk_browser_wid
   private_data = (browser_private_data_t *) xitk_xmalloc(sizeof(browser_private_data_t));
   private_data->skonfig = skonfig;
 
-  b.skin_element_name = br->arrow_up.skin_element_name;
-  b.callback          = browser_up;
-  b.userdata          = (void *)mywidget;
-  xitk_list_append_content(br->parent_wlist->l, 
-			   (private_data->item_tree[WBUP] = xitk_button_create(skonfig, &b)));
   
-  sl.min                      = 0;
-  sl.max                      = (br->browser.num_entries > (br->browser.max_displayed_entries-1) ? br->browser.num_entries-1 : 0);
-  sl.step                     = 1;
-  sl.skin_element_name        = br->slider.skin_element_name;
-  sl.callback                 = browser_slidmove;
-  sl.userdata                 = (void*)mywidget;
-  sl.motion_callback          = browser_slidmove;
-  sl.motion_userdata          = (void*)mywidget;
-  xitk_list_append_content(br->parent_wlist->l,
-	  (private_data->item_tree[WSLID] = xitk_slider_create(skonfig, &sl)));
-  
-  xitk_slider_set_to_max(br->parent_wlist, private_data->item_tree[WSLID]);
-  
-  b.skin_element_name = br->arrow_dn.skin_element_name;
-  b.callback          = browser_down;
-  b.userdata          = (void *)mywidget;
-  xitk_list_append_content(br->parent_wlist->l, 
-			  private_data->item_tree[WBDN] = xitk_button_create (skonfig, &b));
+  br->browser.max_displayed_entries = xitk_skin_get_browser_entries(skonfig, 
+								    br->browser.skin_element_name);
   
   {
 
@@ -608,7 +772,7 @@ xitk_widget_t *xitk_browser_create(xitk_skin_config_t *skonfig, xitk_browser_wid
     x = xitk_skin_get_coord_x(skonfig, br->browser.skin_element_name);
     y = xitk_skin_get_coord_y(skonfig, br->browser.skin_element_name);
     
-    for(i = WBSTART; i < br->browser.max_displayed_entries+WBSTART; i++) {
+    for(i = WBSTART; i < (EXTRA_BTNS + WBSTART); i++) {
       
       bt = (btnlist_t *) xitk_xmalloc(sizeof(btnlist_t));
       bt->itemlist = (xitk_widget_t *) xitk_xmalloc(sizeof(xitk_widget_t));
@@ -623,15 +787,55 @@ xitk_widget_t *xitk_browser_create(xitk_skin_config_t *skonfig, xitk_browser_wid
       lb.skin_element_name = br->browser.skin_element_name;
       xitk_list_append_content(br->parent_wlist->l, 
 			       (private_data->item_tree[i] = 
-				xitk_labelbutton_create (skonfig, &lb)));
+				xitk_labelbutton_create (br->parent_wlist, skonfig, &lb)));
+      private_data->item_tree[i]->widget_type |= WIDGET_GROUP | WIDGET_GROUP_BROWSER;
+      
+      xitk_disable_widget(private_data->item_tree[i]);
+      xitk_hide_widget(br->parent_wlist, private_data->item_tree[i]);
       
       (void) xitk_set_widget_pos(private_data->item_tree[i], x, y);
-
+      
       y += xitk_get_widget_height(private_data->item_tree[i]) + 1;
     }
+    
+    for(i = WBSTART; i < (br->browser.max_displayed_entries + WBSTART); i++) {
+      xitk_enable_widget(private_data->item_tree[i]);
+      xitk_show_widget(br->parent_wlist, private_data->item_tree[i]);
+    }
+
   }
   
-  return _xitk_browser_create(skonfig, br, 
+  b.skin_element_name = br->arrow_up.skin_element_name;
+  b.callback          = browser_up;
+  b.userdata          = (void *)mywidget;
+  xitk_list_append_content(br->parent_wlist->l, 
+			   (private_data->item_tree[WBUP] = 
+			    xitk_button_create(br->parent_wlist, skonfig, &b)));
+  private_data->item_tree[WBUP]->widget_type |= WIDGET_GROUP | WIDGET_GROUP_BROWSER;
+  
+  sl.min                      = 0;
+  sl.max                      = (br->browser.num_entries > (br->browser.max_displayed_entries-1) ? br->browser.num_entries-1 : 0);
+  sl.step                     = 1;
+  sl.skin_element_name        = br->slider.skin_element_name;
+  sl.callback                 = browser_slidmove;
+  sl.userdata                 = (void*)mywidget;
+  sl.motion_callback          = browser_slidmove;
+  sl.motion_userdata          = (void*)mywidget;
+  xitk_list_append_content(br->parent_wlist->l,
+	  (private_data->item_tree[WSLID] = xitk_slider_create(br->parent_wlist, skonfig, &sl)));
+  private_data->item_tree[WSLID]->widget_type |= WIDGET_GROUP | WIDGET_GROUP_BROWSER;
+
+  xitk_slider_set_to_max(br->parent_wlist, private_data->item_tree[WSLID]);
+  
+  b.skin_element_name = br->arrow_dn.skin_element_name;
+  b.callback          = browser_down;
+  b.userdata          = (void *)mywidget;
+  xitk_list_append_content(br->parent_wlist->l, 
+			  private_data->item_tree[WBDN] = 
+			   xitk_button_create (br->parent_wlist, skonfig, &b));
+  private_data->item_tree[WBDN]->widget_type |= WIDGET_GROUP | WIDGET_GROUP_BROWSER;
+  
+  return _xitk_browser_create(wl, skonfig, br, 
 			      0, 0, 0, 0, br->browser.skin_element_name, mywidget, private_data,
 			      (xitk_skin_get_visibility(skonfig, br->browser.skin_element_name)) ? 1 : -1,
 			      xitk_skin_get_enability(skonfig, br->browser.skin_element_name));
@@ -640,7 +844,8 @@ xitk_widget_t *xitk_browser_create(xitk_skin_config_t *skonfig, xitk_browser_wid
 /*
  *
  */
-xitk_widget_t *xitk_noskin_browser_create(xitk_browser_widget_t *br, GC gc, int x, int y, 
+xitk_widget_t *xitk_noskin_browser_create(xitk_widget_list_t *wl,
+					  xitk_browser_widget_t *br, GC gc, int x, int y, 
 					  int itemw, int itemh, int slidw, char *fontname) {
   xitk_widget_t              *mywidget;
   browser_private_data_t     *private_data;
@@ -658,14 +863,54 @@ xitk_widget_t *xitk_noskin_browser_create(xitk_browser_widget_t *br, GC gc, int
   private_data = (browser_private_data_t *) xitk_xmalloc(sizeof(browser_private_data_t));
   private_data->skonfig = NULL;
 
+  {
+    int            ix = x, iy = y, i;
+    btnlist_t     *bt;
+    xitk_image_t  *wimage;
+
+    for(i = WBSTART; i < (br->browser.max_displayed_entries+WBSTART); i++) {
+      
+      bt = (btnlist_t *) xitk_xmalloc(sizeof(btnlist_t));
+      bt->itemlist = mywidget;
+      bt->sel = i;
+      
+      lb.button_type       = RADIO_BUTTON;
+      lb.label             = "";
+      lb.align             = LABEL_ALIGN_LEFT;
+      lb.callback          = NULL;
+      lb.state_callback    = browser_select;
+      lb.userdata          = (void *)(bt);
+      lb.skin_element_name = NULL;
+      xitk_list_append_content(br->parent_wlist->l, 
+			       (private_data->item_tree[i] = 
+				xitk_noskin_labelbutton_create (br->parent_wlist, &lb,
+								ix, iy,
+								itemw, itemh,
+								"Black", "Black", "White", 
+								fontname)));
+      
+      private_data->item_tree[i]->widget_type |= WIDGET_GROUP | WIDGET_GROUP_BROWSER;
+
+      wimage = xitk_get_widget_foreground_skin(private_data->item_tree[i]);
+      if(wimage)
+	draw_flat_three_state(br->imlibdata, wimage);				
+
+      (void) xitk_set_widget_pos(private_data->item_tree[i], ix, iy);
+      
+      iy += itemh;
+   
+    }
+  }
+
   b.skin_element_name = NULL;
   b.callback          = browser_up;
   b.userdata          = (void *)mywidget;
   xitk_list_append_content(br->parent_wlist->l, 
 			   (private_data->item_tree[WBUP] = 
-			    xitk_noskin_button_create(&b,
+			    xitk_noskin_button_create(br->parent_wlist, &b,
 						      x + itemw, y + 0, slidw, btnh)));
-  
+  private_data->item_tree[WBUP]->widget_type |= WIDGET_GROUP | WIDGET_GROUP_BROWSER;
+
   { /* Draw arrow in button */
     xitk_image_t *wimage = xitk_get_widget_foreground_skin(private_data->item_tree[WBUP]);
     
@@ -684,10 +929,11 @@ xitk_widget_t *xitk_noskin_browser_create(xitk_browser_widget_t *br, GC gc, int
   sl.motion_userdata          = (void*)mywidget;
   xitk_list_append_content(br->parent_wlist->l,
 	   (private_data->item_tree[WSLID] = 
-	    xitk_noskin_slider_create(&sl,
+	    xitk_noskin_slider_create(br->parent_wlist, &sl,
 				      x + itemw, y + btnh, slidw, 
 				      (itemh * br->browser.max_displayed_entries) - (btnh * 2),
 				      XITK_VSLIDER)));
+  private_data->item_tree[WSLID]->widget_type |= WIDGET_GROUP | WIDGET_GROUP_BROWSER;
 			   
   xitk_slider_set_to_max(br->parent_wlist, private_data->item_tree[WSLID]);
   
@@ -696,11 +942,12 @@ xitk_widget_t *xitk_noskin_browser_create(xitk_browser_widget_t *br, GC gc, int
   b.userdata          = (void *)mywidget;
   xitk_list_append_content(br->parent_wlist->l, 
 	   (private_data->item_tree[WBDN] = 
-	    xitk_noskin_button_create(&b,
+	    xitk_noskin_button_create(br->parent_wlist, &b,
 				      x + itemw, 
 				      y + ((itemh * br->browser.max_displayed_entries) - btnh), 
 				      slidw, btnh)));
-  
+  private_data->item_tree[WBDN]->widget_type |= WIDGET_GROUP | WIDGET_GROUP_BROWSER;
+
   { /* Draw arrow in button */
     xitk_image_t *wimage = xitk_get_widget_foreground_skin(private_data->item_tree[WBDN]);
     
@@ -708,43 +955,8 @@ xitk_widget_t *xitk_noskin_browser_create(xitk_browser_widget_t *br, GC gc, int
       draw_arrow_down(b.imlibdata, wimage);
 
   }
-  {
-    int            ix = x, iy = y, i;
-    btnlist_t     *bt;
-    xitk_image_t  *wimage;
-
-    for(i = WBSTART; i < br->browser.max_displayed_entries+WBSTART; i++) {
-      
-      bt = (btnlist_t *) xitk_xmalloc(sizeof(btnlist_t));
-      bt->itemlist = mywidget;
-      bt->sel = i;
-      
-      lb.button_type       = RADIO_BUTTON;
-      lb.label             = "";
-      lb.align             = LABEL_ALIGN_CENTER;
-      lb.callback          = NULL;
-      lb.state_callback    = browser_select;
-      lb.userdata          = (void *)(bt);
-      lb.skin_element_name = NULL;
-      xitk_list_append_content(br->parent_wlist->l, 
-			       (private_data->item_tree[i] = 
-				xitk_noskin_labelbutton_create (&lb,
-								ix, iy,
-								itemw, itemh,
-								"Black", "Black", "White", 
-								fontname)));
-      
-      wimage = xitk_get_widget_foreground_skin(private_data->item_tree[i]);
-      if(wimage)
-	draw_flat_three_state(br->imlibdata, wimage);				
-
-      (void) xitk_set_widget_pos(private_data->item_tree[i], ix, iy);
-
-      iy += itemh;
-    }
-  }
   
-  return _xitk_browser_create(NULL, br, x, y, 
+  return _xitk_browser_create(wl, NULL, br, x, y, 
 			      (itemw + slidw), (itemh * br->browser.max_displayed_entries), 
 			      NULL, mywidget, private_data, 1, 1);
 }
