@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: combo.h,v 1.5 2002/03/21 00:22:52 f1rmb Exp $
+ * $Id: combo.h,v 1.7 2002/07/17 20:29:31 f1rmb Exp $
  *
  */
 #ifndef HAVE_XITK_COMBO_H
@@ -60,13 +60,15 @@ typedef struct {
 /*
  * Create a combo box.
  */
-xitk_widget_t *xitk_create_combo(xitk_skin_config_t *skonfig, xitk_slider_widget_t *s,
+xitk_widget_t *xitk_combo_create(xitk_widget_list_t *wl,
+				 xitk_skin_config_t *skonfig, xitk_combo_widget_t *s,
 				 xitk_widget_t **lw, xitk_widget_t **bw);
 
 /*
  * Same as above, without skinable feature.
  */
-xitk_widget_t *xitk_noskin_combo_create(xitk_combo_widget_t *c, int x, int y, int width,
+xitk_widget_t *xitk_noskin_combo_create(xitk_widget_list_t *wl,
+					xitk_combo_widget_t *c, int x, int y, int width,
 					xitk_widget_t **lw, xitk_widget_t **bw);
 
 /*
