@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: font.c,v 1.4 2002/03/21 00:22:53 f1rmb Exp $
+ * $Id: font.c,v 1.5 2002/05/11 12:49:09 guenter Exp $
  *
  */
 #ifdef HAVE_CONFIG_H
@@ -230,10 +230,16 @@ void xitk_font_text_extent(xitk_font_t *xtfs, const char *c, int length,
     *rbearing = ov.rbearing;
   if(width)
     *width    = ov.width;
+  if(ascent)
+    *ascent  = xtfs->font->ascent;
+  if(descent)
+    *descent = xtfs->font->descent;
+#if 0
   if(ascent)
     *ascent   = ov.ascent;
   if(descent)
     *descent  = ov.descent;
+#endif
 }
 
 /*
