@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: font.h,v 1.3 2002/03/21 00:22:53 f1rmb Exp $
+ * $Id: font.h,v 1.5 2002/05/23 11:24:42 f1rmb Exp $
  *
  */
 #ifndef HAVE_XITK_FONT_H
@@ -28,9 +28,13 @@
 
 #include "_xitk.h"
 
-#define DEFAULT_FONT_10 "*-lucida-*-r-*-*-10-*-*-*-*-*-*-*"
-#define DEFAULT_FONT_12 "*-lucida-*-r-*-*-12-*-*-*-*-*-*-*"
-#define DEFAULT_FONT_14 "*-lucida-*-r-*-*-14-*-*-*-*-*-*-*"
+#define DEFAULT_FONT_10      "*-helvetica-medium-r-*-*-10-*-*-*-*-*-*-*"
+#define DEFAULT_FONT_12      "*-helvetica-medium-r-*-*-12-*-*-*-*-*-*-*"
+#define DEFAULT_FONT_14      "*-helvetica-medium-r-*-*-14-*-*-*-*-*-*-*"
+#define DEFAULT_BOLD_FONT_10 "*-helvetica-bold-r-*-*-10-*-*-*-*-*-*-*"
+#define DEFAULT_BOLD_FONT_12 "*-helvetica-bold-r-*-*-12-*-*-*-*-*-*-*"
+#define DEFAULT_BOLD_FONT_14 "*-helvetica-bold-r-*-*-14-*-*-*-*-*-*-*"
+#define FONT_HEIGHT_MODEL    "azertyuiopqsdfghjklmwxcvbnAZERTYUIOPQSDFGHJKLMWXCVBN&(-_)=~#{[|`\\^@]}%"
 
 /*
  * load font from font name. Return NULL on failure.
