@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: image.c,v 1.26 2002/04/12 21:38:51 f1rmb Exp $
+ * $Id: image.c,v 1.35 2002/07/17 20:29:31 f1rmb Exp $
  *
  */
 #ifdef HAVE_CONFIG_H
@@ -40,7 +40,7 @@
  * Get a pixel color from rgb values.
  */
 unsigned int xitk_get_pixel_color_from_rgb(ImlibData *im, int r, int g, int b) {
-  XColor xcolor;
+  XColor       xcolor;
   unsigned int pixcol;
 
   assert(im);
@@ -66,34 +66,46 @@ unsigned int xitk_get_pixel_color_from_rgb(ImlibData *im, int r, int g, int b) {
  */
 unsigned int xitk_get_pixel_color_black(ImlibData *im) {
   int user_color = xitk_get_black_color();
-  if(user_color > 0)
+  if(user_color >= 0)
     return (unsigned int) user_color;
   return xitk_get_pixel_color_from_rgb(im, 0, 0, 0);
 }
 unsigned int xitk_get_pixel_color_white(ImlibData *im) {
   int user_color = xitk_get_white_color();
-  if(user_color > 0)
+  if(user_color >= 0)
     return (unsigned int) user_color;
   return xitk_get_pixel_color_from_rgb(im, 255, 255, 255);
 }
 unsigned int xitk_get_pixel_color_lightgray(ImlibData *im) {
   int user_color = xitk_get_focus_color();
-  if(user_color > 0)
+  if(user_color >= 0)
     return (unsigned int) user_color;
   return xitk_get_pixel_color_from_rgb(im, 224, 224, 224);
 }
 unsigned int xitk_get_pixel_color_gray(ImlibData *im) {
   int user_color = xitk_get_background_color();
-  if(user_color > 0)
+  if(user_color >= 0)
     return (unsigned int) user_color;
   return xitk_get_pixel_color_from_rgb(im, 192, 192, 192);
 }
 unsigned int xitk_get_pixel_color_darkgray(ImlibData *im) {
   int user_color = xitk_get_select_color();
-  if(user_color > 0)
+  if(user_color >= 0)
     return (unsigned int) user_color;
   return xitk_get_pixel_color_from_rgb(im, 160, 160, 160);
 }
+unsigned int xitk_get_pixel_color_warning_foreground(ImlibData *im) {
+  int user_color = xitk_get_warning_foreground();
+  if(user_color >= 0)
+    return (unsigned int) user_color;
+  return xitk_get_pixel_color_black(im);
+}
+unsigned int xitk_get_pixel_color_warning_background(ImlibData *im) {
+  int user_color = xitk_get_warning_background();
+  if(user_color >= 0)
+    return (unsigned int) user_color;
+  return xitk_get_pixel_color_from_rgb(im, 255, 255, 0);
+}
 
 /*
  *
@@ -263,6 +275,7 @@ xitk_image_t *xitk_image_create_image_with_colors_from_string(ImlibData *im,
   po = p = &bufsubs[0];
 
   while(*p != '\0') {
+
     switch(*p) {
       
       /* Ignore */
@@ -391,8 +404,8 @@ void xitk_image_add_mask(ImlibData *im, xitk_image_t *dest) {
 static void _draw_arrow(ImlibData *im, xitk_image_t *p, int direction) {
   GC             gc;
   XGCValues      gcv;
-  int            w = p->width / 3;
-  int            h = p->height;
+  int            w;
+  int            h;
   XPoint         points[4];
   int            i, offset = 0;
   short          x1, x2, x3;
@@ -400,6 +413,9 @@ static void _draw_arrow(ImlibData *im, xitk_image_t *p, int direction) {
 
   assert(im && p);
   
+  w = p->width / 3;
+  h = p->height;
+
   XLOCK(im->x.disp);
   
   gcv.graphics_exposures = False;
@@ -512,6 +528,43 @@ static void _draw_rectangular_box(ImlibData *im, Pixmap p,
   gcv.graphics_exposures = False;
   gc = XCreateGC(im->x.disp, p, GCGraphicsExposures, &gcv);
   
+  if(relief == DRAW_OUTTER)
+    XSetForeground(im->x.disp, gc, xitk_get_pixel_color_white(im));
+  else if(relief == DRAW_INNER)
+    XSetForeground(im->x.disp, gc, xitk_get_pixel_color_black(im));
+
+  XDrawLine(im->x.disp, p, gc, x, y, (x + excstart), y);
+  XDrawLine(im->x.disp, p, gc, (x + excstop), y, (x + width), y);
+  XDrawLine(im->x.disp, p, gc, x, y, x, (y + height));
+  if(relief == DRAW_OUTTER)
+    XSetForeground(im->x.disp, gc, xitk_get_pixel_color_black(im));
+  else if(relief == DRAW_INNER)
+    XSetForeground(im->x.disp, gc, xitk_get_pixel_color_white(im));
+  
+  XDrawLine(im->x.disp, p, gc, (x + width), y, (x + width), (y + height));
+  XDrawLine(im->x.disp, p, gc, x, (y + height), (x + width), (y + height));
+
+  XFreeGC(im->x.disp, gc);
+
+  XUNLOCK(im->x.disp);
+}
+
+/*
+ *
+ */
+static void _draw_rectangular_box_light(ImlibData *im, Pixmap p, 
+					int x, int y, int excstart, int excstop,
+					int width, int height, int relief) {
+  GC            gc;
+  XGCValues     gcv;
+
+  assert(im && p && width && height);
+
+  XLOCK(im->x.disp);
+
+  gcv.graphics_exposures = False;
+  gc = XCreateGC(im->x.disp, p, GCGraphicsExposures, &gcv);
+  
   if(relief == DRAW_OUTTER)
     XSetForeground(im->x.disp, gc, xitk_get_pixel_color_white(im));
   else if(relief == DRAW_INNER)
@@ -587,11 +640,14 @@ void draw_rectangular_outter_box(ImlibData *im, Pixmap p, int x, int y, int widt
 static void _draw_three_state(ImlibData *im, xitk_image_t *p, int style) {
   GC            gc;
   XGCValues     gcv;
-  int           w = p->width / 3;
-  int           h = p->height;
+  int           w;
+  int           h;
 
   assert(im && p);
 
+  w = p->width / 3;
+  h = p->height;
+
   XLOCK(im->x.disp);
 
   gcv.graphics_exposures = False;
@@ -612,21 +668,29 @@ static void _draw_three_state(ImlibData *im, xitk_image_t *p, int style) {
   
   XSetForeground(im->x.disp, gc, xitk_get_pixel_color_darkgray(im));
   if(style == STYLE_BEVEL) {
-    XDrawLine(im->x.disp, p->image, gc, (w - 1), 0, (w - 1), (h - 1));
-    XDrawLine(im->x.disp, p->image, gc, 0, (h - 1), w, (h - 1));
+    XDrawLine(im->x.disp, p->image, gc, (w - 2), 2, (w - 2), (h - 3));
+    XDrawLine(im->x.disp, p->image, gc, 2, (h - 2), w-2, (h - 2));
   }
-  XDrawLine(im->x.disp, p->image, gc, (w * 2) - 1, 0, (w * 2) - 1, h);
-  XDrawLine(im->x.disp, p->image, gc, w, (h - 1), (w * 2), (h - 1));
+  XDrawLine(im->x.disp, p->image, gc, 2*w - 2,     2, 2*w - 2, h - 3);
+  XDrawLine(im->x.disp, p->image, gc,     w+2, h - 2, 2*w - 2, h - 2);
+  XDrawLine(im->x.disp, p->image, gc,   w * 2,     0,   w * 3,     0);
+  XDrawLine(im->x.disp, p->image, gc,   w * 2,     0,   w * 2, h - 1);
 
   XFillRectangle(im->x.disp, p->image, gc, w * 2 , 0, (w - 1), (h - 1));
  
   XSetForeground(im->x.disp, gc, xitk_get_pixel_color_black(im));
-  XDrawLine(im->x.disp, p->image, gc, (w * 2), 0, (w * 3), 0);
-  XDrawLine(im->x.disp, p->image, gc, (w * 2), 0, (w * 2), (h - 1));
+  if(style == STYLE_BEVEL) {
+    XDrawLine(im->x.disp, p->image, gc, (w - 1), 0, (w - 1), (h - 1));
+    XDrawLine(im->x.disp, p->image, gc, 0, (h - 1), w-1, (h - 1));
+  }
+  XDrawLine(im->x.disp, p->image, gc, (w * 2)+1, 1, (w * 3)-1, 1);
+  XDrawLine(im->x.disp, p->image, gc, (w * 2)+1, 1, (w * 2)+1, (h - 2));
+  XDrawLine(im->x.disp, p->image, gc, (w * 2) - 1, 0, (w * 2) - 1, h);
+  XDrawLine(im->x.disp, p->image, gc, w, (h - 1), (w * 2)-1, (h - 1));
 
   XSetForeground(im->x.disp, gc, xitk_get_pixel_color_white(im));
-  XDrawLine(im->x.disp, p->image, gc, (w * 3) - 1, 0, (w * 3) - 1, (h - 1));
-  XDrawLine(im->x.disp, p->image, gc, (w * 2), (h - 1), (w * 3), (h - 1));
+  XDrawLine(im->x.disp, p->image, gc, (w * 3) - 1, 1, (w * 3) - 1, (h - 1));
+  XDrawLine(im->x.disp, p->image, gc, (w * 2) + 1, (h - 1), (w * 3), (h - 1));
 
   XFreeGC(im->x.disp, gc);
 
@@ -639,11 +703,14 @@ static void _draw_three_state(ImlibData *im, xitk_image_t *p, int style) {
 static void _draw_two_state(ImlibData *im, xitk_image_t *p, int style) {
   GC            gc;
   XGCValues     gcv;
-  int           w = p->width / 2;
-  int           h = p->height;
+  int           w;
+  int           h;
 
   assert(im && p);
 
+  w = p->width / 2;
+  h = p->height;
+
   XLOCK(im->x.disp);
 
   gcv.graphics_exposures = False;
@@ -678,7 +745,7 @@ static void _draw_two_state(ImlibData *im, xitk_image_t *p, int style) {
 /*
  *
  */
-static void _draw_relief(ImlibData *im, Pixmap p, int w, int h, int relief) {
+static void _draw_relief(ImlibData *im, Pixmap p, int w, int h, int relief, int light) {
   GC            gc;
   XGCValues     gcv;
   
@@ -699,7 +766,9 @@ static void _draw_relief(ImlibData *im, Pixmap p, int w, int h, int relief) {
     if(relief == DRAW_OUTTER)
       _draw_rectangular_box_with_colors(im, p, 0, 0, w-1, h-1, 
 					xitk_get_pixel_color_white(im),	
-					xitk_get_pixel_color_darkgray(im),
+					((light) 
+					 ? xitk_get_pixel_color_darkgray(im)
+					 : xitk_get_pixel_color_black(im)),
 					relief);
     else if(relief == DRAW_INNER)
       _draw_rectangular_box_with_colors(im, p, 0, 0, w-1, h-1, 
@@ -740,11 +809,14 @@ void draw_bevel_two_state(ImlibData *im, xitk_image_t *p) {
 static void _draw_paddle_three_state(ImlibData *im, xitk_image_t *p, int direction) {
   GC            gc, mgc;
   XGCValues     gcv;
-  int           w = p->width / 3;
-  int           h = p->height;
+  int           w;
+  int           h;
 
   assert(im && p);
 
+  w = p->width / 3;
+  h = p->height;
+
   XLOCK(im->x.disp);
 
   gcv.graphics_exposures = False;
@@ -781,11 +853,11 @@ static void _draw_paddle_three_state(ImlibData *im, xitk_image_t *p, int directi
   
   _draw_rectangular_box_with_colors(im, p->image, 2, 2, (w-1)-4, (h-1)-4, 
 				    xitk_get_pixel_color_white(im),
-				    xitk_get_pixel_color_darkgray(im),
+				    xitk_get_pixel_color_black(im),
 				    DRAW_OUTTER);
   _draw_rectangular_box_with_colors(im, p->image, w+2, 2, (w-1)-4, (h-1)-4, 
 				    xitk_get_pixel_color_white(im),
-				    xitk_get_pixel_color_darkgray(im),
+				    xitk_get_pixel_color_black(im),
 				    DRAW_OUTTER);
   _draw_rectangular_box_with_colors(im, p->image, (w*2)+2, 2, (w-1)-4, (h-1)-4, 
 				    xitk_get_pixel_color_white(im),
@@ -833,21 +905,27 @@ void draw_paddle_three_state_horizontal(ImlibData *im, xitk_image_t *p) {
  *
  */
 void draw_inner(ImlibData *im, Pixmap p, int w, int h) {
-  _draw_relief(im, p, w, h, DRAW_INNER);
+  _draw_relief(im, p, w, h, DRAW_INNER, 0);
+}
+void draw_inner_light(ImlibData *im, Pixmap p, int w, int h) {
+  _draw_relief(im, p, w, h, DRAW_INNER, 1);
 }
 
 /*
  *
  */
 void draw_outter(ImlibData *im, Pixmap p, int w, int h) {
-  _draw_relief(im, p, w, h, DRAW_OUTTER);
+  _draw_relief(im, p, w, h, DRAW_OUTTER, 0);
+}
+void draw_outter_light(ImlibData *im, Pixmap p, int w, int h) {
+  _draw_relief(im, p, w, h, DRAW_OUTTER, 1);
 }
 
 /*
  *
  */
 void draw_flat(ImlibData *im, Pixmap p, int w, int h) {
-  _draw_relief(im, p, w, h, DRAW_FLATTER);
+  _draw_relief(im, p, w, h, DRAW_FLATTER, 1);
 }
 
 /*
@@ -926,19 +1004,20 @@ static void _draw_frame(ImlibData *im, Pixmap p,
     xstart = 3;
     xstop = fwidth + 12;
   }
-  _draw_rectangular_box(im, p, x, (y - yoff), 
-			xstart, xstop,
-			w, (h - yoff), sty[0]);
+  _draw_rectangular_box_light(im, p, x, (y - yoff), 
+			      xstart, xstop,
+			      w, (h - yoff), sty[0]);
   
   if(title)
     xstart--;
   
-  _draw_rectangular_box(im, p, (x + 1), ((y - yoff) + 1), 
-			xstart, xstop,
-			(w - 2), ((h - yoff) - 2), sty[1]);
+  _draw_rectangular_box_light(im, p, (x + 1), ((y - yoff) + 1), 
+			      xstart, xstop,
+			      (w - 2), ((h - yoff) - 2), sty[1]);
   
   if(title) {
     XLOCK(im->x.disp);
+    XSetForeground(im->x.disp, gc, xitk_get_pixel_color_black(im));
     XDrawString(im->x.disp, p, gc, (x + 6), y, buf, strlen(buf));
     XUNLOCK(im->x.disp);
     
@@ -974,6 +1053,7 @@ void draw_tab(ImlibData *im, xitk_image_t *p) {
   assert(im && p);
 
   XLOCK(im->x.disp);
+
   w = p->width / 3;
   h = p->height;
 
@@ -981,7 +1061,7 @@ void draw_tab(ImlibData *im, xitk_image_t *p) {
   gc = XCreateGC(im->x.disp, p->image, GCGraphicsExposures, &gcv);
   
   XSetForeground(im->x.disp, gc, xitk_get_pixel_color_gray(im));
-  XFillRectangle(im->x.disp, p->image, gc, 0, 0, (w * 3) , h);
+  XFillRectangle(im->x.disp, p->image, gc, 0, 0, (w * 3), h);
   XSetForeground(im->x.disp, gc, xitk_get_pixel_color_lightgray(im));
   XFillRectangle(im->x.disp, p->image, gc, 0, 3, (w - 1), h);
   XFillRectangle(im->x.disp, p->image, gc, w, 0, (w - 1), h);
@@ -995,7 +1075,7 @@ void draw_tab(ImlibData *im, xitk_image_t *p) {
   XDrawLine(im->x.disp, p->image, gc, (w * 2), 3, (w * 2), (h - 1));
   XDrawLine(im->x.disp, p->image, gc, (w * 2), 3, (w * 2) + 3, 0);
 
-  XSetForeground(im->x.disp, gc, xitk_get_pixel_color_darkgray(im));
+  XSetForeground(im->x.disp, gc, xitk_get_pixel_color_black(im));
   XDrawLine(im->x.disp, p->image, gc, (w - 1), 3, (w - 1), h);
   XDrawLine(im->x.disp, p->image, gc, (w * 2) - 1, 0, (w * 2) - 1, h);
   XDrawLine(im->x.disp, p->image, gc, (w * 3) - 1, 0, (w * 3) - 1, h);
@@ -1013,15 +1093,17 @@ void draw_tab(ImlibData *im, xitk_image_t *p) {
 void draw_paddle_rotate(ImlibData *im, xitk_image_t *p) {
   GC            gc, mgc;
   XGCValues     gcv;
-  int           w = p->width/3;
-  int           h = p->height;
-  unsigned int  red, yellow, gray;
+  int           w;
+  int           h;
+  unsigned int  ccolor, fcolor, ncolor;
 
   assert(im && p);
   
-  red    = xitk_get_pixel_color_from_rgb(im, 255, 0, 0);
-  yellow = xitk_get_pixel_color_from_rgb(im, 255, 255, 0);
-  gray   = xitk_get_pixel_color_darkgray(im);
+  w = p->width/3;
+  h = p->height;
+  ncolor = xitk_get_pixel_color_darkgray(im);
+  fcolor = xitk_get_pixel_color_warning_background(im);
+  ccolor = xitk_get_pixel_color_lightgray(im);
   
   XLOCK(im->x.disp);
   
@@ -1033,7 +1115,7 @@ void draw_paddle_rotate(ImlibData *im, xitk_image_t *p) {
   
   {
     int x, i;
-    unsigned int bg_colors[3] = { gray, yellow, red };
+    unsigned int bg_colors[3] = { ncolor, fcolor, ccolor };
     
     XSetForeground(im->x.disp, mgc, 0);
     XFillRectangle(im->x.disp, p->mask, mgc, 0, 0, w * 3 , h);
@@ -1066,11 +1148,14 @@ void draw_paddle_rotate(ImlibData *im, xitk_image_t *p) {
 void draw_rotate_button(ImlibData *im, xitk_image_t *p) {
   GC            gc, mgc;
   XGCValues     gcv;
-  int           w = p->width;
-  int           h = p->height;
+  int           w;
+  int           h;
   
   assert(im && p);
   
+  w = p->width;
+  h = p->height;
+
   XLOCK(im->x.disp);
 
   gcv.graphics_exposures = False;
@@ -1103,6 +1188,62 @@ void draw_rotate_button(ImlibData *im, xitk_image_t *p) {
   XUNLOCK(im->x.disp);
 }
 
+/*
+ *
+ */
+void draw_button_plus(ImlibData *im, xitk_image_t *p) {
+  GC            gc;
+  XGCValues     gcv;
+  int           w, h;
+  
+  assert(im && p);
+
+  draw_button_minus(im, p);
+  
+  w = p->width / 3;
+  h = p->height;
+  
+  XLOCK(im->x.disp);
+
+  gcv.graphics_exposures = False;
+  gc = XCreateGC(im->x.disp, p->image, GCGraphicsExposures, &gcv);
+
+  XSetForeground(im->x.disp, gc, xitk_get_pixel_color_black(im));
+ 
+  XDrawLine(im->x.disp, p->image, gc, (w >> 1) - 1, 2, (w >> 1) - 1, h - 4);
+  XDrawLine(im->x.disp, p->image, gc, w + (w >> 1) - 1, 2, w + (w >> 1) - 1, h - 4);
+  XDrawLine(im->x.disp, p->image, gc, (w * 2) + (w >> 1), 3, (w * 2) + (w >> 1), h - 3);
+
+  XFreeGC(im->x.disp, gc);
+  XUNLOCK(im->x.disp);
+}
+
+/*
+ *
+ */
+void draw_button_minus(ImlibData *im, xitk_image_t *p) {
+  GC            gc;
+  XGCValues     gcv;
+  int           w, h;
+  
+  assert(im && p);
+  
+  w = p->width / 3;
+  h = p->height;
+  
+  XLOCK(im->x.disp);
+  gcv.graphics_exposures = False;
+  gc = XCreateGC(im->x.disp, p->image, GCGraphicsExposures, &gcv);
+
+  XSetForeground(im->x.disp, gc, xitk_get_pixel_color_black(im));
+ 
+  XDrawLine(im->x.disp, p->image, gc, 2, (h >> 1) - 1, w - 4, (h >> 1) - 1);
+  XDrawLine(im->x.disp, p->image, gc, w + 2, (h >> 1) - 1, (w * 2) - 4, (h >> 1) - 1);
+  XDrawLine(im->x.disp, p->image, gc, (w * 2) + 3, h >> 1, (w * 3) - 3, h >> 1);
+  
+  XFreeGC(im->x.disp, gc);
+  XUNLOCK(im->x.disp);
+}
 
 /*
  *
@@ -1148,9 +1289,10 @@ xitk_image_t *xitk_image_load_image(ImlibData *im, char *image) {
  *
  */
 static void notify_destroy(xitk_widget_t *w, void *data) {
-  image_private_data_t *private_data = (image_private_data_t *) w->private_data;
-
-  if(w->widget_type & WIDGET_TYPE_IMAGE) {
+  image_private_data_t *private_data;
+  
+  if(w && ((w->widget_type & WIDGET_TYPE_MASK) == WIDGET_TYPE_IMAGE)) {
+    private_data = (image_private_data_t *) w->private_data;
     XITK_FREE(private_data->skin_element_name);
     xitk_image_free_image(private_data->imlibdata, &private_data->skin);
     XITK_FREE(private_data);
@@ -1168,9 +1310,10 @@ static int notify_inside(xitk_widget_t *w, int x, int y) {
  *
  */
 static xitk_image_t *get_skin(xitk_widget_t *w, int sk) {
-  image_private_data_t *private_data = (image_private_data_t *) w->private_data;
+  image_private_data_t *private_data;
   
-  if(w->widget_type & WIDGET_TYPE_IMAGE) {
+  if(w && ((w->widget_type & WIDGET_TYPE_MASK) == WIDGET_TYPE_IMAGE)) {
+    private_data = (image_private_data_t *) w->private_data;
     if(sk == BACKGROUND_SKIN && private_data->skin) {
       return private_data->skin;
     }
@@ -1182,13 +1325,13 @@ static xitk_image_t *get_skin(xitk_widget_t *w, int sk) {
 /*
  *
  */
-static void paint_image (xitk_widget_t *i, Window win, GC gc) {
-  xitk_image_t *skin;
-  GC lgc;
-  image_private_data_t *private_data = 
-    (image_private_data_t *) i->private_data;
+static void paint_image (xitk_widget_t *w, Window win, GC gc) {
+  image_private_data_t *private_data;
+  xitk_image_t         *skin;
+  GC                    lgc;
   
-  if ((i->widget_type & WIDGET_TYPE_IMAGE) && (i->visible == 1)) {
+  if(w && (((w->widget_type & WIDGET_TYPE_MASK) == WIDGET_TYPE_IMAGE) && (w->visible == 1))) {
+    private_data = (image_private_data_t *) w->private_data;
 
     skin = private_data->skin;
     
@@ -1198,12 +1341,12 @@ static void paint_image (xitk_widget_t *i, Window win, GC gc) {
     XCopyGC(private_data->imlibdata->x.disp, gc, (1 << GCLastBit) - 1, lgc);
     
     if (skin->mask) {
-      XSetClipOrigin(private_data->imlibdata->x.disp, lgc, i->x, i->y);
+      XSetClipOrigin(private_data->imlibdata->x.disp, lgc, w->x, w->y);
       XSetClipMask(private_data->imlibdata->x.disp, lgc, skin->mask);
     }
     
     XCopyArea (private_data->imlibdata->x.disp, skin->image, win, lgc, 0, 0,
-	       skin->width, skin->height, i->x, i->y);
+	       skin->width, skin->height, w->x, w->y);
     
     XFreeGC(private_data->imlibdata->x.disp, lgc);
 
@@ -1215,12 +1358,13 @@ static void paint_image (xitk_widget_t *i, Window win, GC gc) {
 /*
  *
  */
-static void notify_change_skin(xitk_widget_list_t *wl, 
-			       xitk_widget_t *i, xitk_skin_config_t *skonfig) {
-  image_private_data_t *private_data = 
-    (image_private_data_t *) i->private_data;
+static void notify_change_skin(xitk_widget_list_t *wl,
+			       xitk_widget_t *w, xitk_skin_config_t *skonfig) {
+  image_private_data_t *private_data;
   
-  if ((i->widget_type & WIDGET_TYPE_IMAGE) && (i->visible == 1)) {
+  if(w && (((w->widget_type & WIDGET_TYPE_MASK) == WIDGET_TYPE_IMAGE) && (w->visible == 1))) {
+    private_data = (image_private_data_t *) w->private_data;
+
     if(private_data->skin_element_name) {
       
       xitk_skin_lock(skonfig);
@@ -1229,14 +1373,14 @@ static void notify_change_skin(xitk_widget_list_t *wl,
       private_data->skin = xitk_image_load_image(private_data->imlibdata,
 						 xitk_skin_get_skin_filename(skonfig, private_data->skin_element_name));
       
-      i->x               = xitk_skin_get_coord_x(skonfig, private_data->skin_element_name);
-      i->y               = xitk_skin_get_coord_y(skonfig, private_data->skin_element_name);
-      i->width           = private_data->skin->width;
-      i->height          = private_data->skin->height;
+      w->x               = xitk_skin_get_coord_x(skonfig, private_data->skin_element_name);
+      w->y               = xitk_skin_get_coord_y(skonfig, private_data->skin_element_name);
+      w->width           = private_data->skin->width;
+      w->height          = private_data->skin->height;
 
       xitk_skin_unlock(skonfig);
       
-      xitk_set_widget_pos(i, i->x, i->y);
+      xitk_set_widget_pos(w, w->x, w->y);
     }
   }
 }
@@ -1244,7 +1388,8 @@ static void notify_change_skin(xitk_widget_list_t *wl,
 /*
  *
  */
-static xitk_widget_t *_xitk_image_create (xitk_skin_config_t *skonfig, 
+static xitk_widget_t *_xitk_image_create (xitk_widget_list_t *wl,
+					  xitk_skin_config_t *skonfig, 
 					  xitk_image_widget_t *im,
 					  int x, int y,
 					  char *skin_element_name,
@@ -1264,6 +1409,8 @@ static xitk_widget_t *_xitk_image_create (xitk_skin_config_t *skonfig,
 
   mywidget->private_data          = private_data;
 
+  mywidget->widget_list           = wl;
+
   mywidget->enable                = 1;
   mywidget->running               = 1;
   mywidget->visible               = 1;
@@ -1289,11 +1436,12 @@ static xitk_widget_t *_xitk_image_create (xitk_skin_config_t *skonfig,
   return mywidget;
 }
 
-xitk_widget_t *xitk_image_create (xitk_skin_config_t *skonfig, xitk_image_widget_t *im) {
+xitk_widget_t *xitk_image_create (xitk_widget_list_t *wl,
+				  xitk_skin_config_t *skonfig, xitk_image_widget_t *im) {
 
   XITK_CHECK_CONSTITENCY(im);
 
-  return _xitk_image_create(skonfig, im, 
+  return _xitk_image_create(wl, skonfig, im, 
 			    (xitk_skin_get_coord_x(skonfig, im->skin_element_name)),
 			    (xitk_skin_get_coord_y(skonfig, im->skin_element_name)),
 			    im->skin_element_name,
@@ -1305,9 +1453,10 @@ xitk_widget_t *xitk_image_create (xitk_skin_config_t *skonfig, xitk_image_widget
 /*
  *
  */
-xitk_widget_t *xitk_noskin_image_create (xitk_image_widget_t *im, 
+xitk_widget_t *xitk_noskin_image_create (xitk_widget_list_t *wl,
+					 xitk_image_widget_t *im, 
 					 xitk_image_t *image, int x, int y) {
   XITK_CHECK_CONSTITENCY(im);
 
-  return _xitk_image_create(NULL, im, x, y, NULL, image);
+  return _xitk_image_create(wl, NULL, im, x, y, NULL, image);
 }
