@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: image.h,v 1.12 2002/03/21 00:22:53 f1rmb Exp $
+ * $Id: image.h,v 1.16 2002/07/17 20:29:31 f1rmb Exp $
  *
  */
 
@@ -69,6 +69,8 @@ unsigned int xitk_get_pixel_color_white(ImlibData *im);
 unsigned int xitk_get_pixel_color_lightgray(ImlibData *im);
 unsigned int xitk_get_pixel_color_gray(ImlibData *im);
 unsigned int xitk_get_pixel_color_darkgray(ImlibData *im);
+unsigned int xitk_get_pixel_color_warning_foreground(ImlibData *im);
+unsigned int xitk_get_pixel_color_warning_background(ImlibData *im);
 
 /*
  * Create an image object.
@@ -78,7 +80,8 @@ xitk_image_t *xitk_image_create_image(ImlibData *im, int width, int height);
 /*
  *
  */
-xitk_widget_t *xitk_noskin_image_create (xitk_image_widget_t *im, 
+xitk_widget_t *xitk_noskin_image_create (xitk_widget_list_t *wl,
+					 xitk_image_widget_t *im, 
 					 xitk_image_t *image, int x, int y);
 
 /*
@@ -110,7 +113,8 @@ xitk_image_t *xitk_image_load_image(ImlibData *idata, char *image);
 /**
  * Create an image widget type.
  */
-xitk_widget_t *xitk_image_create(xitk_skin_config_t *skonfig, xitk_image_widget_t *im);
+xitk_widget_t *xitk_image_create(xitk_widget_list_t *wl,
+				 xitk_skin_config_t *skonfig, xitk_image_widget_t *im);
 
 /*
  * Create and image object from a string, width is limited.
@@ -150,11 +154,13 @@ void draw_paddle_three_state_horizontal(ImlibData *im, xitk_image_t *p);
  * Draw an inner box.
  */
 void draw_inner(ImlibData *im, Pixmap p, int w, int h);
+void draw_inner_light(ImlibData *im, Pixmap p, int w, int h);
 
 /*
  * Draw and outter box.
  */
 void draw_outter(ImlibData *im, Pixmap p, int w, int h);
+void draw_outter_light(ImlibData *im, Pixmap p, int w, int h);
 
 /*
  * Draw a flat box.
@@ -209,4 +215,6 @@ void draw_tab(ImlibData *im, xitk_image_t *p);
 void draw_paddle_rotate(ImlibData *im, xitk_image_t *p);
 void draw_rotate_button(ImlibData *im, xitk_image_t *p);
 
+void draw_button_plus(ImlibData *im, xitk_image_t *p);
+void draw_button_minus(ImlibData *im, xitk_image_t *p);
 #endif
