@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: inputtext.c,v 1.27 2002/04/18 21:14:36 f1rmb Exp $
+ * $Id: inputtext.c,v 1.35 2002/07/31 00:50:41 f1rmb Exp $
  *
  */
 #ifdef HAVE_CONFIG_H
@@ -44,14 +44,17 @@
 #define NORMAL                  1
 #define FOCUS                   2
 
+unsigned int mods_used = (ShiftMask | ControlMask | Mod1Mask |
+			  Mod2Mask| Mod3Mask| Mod4Mask| Mod5Mask);
 /*
  *
  */
 static void notify_destroy(xitk_widget_t *w, void *data) {
-  inputtext_private_data_t *private_data = 
-    (inputtext_private_data_t *) w->private_data;
+  inputtext_private_data_t *private_data;
 
-  if(w->widget_type & WIDGET_TYPE_INPUTTEXT) {
+  if(w && ((w->widget_type & WIDGET_TYPE_MASK) == WIDGET_TYPE_INPUTTEXT)) {
+    private_data = (inputtext_private_data_t *) w->private_data;
+    
     XITK_FREE(private_data->skin_element_name);
     xitk_image_free_image(private_data->imlibdata, &private_data->skin);
     XITK_FREE(private_data->text);
@@ -66,30 +69,30 @@ static void notify_destroy(xitk_widget_t *w, void *data) {
  *
  */
 static xitk_image_t *get_skin(xitk_widget_t *w, int sk) {
-  inputtext_private_data_t *private_data = 
-    (inputtext_private_data_t *) w->private_data;
+  inputtext_private_data_t *private_data = (inputtext_private_data_t *) w->private_data;
   
-  if(w->widget_type & WIDGET_TYPE_INPUTTEXT) {
+  if(w && ((w->widget_type & WIDGET_TYPE_MASK) == WIDGET_TYPE_INPUTTEXT)) {
+    private_data = (inputtext_private_data_t *) w->private_data;
     if(sk == BACKGROUND_SKIN && private_data->skin) {
       return private_data->skin;
     }
   }
-
+  
   return NULL;
 }
 
 /*
  *
  */
-static int notify_inside(xitk_widget_t *it, int x, int y) {
-  inputtext_private_data_t *private_data = 
-    (inputtext_private_data_t *) it->private_data;
+static int notify_inside(xitk_widget_t *w, int x, int y) {
+  inputtext_private_data_t *private_data;
   
-  if(it->widget_type & WIDGET_TYPE_INPUTTEXT) {
-    if((it->visible == 1)) {
+  if(w && ((w->widget_type & WIDGET_TYPE_MASK) == WIDGET_TYPE_INPUTTEXT)) {
+    private_data = (inputtext_private_data_t *) w->private_data;
+    if((w->visible == 1)) {
       xitk_image_t *skin = private_data->skin;
       
-      return xitk_is_cursor_out_mask(private_data->imlibdata->x.disp, it, skin->mask, x, y);
+      return xitk_is_cursor_out_mask(private_data->imlibdata->x.disp, w, skin->mask, x, y);
     }
     else
       return 0;
@@ -102,57 +105,35 @@ static int notify_inside(xitk_widget_t *it, int x, int y) {
  * Extract modifier keys.
  */
 int xitk_get_key_modifier(XEvent *xev, int *modifier) {
-  
+  unsigned int state = 0;
+
   *modifier = MODIFIER_NOMOD;
 
   if(!xev)
     return 0;
 
-  /*
-  printf("%s(): ", __FUNCTION__);
-  if(xev->xbutton.state & ShiftMask)
-    printf("Shift ");
-  if(xev->xbutton.state & LockMask)
-    printf("CapsLock ");
-  if(xev->xbutton.state & ControlMask)
-    printf("Control ");
-  if(xev->xbutton.state & Mod1Mask)
-    printf("Meta(mod1) ");
-  if(xev->xbutton.state & Mod2Mask)
-    printf("NumLock(mod2) ");
-  if(xev->xbutton.state & Mod3Mask)
-    printf("Mod3 ");
-  if(xev->xbutton.state & Mod4Mask)
-    printf("Mod4 ");
-  if(xev->xbutton.state & Mod5Mask)
-    printf("Mod5 ");
-  printf(".\n");
-  */
-
-  if(xev->xbutton.state & ShiftMask)
+  if((xev->type == ButtonPress) || (xev->type == ButtonRelease))
+    state = xev->xbutton.state;
+  else if ((xev->type == KeyPress) || (xev->type == KeyRelease))
+    state = xev->xkey.state;
+  
+  if(state & ShiftMask)
     *modifier |= MODIFIER_SHIFT;
-
-  if(xev->xbutton.state & LockMask)
+  if(state & LockMask)
     *modifier |= MODIFIER_LOCK;
-
-  if(xev->xbutton.state & ControlMask)
+  if(state & ControlMask)
     *modifier |= MODIFIER_CTRL;
-
-  if(xev->xbutton.state & Mod1Mask)
+  if(state & Mod1Mask)
     *modifier |= MODIFIER_META;
-
-  if(xev->xbutton.state & Mod2Mask)
+  if(state & Mod2Mask)
     *modifier |= MODIFIER_NUML;
-
-  if(xev->xbutton.state & Mod3Mask)
+  if(state & Mod3Mask)
     *modifier |= MODIFIER_MOD3;
-
-  if(xev->xbutton.state & Mod4Mask)
+  if(state & Mod4Mask)
     *modifier |= MODIFIER_MOD4;
-
-  if(xev->xbutton.state & Mod5Mask)
+  if(state & Mod5Mask)
     *modifier |= MODIFIER_MOD5;
-
+  
   return (*modifier != MODIFIER_NOMOD);
 }
 
@@ -208,22 +189,20 @@ static void inputtext_recalc_offsets(xitk_widget_list_t *wl,
 /*
  * Return a pixmap with text drawed.
  */
-static void create_labelofinputtext(xitk_widget_t *it, 
+static void create_labelofinputtext(xitk_widget_t *w, 
 				    Window win, GC gc, Pixmap pix, 
-				    int xsize, int ysize, 
-				    char *label, int state) {
-  inputtext_private_data_t *private_data = 
-    (inputtext_private_data_t *) it->private_data;
-  char               *plabel = label;
-  char               *p = label;
-  xitk_font_t        *fs = NULL;
-  int                 lbear, rbear, width, asc, des;
-  int                 yoff = 0, DefaultColor = -1;
-  unsigned int        fg;
-  XColor              color;
-  xitk_color_names_t  *gColor = NULL;
+				    int xsize, int ysize, char *label, int state) {
+  inputtext_private_data_t *private_data = (inputtext_private_data_t *) w->private_data;
+  char                     *plabel = label;
+  char                     *p = label;
+  xitk_font_t              *fs = NULL;
+  int                       lbear, rbear, width, asc, des;
+  int                       yoff = 0, DefaultColor = -1;
+  unsigned int              fg = 0;
+  XColor                    xcolor;
+  xitk_color_names_t       *color = NULL;
 
-  color.flags = DoRed|DoBlue|DoGreen;
+  xcolor.flags = DoRed|DoBlue|DoGreen;
 
   /* Try to load font */
 
@@ -239,45 +218,51 @@ static void create_labelofinputtext(xitk_widget_t *it,
   xitk_font_set_font(fs, gc);
   xitk_font_string_extent(fs, label, &lbear, &rbear, &width, &asc, &des);
 
-  /*  Some colors configurations */
-  switch(state) {
-  case NORMAL:
-    if(!strcasecmp(private_data->normal_color, "Default")) {
-      DefaultColor = 0;
+  if((private_data->skin_element_name != NULL) ||
+     ((private_data->skin_element_name == NULL) && ((fg = xitk_get_black_color()) == -1))) {
+    
+    /*  Some colors configurations */
+    switch(state) {
+    case NORMAL:
+      if(!strcasecmp(private_data->normal_color, "Default")) {
+	DefaultColor = 0;
+      }
+      else {
+	color = xitk_get_color_name(private_data->normal_color);
+      }
+      break;
+      
+    case FOCUS:
+      if(!strcasecmp(private_data->focused_color, "Default")) {
+	DefaultColor = 0;
+      }
+      else {
+	color = xitk_get_color_name(private_data->focused_color);
+      }
+      break;
+      
+    }
+    
+    if(color == NULL || DefaultColor != -1) {
+      xcolor.red = xcolor.blue = xcolor.green = DefaultColor<<8;
     }
     else {
-      gColor = xitk_get_color_name(private_data->normal_color);
+      xcolor.red = color->red<<8; 
+      xcolor.blue = color->blue<<8;
+      xcolor.green = color->green<<8;
     }
-    break;
+    
+    XLOCK(private_data->imlibdata->x.disp);
+    XAllocColor(private_data->imlibdata->x.disp,
+		Imlib_get_colormap(private_data->imlibdata), &xcolor);
+    XUNLOCK(private_data->imlibdata->x.disp);
 
-  case FOCUS:
-    if(!strcasecmp(private_data->focused_color, "Default")) {
-      DefaultColor = 0;
-    }
-    else {
-      gColor = xitk_get_color_name(private_data->focused_color);
-    }
-    break;
+    fg = xcolor.pixel;
     
   }
   
-  if(gColor == NULL || DefaultColor != -1) {
-    color.red = color.blue = color.green = DefaultColor<<8;
-  }
-  else {
-    color.red = gColor->red<<8; 
-    color.blue = gColor->blue<<8;
-    color.green = gColor->green<<8;
-  }
-
   XLOCK(private_data->imlibdata->x.disp);
-
-  XAllocColor(private_data->imlibdata->x.disp,
-	      Imlib_get_colormap(private_data->imlibdata), &color);
-  fg = color.pixel;
-  
   XSetForeground(private_data->imlibdata->x.disp, gc, fg);
-
   XUNLOCK(private_data->imlibdata->x.disp);
 
   if(private_data->cursor_pos >= 0) {
@@ -312,10 +297,37 @@ static void create_labelofinputtext(xitk_widget_t *it,
 
   /*  Put text in the right place */
   XLOCK(private_data->imlibdata->x.disp);
+  if(private_data->skin_element_name) {
+    XDrawString(private_data->imlibdata->x.disp, pix, gc, 
+		2, ((ysize+asc+des+yoff)>>1)-des, 
+		plabel, strlen(plabel));
+  }
+  else {
+    XWindowAttributes attr;
+    Pixmap            tpix;
+    GC                lgc;
+    
+    XGetWindowAttributes(private_data->imlibdata->x.disp, win, &attr);
+    
+    lgc = XCreateGC(private_data->imlibdata->x.disp, win, None, None);
+    XCopyGC(private_data->imlibdata->x.disp, gc, (1 << GCLastBit) - 1, lgc);
+    
+    tpix = XCreatePixmap(private_data->imlibdata->x.disp, win,
+			 xsize, ysize, attr.depth);
+    
+    XCopyArea (private_data->imlibdata->x.disp, pix, tpix, lgc, 0, 0,
+	       xsize, ysize, 0, 0);
+    
+    XDrawString(private_data->imlibdata->x.disp, tpix, lgc, 
+		2, ((ysize+asc+des+yoff)>>1)-des, 
+		plabel, strlen(plabel));
+    
+    XCopyArea (private_data->imlibdata->x.disp, tpix, pix, lgc, 0, 0,
+	       xsize - 1, ysize, 0, 0);
 
-  XDrawString(private_data->imlibdata->x.disp, pix, gc, 
-	      2, ((ysize+asc+des+yoff)>>1)-des, 
-	      plabel, strlen(plabel));
+    XFreePixmap(private_data->imlibdata->x.disp, tpix);
+    XFreeGC(private_data->imlibdata->x.disp, lgc);
+  }
   XUNLOCK(private_data->imlibdata->x.disp);
 
   width = xitk_font_get_text_width(fs, plabel,
@@ -338,27 +350,25 @@ static void create_labelofinputtext(xitk_widget_t *it,
 
   xitk_font_unload_font(fs);
 
-  if(gColor) {
-    XITK_FREE(gColor->colorname);
-    XITK_FREE(gColor);
-  }
-
+  if(color)
+    xitk_free_color_name(color);
 }
 
 /*
  * Paint the input text box.
  */
-static void paint_inputtext(xitk_widget_t *it, Window win, GC gc) {
-  inputtext_private_data_t *private_data = 
-    (inputtext_private_data_t *) it->private_data;
-  int                 button_width, state = 0;
-  xitk_image_t        *skin;
-  Pixmap              btn;
-  GC                  lgc;
-  XWindowAttributes   attr;
+static void paint_inputtext(xitk_widget_t *w, Window win, GC gc) {
+  inputtext_private_data_t *private_data;
+  int                       button_width, state = 0;
+  xitk_image_t             *skin;
+  Pixmap                    btn;
+  GC                        lgc;
+  XWindowAttributes         attr;
 
-  if ((it->widget_type & WIDGET_TYPE_INPUTTEXT) && (it->visible == 1)) {
-        
+  if(w && (((w->widget_type & WIDGET_TYPE_MASK) == WIDGET_TYPE_INPUTTEXT) && (w->visible == 1))) {
+    
+    private_data = (inputtext_private_data_t *) w->private_data;
+    
     XLOCK(private_data->imlibdata->x.disp);
     XGetWindowAttributes(private_data->imlibdata->x.disp, win, &attr);
     
@@ -368,16 +378,16 @@ static void paint_inputtext(xitk_widget_t *it, Window win, GC gc) {
     XCopyGC(private_data->imlibdata->x.disp, gc, (1 << GCLastBit) - 1, lgc);
 
     if (skin->mask) {
-      XSetClipOrigin(private_data->imlibdata->x.disp, lgc, it->x, it->y);
+      XSetClipOrigin(private_data->imlibdata->x.disp, lgc, w->x, w->y);
       XSetClipMask(private_data->imlibdata->x.disp, lgc, skin->mask);
     }
 
-    button_width = skin->width/2;
+    button_width = skin->width / 2;
     
     btn = XCreatePixmap(private_data->imlibdata->x.disp, skin->image,
 			button_width, skin->height, attr.depth);
     
-    if(private_data->have_focus) {
+    if((w->have_focus == FOCUS_RECEIVED) || (private_data->have_focus == FOCUS_MOUSE_IN)) {
       state = FOCUS;
       XCopyArea (private_data->imlibdata->x.disp, skin->image,
 		 btn, gc, button_width, 0,
@@ -392,13 +402,12 @@ static void paint_inputtext(xitk_widget_t *it, Window win, GC gc) {
     
     XUNLOCK(private_data->imlibdata->x.disp);
     
-    create_labelofinputtext(it, win, gc, btn,
-			    button_width, skin->height, 
-			    private_data->text, state);
+    create_labelofinputtext(w, win, gc, btn, 
+			    button_width, skin->height, private_data->text, state);
     
     XLOCK(private_data->imlibdata->x.disp);
     XCopyArea (private_data->imlibdata->x.disp, btn, win, lgc, 0, 0,
-	       button_width, skin->height, it->x, it->y);
+	       button_width, skin->height, w->x, w->y);
     XFreePixmap(private_data->imlibdata->x.disp, btn);
     XFreeGC(private_data->imlibdata->x.disp, lgc);
     XUNLOCK(private_data->imlibdata->x.disp);
@@ -409,20 +418,20 @@ static void paint_inputtext(xitk_widget_t *it, Window win, GC gc) {
 /*
  * Handle click events.
  */
-static int notify_click_inputtext(xitk_widget_list_t *wl, xitk_widget_t *it, 
-				   int bUp, int x, int y) {
-  inputtext_private_data_t *private_data = 
-    (inputtext_private_data_t *) it->private_data;
+static int notify_click_inputtext(xitk_widget_list_t *wl, 
+				  xitk_widget_t *w, int bUp, int x, int y) {
+  inputtext_private_data_t *private_data;
   int pos;
   
-  if (it->widget_type & WIDGET_TYPE_INPUTTEXT) {
+  if(w && ((w->widget_type & WIDGET_TYPE_MASK) == WIDGET_TYPE_INPUTTEXT)) {
+    private_data = (inputtext_private_data_t *) w->private_data;
 
-    if(it->have_focus == FOCUS_LOST) {
-      it->have_focus = private_data->have_focus = FOCUS_RECEIVED;
+    if(w->have_focus == FOCUS_LOST) {
+      w->have_focus = private_data->have_focus = FOCUS_RECEIVED;
     }
 
-    pos = x - it->x;
-    
+    pos = x - w->x;
+
     {
       char        *p = private_data->text;
       xitk_font_t *fs = NULL;
@@ -447,7 +456,7 @@ static int notify_click_inputtext(xitk_widget_list_t *wl, xitk_widget_t *it,
 	i++;
       }
       xitk_font_unload_font(fs);
-      pos = (i-2) + private_data->disp_offset;
+      pos = (i - 2) + private_data->disp_offset;
     }
     
     if(private_data->text) {
@@ -459,7 +468,7 @@ static int notify_click_inputtext(xitk_widget_list_t *wl, xitk_widget_t *it,
 
     private_data->cursor_pos = (pos < 0) ? 0 : pos;
 
-    paint_inputtext(it, wl->win, wl->gc);
+    paint_inputtext(w, wl->win, wl->gc);
 
   }
 
@@ -469,20 +478,17 @@ static int notify_click_inputtext(xitk_widget_list_t *wl, xitk_widget_t *it,
 /*
  * Handle motion on input text box.
  */
-static int notify_focus_inputtext(xitk_widget_list_t *wl, 
-				   xitk_widget_t *it, int bEntered) {
-  inputtext_private_data_t *private_data = 
-    (inputtext_private_data_t *) it->private_data;
+static int notify_focus_inputtext(xitk_widget_list_t *wl, xitk_widget_t *w, int focus) {
+  inputtext_private_data_t *private_data;
   
-  if (it->widget_type & WIDGET_TYPE_INPUTTEXT) {
-    private_data->have_focus = bEntered;
-    if(bEntered == FOCUS_LOST) {
-      private_data->cursor_pos = 0;
-    }
-    paint_inputtext(it, wl->win, wl->gc);
-
+  if(w && ((w->widget_type & WIDGET_TYPE_MASK) == WIDGET_TYPE_INPUTTEXT)) {
+    private_data = (inputtext_private_data_t *) w->private_data;
+    
+    if((private_data->have_focus = focus) == FOCUS_LOST)
+      private_data->cursor_pos = -1;
+    
   }
-
+  
   return 1;
 }
 
@@ -490,11 +496,12 @@ static int notify_focus_inputtext(xitk_widget_list_t *wl,
  *
  */
 static void notify_change_skin(xitk_widget_list_t *wl, 
-			       xitk_widget_t *it, xitk_skin_config_t *skonfig) {
-  inputtext_private_data_t *private_data = 
-    (inputtext_private_data_t *) it->private_data;
+			       xitk_widget_t *w, xitk_skin_config_t *skonfig) {
+  inputtext_private_data_t *private_data;
   
-  if (it->widget_type & WIDGET_TYPE_INPUTTEXT) {
+  if(w && ((w->widget_type & WIDGET_TYPE_MASK) == WIDGET_TYPE_INPUTTEXT)) {
+    private_data = (inputtext_private_data_t *) w->private_data;
+
     if(private_data->skin_element_name) {
 
       xitk_skin_lock(skonfig);
@@ -513,16 +520,16 @@ static void notify_change_skin(xitk_widget_list_t *wl,
       XITK_FREE(private_data->focused_color);
       private_data->focused_color = strdup(xitk_skin_get_label_color_focus(skonfig, private_data->skin_element_name));
       
-      it->x                       = xitk_skin_get_coord_x(skonfig, private_data->skin_element_name);
-      it->y                       = xitk_skin_get_coord_y(skonfig, private_data->skin_element_name);
-      it->width                   = private_data->skin->width/2;
-      it->height                  = private_data->skin->height;
-      it->visible                 = (xitk_skin_get_visibility(skonfig, private_data->skin_element_name)) ? 1 : -1;
-      it->enable                  = xitk_skin_get_enability(skonfig, private_data->skin_element_name);
+      w->x                       = xitk_skin_get_coord_x(skonfig, private_data->skin_element_name);
+      w->y                       = xitk_skin_get_coord_y(skonfig, private_data->skin_element_name);
+      w->width                   = private_data->skin->width/2;
+      w->height                  = private_data->skin->height;
+      w->visible                 = (xitk_skin_get_visibility(skonfig, private_data->skin_element_name)) ? 1 : -1;
+      w->enable                  = xitk_skin_get_enability(skonfig, private_data->skin_element_name);
      
       xitk_skin_unlock(skonfig);
 
-      xitk_set_widget_pos(it, it->x, it->y);
+      xitk_set_widget_pos(w, w->x, w->y);
     }
   }
 }
@@ -530,12 +537,11 @@ static void notify_change_skin(xitk_widget_list_t *wl,
 /*
  * Erase one char from right of cursor.
  */
-static void inputtext_erase_with_delete(xitk_widget_list_t *wl, xitk_widget_t *it) {
-  inputtext_private_data_t *private_data = 
-    (inputtext_private_data_t *) it->private_data;
-  char *oldtext, *newtext;
-  char *p, *pp;
-  int offset;
+static void inputtext_erase_with_delete(xitk_widget_list_t *wl, xitk_widget_t *w) {
+  inputtext_private_data_t *private_data = (inputtext_private_data_t *) w->private_data;
+  char                      *oldtext, *newtext;
+  char                      *p, *pp;
+  int                        offset;
   
   if(strlen(private_data->text) > 0) {
     if((private_data->cursor_pos >= 0)
@@ -568,7 +574,7 @@ static void inputtext_erase_with_delete(xitk_widget_list_t *wl, xitk_widget_t *i
       XITK_FREE(private_data->text);
       private_data->text = strdup((newtext != NULL) ? newtext : "");
       
-      paint_inputtext(it, wl->win, wl->gc);
+      paint_inputtext(w, wl->win, wl->gc);
       
       XITK_FREE(oldtext);
       XITK_FREE(newtext);
@@ -576,19 +582,18 @@ static void inputtext_erase_with_delete(xitk_widget_list_t *wl, xitk_widget_t *i
   }
   else {
     sprintf(private_data->text, "%s", "");
-    paint_inputtext(it, wl->win, wl->gc);
+    paint_inputtext(w, wl->win, wl->gc);
   }
 }
 
 /*
  * Erase one char from left of cursor.
  */
-static void inputtext_erase_with_backspace(xitk_widget_list_t *wl, xitk_widget_t *it) {
-  inputtext_private_data_t *private_data = 
-    (inputtext_private_data_t *) it->private_data;
-  char *oldtext, *newtext;
-  char *p, *pp;
-  int offset;
+static void inputtext_erase_with_backspace(xitk_widget_list_t *wl, xitk_widget_t *w) {
+  inputtext_private_data_t *private_data = (inputtext_private_data_t *) w->private_data;
+  char                     *oldtext, *newtext;
+  char                     *p, *pp;
+  int                       offset;
   
   if(strlen(private_data->text) > 1) {
     if(private_data->cursor_pos > 0) {
@@ -621,7 +626,7 @@ static void inputtext_erase_with_backspace(xitk_widget_list_t *wl, xitk_widget_t
       XITK_FREE(private_data->text);
       private_data->text = strdup(newtext);
       
-      paint_inputtext(it, wl->win, wl->gc);
+      paint_inputtext(w, wl->win, wl->gc);
       
       XITK_FREE(oldtext);
       XITK_FREE(newtext);
@@ -630,17 +635,16 @@ static void inputtext_erase_with_backspace(xitk_widget_list_t *wl, xitk_widget_t
   else {
     sprintf(private_data->text, "%s", "");
     private_data->cursor_pos = 0;
-    paint_inputtext(it, wl->win, wl->gc);
+    paint_inputtext(w, wl->win, wl->gc);
   }
 }
 
 /*
  * Erase chars from cursor pos to EOL.
  */
-static void inputtext_kill_line(xitk_widget_list_t *wl, xitk_widget_t *it) {
-  inputtext_private_data_t *private_data = 
-    (inputtext_private_data_t *) it->private_data;
-  char *oldtext, *newtext;
+static void inputtext_kill_line(xitk_widget_list_t *wl, xitk_widget_t *w) {
+  inputtext_private_data_t *private_data = (inputtext_private_data_t *) w->private_data;
+  char                      *oldtext, *newtext;
   
   if(strlen(private_data->text) > 1) {
     if(private_data->cursor_pos >= 0) {
@@ -653,7 +657,7 @@ static void inputtext_kill_line(xitk_widget_list_t *wl, xitk_widget_t *it) {
       XITK_FREE(private_data->text);
       private_data->text = strdup(newtext);
       
-      paint_inputtext(it, wl->win, wl->gc);
+      paint_inputtext(w, wl->win, wl->gc);
       
       XITK_FREE(oldtext);
       XITK_FREE(newtext);
@@ -664,93 +668,89 @@ static void inputtext_kill_line(xitk_widget_list_t *wl, xitk_widget_t *it) {
 /*
  * Move cursor pos to left.
  */
-static void inputtext_move_left(xitk_widget_list_t *wl, xitk_widget_t *it) {
-  inputtext_private_data_t *private_data = 
-    (inputtext_private_data_t *) it->private_data;
+static void inputtext_move_left(xitk_widget_list_t *wl, xitk_widget_t *w) {
+  inputtext_private_data_t *private_data = (inputtext_private_data_t *) w->private_data;
   
   if(private_data->cursor_pos > 0) {
     private_data->cursor_pos--;
-    paint_inputtext(it, wl->win, wl->gc);
+    paint_inputtext(w, wl->win, wl->gc);
   }
 }
 
 /*
  * Move cursor pos to right.
  */
-static void inputtext_move_right(xitk_widget_list_t *wl, xitk_widget_t *it) {
-  inputtext_private_data_t *private_data = 
-    (inputtext_private_data_t *) it->private_data;
+static void inputtext_move_right(xitk_widget_list_t *wl, xitk_widget_t *w) {
+  inputtext_private_data_t *private_data = (inputtext_private_data_t *) w->private_data;
 
   if(private_data->cursor_pos < strlen(private_data->text)) {
     private_data->cursor_pos++;
-    paint_inputtext(it, wl->win, wl->gc);
+    paint_inputtext(w, wl->win, wl->gc);
   }
 }
 
 /*
  * Remove focus of widget, then call callback function.
  */
-static void inputtext_exec_return(xitk_widget_list_t *wl, xitk_widget_t *it) {
-  inputtext_private_data_t *private_data = 
-    (inputtext_private_data_t *) it->private_data;
+static void inputtext_exec_return(xitk_widget_list_t *wl, xitk_widget_t *w) {
+  inputtext_private_data_t *private_data = (inputtext_private_data_t *) w->private_data;
 
-  private_data->cursor_pos = -1;
-  it->have_focus = private_data->have_focus = FOCUS_LOST;
-  paint_inputtext(it, wl->win, wl->gc);
+  private_data->cursor_pos   = -1;
+  w->have_focus              = 
+    private_data->have_focus = FOCUS_LOST;
+  //  wl->widget_focused = NULL;
+  paint_inputtext(w, wl->win, wl->gc);
   
   if(strlen(private_data->text) > 0) {
     if(private_data->callback)
-      private_data->callback(it, private_data->userdata, private_data->text);
+      private_data->callback(w, private_data->userdata, private_data->text);
   }
 }
 
 /*
  * Remove focus of widget.
  */
-static void inputtext_exec_escape(xitk_widget_list_t *wl, xitk_widget_t *it) {
-  inputtext_private_data_t *private_data = 
-    (inputtext_private_data_t *) it->private_data;
+static void inputtext_exec_escape(xitk_widget_list_t *wl, xitk_widget_t *w) {
+  inputtext_private_data_t *private_data = (inputtext_private_data_t *) w->private_data;
   
   private_data->cursor_pos = -1;
-  it->have_focus = private_data->have_focus = FOCUS_LOST;
-  paint_inputtext(it, wl->win, wl->gc);
+  w->have_focus = private_data->have_focus = FOCUS_LOST;
+  wl->widget_focused = NULL;
+  paint_inputtext(w, wl->win, wl->gc);
 }
 
 /*
  *
  */
-static void inputtext_move_bol(xitk_widget_list_t *wl, xitk_widget_t *it) {
-  inputtext_private_data_t *private_data = 
-    (inputtext_private_data_t *) it->private_data;
+static void inputtext_move_bol(xitk_widget_list_t *wl, xitk_widget_t *w) {
+  inputtext_private_data_t *private_data = (inputtext_private_data_t *) w->private_data;
 
   if(private_data->text) {
     private_data->cursor_pos = private_data->disp_offset = 0;
     inputtext_recalc_offsets(wl, private_data);
-    paint_inputtext(it, wl->win, wl->gc);
+    paint_inputtext(w, wl->win, wl->gc);
   }
 }
 
 /*
  *
  */
-static void inputtext_move_eol(xitk_widget_list_t *wl, xitk_widget_t *it) {
-  inputtext_private_data_t *private_data = 
-    (inputtext_private_data_t *) it->private_data;
+static void inputtext_move_eol(xitk_widget_list_t *wl, xitk_widget_t *w) {
+  inputtext_private_data_t *private_data = (inputtext_private_data_t *) w->private_data;
 
   if(private_data->text) {
     private_data->cursor_pos = strlen(private_data->text);
     inputtext_recalc_offsets(wl, private_data);
     
-    paint_inputtext(it, wl->win, wl->gc);
+    paint_inputtext(w, wl->win, wl->gc);
   }
 }
 
 /*
  * Transpose two characters.
  */
-static void inputtext_transpose_chars(xitk_widget_list_t *wl, xitk_widget_t *it) {
-  inputtext_private_data_t *private_data = 
-    (inputtext_private_data_t *) it->private_data;
+static void inputtext_transpose_chars(xitk_widget_list_t *wl, xitk_widget_t *w) {
+  inputtext_private_data_t *private_data = (inputtext_private_data_t *) w->private_data;
   
   if(private_data->text && (strlen(private_data->text) >= 2)) {
     if((private_data->cursor_pos >= 2)) {
@@ -759,7 +759,7 @@ static void inputtext_transpose_chars(xitk_widget_list_t *wl, xitk_widget_t *it)
       private_data->text[private_data->cursor_pos - 2] = private_data->text[private_data->cursor_pos - 1];
       private_data->text[private_data->cursor_pos - 1] = c;
       
-      paint_inputtext(it, wl->win, wl->gc);
+      paint_inputtext(w, wl->win, wl->gc);
     }
   }
 }
@@ -767,18 +767,20 @@ static void inputtext_transpose_chars(xitk_widget_list_t *wl, xitk_widget_t *it)
 /*
  * Handle keyboard event in input text box.
  */
-static void notify_keyevent_inputtext(xitk_widget_list_t *wl,
-				      xitk_widget_t *it, XEvent *xev) {
-  inputtext_private_data_t *private_data = 
-    (inputtext_private_data_t *) it->private_data;
+static void notify_keyevent_inputtext(xitk_widget_list_t *wl, xitk_widget_t *w, XEvent *xev) {
+  inputtext_private_data_t *private_data;
   
-  if (it->widget_type & WIDGET_TYPE_INPUTTEXT) {
+  if(w && ((w->widget_type & WIDGET_TYPE_MASK) == WIDGET_TYPE_INPUTTEXT)) {
     XKeyEvent   keyevent = xev->xkey;
     KeySym      key;
     char        buf[256];
     int         len;
     int         modifier;
     
+    private_data = (inputtext_private_data_t *) w->private_data;
+
+    private_data->have_focus = FOCUS_RECEIVED;
+
     XLOCK(private_data->imlibdata->x.disp);
     len = XLookupString(&keyevent, buf, sizeof(buf), &key, NULL);
     XUNLOCK(private_data->imlibdata->x.disp);
@@ -801,7 +803,7 @@ static void notify_keyevent_inputtext(xitk_widget_list_t *wl,
       case XK_a:
       case XK_A:
 	if(modifier & MODIFIER_CTRL) {
-	  inputtext_move_bol(wl, it);
+	  inputtext_move_bol(wl, w);
 	}
 	break;
 	
@@ -809,7 +811,7 @@ static void notify_keyevent_inputtext(xitk_widget_list_t *wl,
       case XK_b:
       case XK_B:
 	if(modifier & MODIFIER_CTRL) {
-	  inputtext_move_left(wl, it);
+	  inputtext_move_left(wl, w);
 	}
 	break;
 	
@@ -817,7 +819,7 @@ static void notify_keyevent_inputtext(xitk_widget_list_t *wl,
       case XK_c:
       case XK_C:
 	if(modifier & MODIFIER_CTRL) {
-	  inputtext_exec_escape(wl, it);
+	  inputtext_exec_escape(wl, w);
 	}
 	break;
 	
@@ -825,7 +827,7 @@ static void notify_keyevent_inputtext(xitk_widget_list_t *wl,
       case XK_d:
       case XK_D:
 	if(modifier & MODIFIER_CTRL) {
-	  inputtext_erase_with_delete(wl, it);
+	  inputtext_erase_with_delete(wl, w);
 	}
 	break;
 	
@@ -833,7 +835,7 @@ static void notify_keyevent_inputtext(xitk_widget_list_t *wl,
       case XK_e:
       case XK_E:
 	if(modifier & MODIFIER_CTRL) {
-	  inputtext_move_eol(wl, it);
+	  inputtext_move_eol(wl, w);
 	}
 	break;
 
@@ -841,7 +843,7 @@ static void notify_keyevent_inputtext(xitk_widget_list_t *wl,
       case XK_f:
       case XK_F:
 	if(modifier & MODIFIER_CTRL) {
-	  inputtext_move_right(wl, it);
+	  inputtext_move_right(wl, w);
 	}
 	break;
 
@@ -849,7 +851,7 @@ static void notify_keyevent_inputtext(xitk_widget_list_t *wl,
       case XK_k:
       case XK_K:
 	if(modifier & MODIFIER_CTRL) {
-	  inputtext_kill_line(wl, it);
+	  inputtext_kill_line(wl, w);
 	}
 	break;
 
@@ -857,7 +859,7 @@ static void notify_keyevent_inputtext(xitk_widget_list_t *wl,
       case XK_m:
       case XK_M:
 	if(modifier & MODIFIER_CTRL) {
-	  inputtext_exec_return(wl, it);
+	  inputtext_exec_return(wl, w);
 	}
 	break;
 
@@ -865,41 +867,44 @@ static void notify_keyevent_inputtext(xitk_widget_list_t *wl,
       case XK_t:
       case XK_T:
 	if(modifier & MODIFIER_CTRL) {
-	  inputtext_transpose_chars(wl, it);
+	  inputtext_transpose_chars(wl, w);
 	}
 	break;
 
       case XK_question:
 	if(modifier & MODIFIER_CTRL) {
-	  inputtext_erase_with_backspace(wl, it);
+	  inputtext_erase_with_backspace(wl, w);
 	}
 	break;
 
       }
     }
+    else if(key == XK_Tab) {
+      return;
+    }
     else if(key == XK_Delete) {
-      inputtext_erase_with_delete(wl, it);
+      inputtext_erase_with_delete(wl, w);
     }
     else if(key == XK_BackSpace) {
-      inputtext_erase_with_backspace(wl, it);
+      inputtext_erase_with_backspace(wl, w);
     }
     else if(key == XK_Left) {
-      inputtext_move_left(wl, it);
+      inputtext_move_left(wl, w);
     }
     else if(key == XK_Right) {
-      inputtext_move_right(wl, it);
+      inputtext_move_right(wl, w);
     }
     else if(key == XK_Home) {
-      inputtext_move_bol(wl, it);
+      inputtext_move_bol(wl, w);
     }
     else if(key == XK_End) {
-      inputtext_move_eol(wl, it);
+      inputtext_move_eol(wl, w);
     }
     else if((key == XK_Return) || (key == XK_KP_Enter)) {
-      inputtext_exec_return(wl, it);
+      inputtext_exec_return(wl, w);
     }
     else if((key == XK_Escape) || (key == XK_Tab)) {
-      inputtext_exec_escape(wl, it);
+      inputtext_exec_escape(wl, w);
     }
     else if((buf[0] != 0) && (buf[1] == 0)) {
       char *oldtext;
@@ -919,18 +924,19 @@ static void notify_keyevent_inputtext(xitk_widget_list_t *wl,
 	private_data->text = (char *) xitk_xmalloc(strlen(oldtext) + 3);
 
 	sprintf(private_data->text, "%s", oldtext);
-	sprintf(&private_data->text[pos], "%c%s%c",
-		buf[0], &oldtext[pos], 0);
+	sprintf(&private_data->text[pos], "%c%s%c", buf[0], &oldtext[pos], 0);
 
 	private_data->cursor_pos++;
-	paint_inputtext(it, wl->win, wl->gc);
+	paint_inputtext(w, wl->win, wl->gc);
 	XITK_FREE(oldtext);
       }
       
     }
+#if 0
     else {
-      //      printf("got unhandled key = [%ld]\n", key);
+      printf("got unhandled key = [%ld]\n", key);
     }
+#endif
   }
 
 }
@@ -938,11 +944,11 @@ static void notify_keyevent_inputtext(xitk_widget_list_t *wl,
 /*
  * Return the text of widget.
  */
-char *xitk_inputtext_get_text(xitk_widget_t *it) {
-  inputtext_private_data_t *private_data = 
-    (inputtext_private_data_t *) it->private_data;
+char *xitk_inputtext_get_text(xitk_widget_t *w) {
+  inputtext_private_data_t *private_data;
   
-  if (it->widget_type & WIDGET_TYPE_INPUTTEXT) {
+  if(w && ((w->widget_type & WIDGET_TYPE_MASK) == WIDGET_TYPE_INPUTTEXT)) {
+    private_data = (inputtext_private_data_t *) w->private_data;
     return private_data->text;
   }
   
@@ -952,15 +958,16 @@ char *xitk_inputtext_get_text(xitk_widget_t *it) {
 /*
  * Change and redisplay the text of widget.
  */
-void xitk_inputtext_change_text(xitk_widget_list_t *wl, xitk_widget_t *it, char *text) {
-  inputtext_private_data_t *private_data = 
-    (inputtext_private_data_t *) it->private_data;
+void xitk_inputtext_change_text(xitk_widget_list_t *wl, xitk_widget_t *w, char *text) {
+  inputtext_private_data_t *private_data;
   
-  if (it->widget_type & WIDGET_TYPE_INPUTTEXT) {
+  if(w && ((w->widget_type & WIDGET_TYPE_MASK) == WIDGET_TYPE_INPUTTEXT)) {
+    private_data = (inputtext_private_data_t *) w->private_data;
     XITK_FREE(private_data->text);
-    private_data->text = strdup((text != NULL)?text:"");
+    private_data->text = strdup((text != NULL) ? text : "");
+    private_data->disp_offset = 0;
     private_data->cursor_pos = -1;
-    paint_inputtext(it, wl->win, wl->gc);
+    paint_inputtext(w, wl->win, wl->gc);
   }
 
 }
@@ -968,7 +975,8 @@ void xitk_inputtext_change_text(xitk_widget_list_t *wl, xitk_widget_t *it, char
 /*
  * Create input text box
  */
-static xitk_widget_t *_xitk_inputtext_create (xitk_skin_config_t *skonfig, 
+static xitk_widget_t *_xitk_inputtext_create (xitk_widget_list_t *wl,
+					      xitk_skin_config_t *skonfig, 
 					      xitk_inputtext_widget_t *it,
 					      int x, int y, char *skin_element_name,
 					      xitk_image_t *skin,
@@ -996,7 +1004,10 @@ static xitk_widget_t *_xitk_inputtext_create (xitk_skin_config_t *skonfig,
 
   private_data->skin              = skin;
 
-  private_data->max_visible       = (private_data->skin->width/2);
+  private_data->max_visible   = (private_data->skin->width/2);
+  if(private_data->skin_element_name == NULL)
+    private_data->max_visible -= 2;
+
   private_data->disp_offset       = 0;
   
   private_data->callback          = it->callback;
@@ -1010,6 +1021,8 @@ static xitk_widget_t *_xitk_inputtext_create (xitk_skin_config_t *skonfig,
 
   mywidget->private_data          = private_data;
 
+  mywidget->widget_list           = wl;
+
   mywidget->enable                = enable;
   mywidget->running               = 1;
   mywidget->visible               = visible;
@@ -1037,11 +1050,12 @@ static xitk_widget_t *_xitk_inputtext_create (xitk_skin_config_t *skonfig,
 }
 
 
-xitk_widget_t *xitk_inputtext_create (xitk_skin_config_t *skonfig, xitk_inputtext_widget_t *it) {
+xitk_widget_t *xitk_inputtext_create (xitk_widget_list_t *wl,
+				      xitk_skin_config_t *skonfig, xitk_inputtext_widget_t *it) {
 
   XITK_CHECK_CONSTITENCY(it);
 
-  return _xitk_inputtext_create (skonfig, it,
+  return _xitk_inputtext_create (wl, skonfig, it,
 				 (xitk_skin_get_coord_x(skonfig, it->skin_element_name)),
 				 (xitk_skin_get_coord_y(skonfig, it->skin_element_name)),
 				 it->skin_element_name,
@@ -1057,7 +1071,8 @@ xitk_widget_t *xitk_inputtext_create (xitk_skin_config_t *skonfig, xitk_inputtex
 /*
  *
  */
-xitk_widget_t *xitk_noskin_inputtext_create (xitk_inputtext_widget_t *it,
+xitk_widget_t *xitk_noskin_inputtext_create (xitk_widget_list_t *wl,
+					     xitk_inputtext_widget_t *it,
 					     int x, int y, int width, int height,
 					     char *ncolor, char *fcolor, char *fontname) {
   xitk_image_t *i;
@@ -1067,5 +1082,5 @@ xitk_widget_t *xitk_noskin_inputtext_create (xitk_inputtext_widget_t *it,
   draw_bevel_two_state(it->imlibdata, i);
   
 
-  return _xitk_inputtext_create(NULL, it, x, y, NULL, i, fontname, ncolor, fcolor, 1, 1);
+  return _xitk_inputtext_create(wl, NULL, it, x, y, NULL, i, fontname, ncolor, fcolor, 1, 1);
 }
