@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: intbox.c,v 1.9 2002/04/12 21:38:51 f1rmb Exp $
+ * $Id: intbox.c,v 1.16 2002/07/17 20:29:31 f1rmb Exp $
  *
  */
 #ifdef HAVE_CONFIG_H
@@ -31,18 +31,23 @@
 #include "button.h"
 #include "intbox.h"
 #include "font.h"
+#include "image.h"
 #include "_xitk.h"
 
+
 /*
  *
  */
 static void notify_destroy(xitk_widget_t *w, void *data) {
-  intbox_private_data_t *private_data = (intbox_private_data_t *) w->private_data;
+  intbox_private_data_t *private_data;
   
-  if((w->widget_type & WIDGET_TYPE_INTBOX)) {
+  if(w && (((w->widget_type & WIDGET_GROUP_MASK) & WIDGET_GROUP_INTBOX) &&
+	   (w->widget_type & WIDGET_GROUP_WIDGET))) {
+    
+    private_data = (intbox_private_data_t *) w->private_data;
+    
     XITK_FREE(private_data->skin_element_name);
     XITK_FREE(private_data);
-
   }
 }
 
@@ -50,10 +55,13 @@ static void notify_destroy(xitk_widget_t *w, void *data) {
  *
  */
 static void paint(xitk_widget_t *w, Window win, GC gc) {
-  intbox_private_data_t *private_data = (intbox_private_data_t *) w->private_data;
+  intbox_private_data_t *private_data;
   
-  if(w->widget_type & WIDGET_TYPE_INTBOX) {
-
+  if(w && (((w->widget_type & WIDGET_GROUP_MASK) & WIDGET_GROUP_INTBOX) &&
+	   (w->widget_type & WIDGET_GROUP_WIDGET))) {
+    
+    private_data = (intbox_private_data_t *) w->private_data;
+    
     if((w->visible == 1)) {
       int bx, ih, iw;
 
@@ -80,10 +88,13 @@ static void paint(xitk_widget_t *w, Window win, GC gc) {
  *
  */
 static void notify_change_skin(xitk_widget_list_t *wl, 
-			       xitk_widget_t *c, xitk_skin_config_t *skonfig) {
-  intbox_private_data_t *private_data = (intbox_private_data_t *) c->private_data;
+			       xitk_widget_t *w, xitk_skin_config_t *skonfig) {
+  intbox_private_data_t *private_data;
   
-  if(c->widget_type & WIDGET_TYPE_INTBOX) {
+  if(w && (((w->widget_type & WIDGET_GROUP_MASK) & WIDGET_GROUP_INTBOX) &&
+	   (w->widget_type & WIDGET_GROUP_WIDGET))) {
+    private_data = (intbox_private_data_t *) w->private_data;
+
     if(private_data->skin_element_name) {
       /*      
       int x, y;
@@ -108,10 +119,11 @@ static void notify_change_skin(xitk_widget_list_t *wl,
  *
  */
 static void intbox_change_value(xitk_widget_t *x, void *data, char *string) {
-  xitk_widget_t *w = (xitk_widget_t *)data;
+  xitk_widget_t         *w = (xitk_widget_t *)data;
   intbox_private_data_t *private_data;
-
-  if(w->widget_type & WIDGET_TYPE_INTBOX) {
+  
+  if(w && (((w->widget_type & WIDGET_GROUP_MASK) & WIDGET_GROUP_INTBOX) &&
+	   (w->widget_type & WIDGET_GROUP_WIDGET))) {
     char  buf[256];
     
     private_data = (intbox_private_data_t *)w->private_data;
@@ -132,9 +144,10 @@ static void intbox_change_value(xitk_widget_t *x, void *data, char *string) {
 void xitk_intbox_set_value(xitk_widget_t *w, int value) {
   intbox_private_data_t *private_data;
   
-  if(w->widget_type & WIDGET_TYPE_INTBOX) {
+  if(w && (((w->widget_type & WIDGET_GROUP_MASK) & WIDGET_GROUP_INTBOX) &&
+	   (w->widget_type & WIDGET_GROUP_WIDGET))) {
     char buf[256];
-
+    
     private_data = (intbox_private_data_t *) w->private_data;
 
     memset(&buf, 0, sizeof(buf));
@@ -151,7 +164,9 @@ void xitk_intbox_set_value(xitk_widget_t *w, int value) {
 int xitk_intbox_get_value(xitk_widget_t *w) {
   intbox_private_data_t *private_data;
 
-  if(w->widget_type & WIDGET_TYPE_INTBOX) {
+  if(w && (((w->widget_type & WIDGET_GROUP_MASK) & WIDGET_GROUP_INTBOX) &&
+	   (w->widget_type & WIDGET_GROUP_WIDGET))) {
+    
     private_data = (intbox_private_data_t *)w->private_data;
     return private_data->value;
   }
@@ -165,7 +180,9 @@ static void intbox_stepdown(xitk_widget_t *x, void *data) {
   xitk_widget_t *w = (xitk_widget_t *) data;
   intbox_private_data_t *private_data;
 
-  if(w->widget_type & WIDGET_TYPE_INTBOX) {
+  if(w && (((w->widget_type & WIDGET_GROUP_MASK) & WIDGET_GROUP_INTBOX) &&
+	   (w->widget_type & WIDGET_GROUP_WIDGET))) {
+    
     private_data = (intbox_private_data_t *)w->private_data;
     private_data->value -= private_data->step;
     xitk_intbox_set_value(w, private_data->value);
@@ -181,7 +198,9 @@ static void intbox_stepup(xitk_widget_t *x, void *data) {
   xitk_widget_t *w = (xitk_widget_t *) data;
   intbox_private_data_t *private_data;
   
-  if(w->widget_type & WIDGET_TYPE_INTBOX) {
+  if(w && (((w->widget_type & WIDGET_GROUP_MASK) & WIDGET_GROUP_INTBOX) &&
+	   (w->widget_type & WIDGET_GROUP_WIDGET))) {
+    
     private_data = (intbox_private_data_t *)w->private_data;
     private_data->value += private_data->step;
     xitk_intbox_set_value(w, private_data->value);
@@ -193,7 +212,8 @@ static void intbox_stepup(xitk_widget_t *x, void *data) {
 /*
  *
  */
-static xitk_widget_t *_xitk_intbox_create(xitk_skin_config_t *skonfig,
+static xitk_widget_t *_xitk_intbox_create(xitk_widget_list_t *wl,
+					  xitk_skin_config_t *skonfig,
 					  xitk_intbox_widget_t *ib, char *skin_element_name,
 					  xitk_widget_t *mywidget, 
 					  intbox_private_data_t *private_data,
@@ -210,6 +230,8 @@ static xitk_widget_t *_xitk_intbox_create(xitk_skin_config_t *skonfig,
 
   mywidget->private_data                 = private_data;
 
+  mywidget->widget_list                  = wl;
+
   mywidget->enable                       = enable;
   mywidget->running                      = 1;
   mywidget->visible                      = visible;
@@ -217,7 +239,7 @@ static xitk_widget_t *_xitk_intbox_create(xitk_skin_config_t *skonfig,
   
   mywidget->imlibdata                    = private_data->imlibdata;
 
-  mywidget->widget_type                  = WIDGET_TYPE_INTBOX | WIDGET_TYPE_GROUP;
+  mywidget->widget_type                  = WIDGET_GROUP | WIDGET_GROUP_WIDGET | WIDGET_GROUP_INTBOX;
   mywidget->paint                        = paint;
   mywidget->notify_click                 = NULL;
   mywidget->notify_focus                 = NULL;
@@ -236,11 +258,12 @@ static xitk_widget_t *_xitk_intbox_create(xitk_skin_config_t *skonfig,
 /*
  *
  */
-xitk_widget_t *xitk_noskin_intbox_create(xitk_intbox_widget_t *ib,
+xitk_widget_t *xitk_noskin_intbox_create(xitk_widget_list_t *wl,
+					 xitk_intbox_widget_t *ib,
 					 int x, int y, int width, int height, 
 					 xitk_widget_t **iw, xitk_widget_t **mw, xitk_widget_t **lw) {
   xitk_widget_t              *mywidget;
-  intbox_private_data_t       *private_data;
+  intbox_private_data_t      *private_data;
   xitk_button_widget_t        b;
   xitk_inputtext_widget_t     inp;
 
@@ -255,7 +278,8 @@ xitk_widget_t *xitk_noskin_intbox_create(xitk_intbox_widget_t *ib,
   
   /* Create inputtext and buttons (not skinable) */
   {
-    char buf[256];
+    char          buf[256];
+    xitk_image_t *wimage;
 
     memset(&buf, 0, sizeof(buf));
     snprintf(buf, 256, "%d", ib->value);
@@ -266,28 +290,43 @@ xitk_widget_t *xitk_noskin_intbox_create(xitk_intbox_widget_t *ib,
     inp.callback          = intbox_change_value;
     inp.userdata          = (void *)mywidget;
     xitk_list_append_content(ib->parent_wlist->l, 
-			     (private_data->input_widget = 
-			      xitk_noskin_inputtext_create(&inp,
-							   x, y, (width - 10), height,
-							   "Black", "Black", DEFAULT_FONT_10)));
+	     (private_data->input_widget = 
+	      xitk_noskin_inputtext_create(ib->parent_wlist, &inp,
+					   x, y, (width - 10), height,
+					   "Black", "Black", DEFAULT_FONT_10)));
+    private_data->input_widget->widget_type |= WIDGET_GROUP | WIDGET_GROUP_INTBOX;
     
     b.skin_element_name = NULL;
     b.callback          = intbox_stepup;
     b.userdata          = (void *)mywidget;
     xitk_list_append_content(ib->parent_wlist->l, 
-			     (private_data->more_widget = 
-			      xitk_noskin_button_create(&b,
-							(x + width) - (height>>1), y, 
-							(height>>1), (height>>1))));
+	     (private_data->more_widget = 
+	      xitk_noskin_button_create(ib->parent_wlist, &b,
+					(x + width) - (height>>1), y, 
+					(height>>1), (height>>1))));
+    private_data->more_widget->widget_type |= WIDGET_GROUP | WIDGET_GROUP_INTBOX;
+
     b.skin_element_name = NULL;
     b.callback          = intbox_stepdown;
     b.userdata          = (void *)mywidget;
     xitk_list_append_content(ib->parent_wlist->l, 
-			     (private_data->less_widget = 
-			      xitk_noskin_button_create(&b,
-						(x + width) - (height>>1), (y + (height>>1)),
-							(height>>1), (height>>1))));
+	     (private_data->less_widget = 
+	      xitk_noskin_button_create(ib->parent_wlist, &b,
+					(x + width) - (height>>1), (y + (height>>1)),
+					(height>>1), (height>>1))));
+    private_data->less_widget->widget_type |= WIDGET_GROUP | WIDGET_GROUP_INTBOX;
+
+    /* Draw '+' and '-' in buttons */
+    wimage = xitk_get_widget_foreground_skin(private_data->more_widget);
     
+    if(wimage)
+      draw_button_plus(ib->imlibdata, wimage);
+
+    wimage = xitk_get_widget_foreground_skin(private_data->less_widget);
+    
+    if(wimage)
+      draw_button_minus(ib->imlibdata, wimage);
+
   }
 
   if(iw)
@@ -302,5 +341,5 @@ xitk_widget_t *xitk_noskin_intbox_create(xitk_intbox_widget_t *ib,
   mywidget->width = width;
   mywidget->height = height;
   
-  return _xitk_intbox_create(NULL, ib, NULL, mywidget, private_data, 1, 1);
+  return _xitk_intbox_create(wl, NULL, ib, NULL, mywidget, private_data, 1, 1);
 }
