@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: intbox.h,v 1.3 2002/03/21 00:22:53 f1rmb Exp $
+ * $Id: intbox.h,v 1.4 2002/07/17 20:29:31 f1rmb Exp $
  *
  */
 #ifndef HAVE_XITK_INTBOX_H
@@ -53,7 +53,8 @@ typedef struct {
 /*
  * Same as above, without skinable feature.
  */
-xitk_widget_t *xitk_noskin_intbox_create(xitk_intbox_widget_t *ib,
+xitk_widget_t *xitk_noskin_intbox_create(xitk_widget_list_t *wl,
+					 xitk_intbox_widget_t *ib,
 					 int x, int y, int width, int height, 
 					 xitk_widget_t **iw, xitk_widget_t **mw, xitk_widget_t **lw);
 
