@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: mrlbrowser.c,v 1.33 2002/04/21 22:17:12 f1rmb Exp $
+ * $Id: mrlbrowser.c,v 1.47 2002/07/17 20:29:32 f1rmb Exp $
  *
  */
 #ifdef HAVE_CONFIG_H
@@ -45,17 +45,46 @@
 #include "label.h"
 #include "button.h"
 #include "labelbutton.h"
+#include "inputtext.h"
 #include "dnd.h"
 #include "window.h"
+#include "combo.h"
 #include "widget.h"
 #include "widget_types.h"
 
+#undef DEBUG_MRLB
+
+typedef struct {
+  char *name;
+  char *ending;
+} mrl_filter_t;
+
+static mrl_filter_t mrl_filters[] = {
+  { "All"                 , "*"            },
+  { "*.vob"               , "vob"          }, /* mpeg block */
+  { "*.mpv"               , "mpv"          }, /* elementary */
+  { "*.mpg, *.mpeg, *.mpe", "mpg,mpeg,mpe" }, /* mpeg */
+  { "*.avi"               , "avi"          }, /* avi */
+  { "*.mp3"               , "mp3"          }, /* mp3 */
+  { "*.asf, *.wmv"        , "asf,wmv"      }, /* asf */
+  { "*.cpk, *.cak, *.film", "cpk,cak,film" }, /* film */
+  { "*.ogg"               , "ogg"          }, /* ogg */
+  { "*.vdr"               , "vdr"          }, /* pes */
+  { "*.mov, *.mp4, *.qt"  , "mov,mp4,qt"   }, /* QT */
+  { "*.roq"               , "roq"          }, /* roq */
+  { "*.m2t, *.ts, *.trp"  , "m2t,ts,trp"   }, /* ts */
+  { NULL                  , NULL           }
+};
+
+
 /*
  *
  */
 static void notify_destroy(xitk_widget_t *w, void *data) {
 
-  if(w->widget_type & WIDGET_TYPE_MRLBROWSER) {
+  if(w && (((w->widget_type & WIDGET_GROUP_MASK) & WIDGET_GROUP_MRLBROWSER) &&
+	   (w->widget_type & WIDGET_GROUP_WIDGET))) {
+    
     xitk_mrlbrowser_destroy(w);
   }
 }
@@ -98,37 +127,113 @@ static int get_mrl_marker(mrl_t *mrl) {
   return '\0';
 }
 
+/*
+ *
+ */
+static void mrlbrowser_filter_mrls(mrlbrowser_private_data_t *private_data) {
+  int i;
+  int old_filtered_mrls_num = private_data->mc->mrls_to_disp;
+
+  /* filtering is enabled */
+  if(private_data->filter_selected) {
+    char *filter_ends;
+    char *p, *m;
+    char *ending;
+
+    private_data->mc->mrls_to_disp = 0;
+
+    xitk_strdupa(filter_ends, mrl_filters[private_data->filter_selected].ending);
+    p = filter_ends;
+
+    for(i = 0; i < private_data->mrls_num; i++) {
+
+      if(private_data->mc->mrls[i]->type & mrl_file_directory) {
+
+	if(private_data->mc->filtered_mrls[private_data->mc->mrls_to_disp] == NULL)
+	  private_data->mc->filtered_mrls[private_data->mc->mrls_to_disp] = (mrl_t *) xitk_xmalloc(sizeof(mrl_t));
+
+	MRL_DUPLICATE(private_data->mc->mrls[i], private_data->mc->filtered_mrls[private_data->mc->mrls_to_disp]);
+	private_data->mc->mrls_to_disp++;
+      }
+      else {
+	ending = strrchr(private_data->mc->mrls[i]->mrl, '.');
+	
+	if(ending) {
+	  while((m = xitk_strsep(&filter_ends, ",")) != NULL) {
+	    
+	    while(*m == ' ' || *m == '\t') m++;
+	    
+	    if(!strcasecmp((ending + 1), m)) {
+
+	      if(private_data->mc->filtered_mrls[private_data->mc->mrls_to_disp] == NULL)
+		private_data->mc->filtered_mrls[private_data->mc->mrls_to_disp] = (mrl_t *) xitk_xmalloc(sizeof(mrl_t));
+	      
+	      MRL_DUPLICATE(private_data->mc->mrls[i], private_data->mc->filtered_mrls[private_data->mc->mrls_to_disp]);
+	      private_data->mc->mrls_to_disp++;
+	    }
+	  }
+	  
+	  filter_ends = p;
+	}
+      }    
+    }
+  }
+  else { /* no filtering */
+
+    for(i = 0; i < private_data->mrls_num; i++) {
+      
+      if(private_data->mc->filtered_mrls[i] == NULL)
+	private_data->mc->filtered_mrls[i] = (mrl_t *) xitk_xmalloc(sizeof(mrl_t));
+      
+      MRL_DUPLICATE(private_data->mc->mrls[i], private_data->mc->filtered_mrls[i]);
+    }
+    private_data->mc->mrls_to_disp = i;
+  }
+  
+  /* free unused mrls */
+  while(old_filtered_mrls_num > private_data->mc->mrls_to_disp) {
+    MRL_ZERO(private_data->mc->filtered_mrls[old_filtered_mrls_num - 1]);
+    XITK_FREE(private_data->mc->filtered_mrls[old_filtered_mrls_num - 1]);
+    private_data->mc->filtered_mrls[old_filtered_mrls_num - 1] = NULL;
+    old_filtered_mrls_num--;
+  }
+
+  private_data->mc->filtered_mrls[private_data->mc->mrls_to_disp] = NULL;
+}
+
 /*
  * Create *eye candy* entries in browser.
  */
 static void mrlbrowser_create_enlighted_entries(mrlbrowser_private_data_t *private_data) {
   int i = 0;
   
-  while(private_data->mc->mrls[i]) {
-    char *p = private_data->mc->mrls[i]->mrl;
+  mrlbrowser_filter_mrls(private_data);
 
-    if(private_data->mc->mrls[i]->origin) {
+  while(private_data->mc->filtered_mrls[i]) {
+    char *p = private_data->mc->filtered_mrls[i]->mrl;
 
-      p += strlen(private_data->mc->mrls[i]->origin);
+    if(private_data->mc->filtered_mrls[i]->origin) {
+
+      p += strlen(private_data->mc->filtered_mrls[i]->origin);
       
       if(*p == '/') 
 	p++;
     }
     
-    if(private_data->mc->mrls[i]->type & mrl_file_symlink) {
+    if(private_data->mc->filtered_mrls[i]->type & mrl_file_symlink) {
 
       if(private_data->mc->mrls_disp[i])
 	private_data->mc->mrls_disp[i] = (char *) 
 	  realloc(private_data->mc->mrls_disp[i], strlen(p) + 4 + 
-		  strlen(private_data->mc->mrls[i]->link) + 2);
+		  strlen(private_data->mc->filtered_mrls[i]->link) + 2);
       else
 	private_data->mc->mrls_disp[i] = (char *) 
 	  xitk_xmalloc(strlen(p) + 4 + 
-		      strlen(private_data->mc->mrls[i]->link) + 2);
+		      strlen(private_data->mc->filtered_mrls[i]->link) + 2);
       
       sprintf(private_data->mc->mrls_disp[i], "%s%c -> %s", 
-	      p, get_mrl_marker(private_data->mc->mrls[i]), 
-	      private_data->mc->mrls[i]->link);
+	      p, get_mrl_marker(private_data->mc->filtered_mrls[i]), 
+	      private_data->mc->filtered_mrls[i]->link);
 
     }
     else {
@@ -136,10 +241,10 @@ static void mrlbrowser_create_enlighted_entries(mrlbrowser_private_data_t *priva
 	private_data->mc->mrls_disp[i] = (char *) 
 	  realloc(private_data->mc->mrls_disp[i], strlen(p) + 2);
       else
-      private_data->mc->mrls_disp[i] = (char *) xitk_xmalloc(strlen(p) + 2);
+	private_data->mc->mrls_disp[i] = (char *) xitk_xmalloc(strlen(p) + 2);
       
       sprintf(private_data->mc->mrls_disp[i], "%s%c", 
-	      p, get_mrl_marker(private_data->mc->mrls[i]));
+	      p, get_mrl_marker(private_data->mc->filtered_mrls[i]));
     }
 
     i++;
@@ -155,7 +260,7 @@ static void mrlbrowser_duplicate_mrls(mrlbrowser_private_data_t *private_data,
   int i;
   int old_mrls_num = private_data->mrls_num;
 
-  for (i=0; i<num_mrls; i++) {
+  for (i = 0; i < num_mrls; i++) {
 
     if(private_data->mc->mrls[i] == NULL)
       private_data->mc->mrls[i] = (mrl_t *) xitk_xmalloc(sizeof(mrl_t));
@@ -173,6 +278,7 @@ static void mrlbrowser_duplicate_mrls(mrlbrowser_private_data_t *private_data,
   }
 
   private_data->mc->mrls[private_data->mrls_num] = NULL;
+
 }
 
 /*
@@ -214,10 +320,11 @@ static void mrlbrowser_grab_mrls(xitk_widget_t *w, void *data) {
     mrlbrowser_create_enlighted_entries(private_data);
     xitk_browser_update_list(private_data->mrlb_list, 
 			     private_data->mc->mrls_disp, 
-			     private_data->mrls_num, 0);
+			     private_data->mc->mrls_to_disp, 0);
   }
 }
 
+#ifdef DEBUG_MRLB
 /*
  * Dump informations, on terminal, about selected mrl.
  */
@@ -286,6 +393,7 @@ static void mrlbrowser_dumpmrl(xitk_widget_t *w, void *data) {
       printf("] (%Ld byte%c)\n", ms->size, (ms->size > 0) ?'s':'\0');
   }
 }
+#endif
 /*
  *                                 END OF PRIVATES
  *****************************************************************************/
@@ -296,7 +404,9 @@ static void mrlbrowser_dumpmrl(xitk_widget_t *w, void *data) {
 void xitk_mrlbrowser_set_tips_timeout(xitk_widget_t *w, int enabled, unsigned long timeout) {
   mrlbrowser_private_data_t *private_data;
   
-  if(w && (w->widget_type & WIDGET_TYPE_MRLBROWSER)) {
+  if(w && (((w->widget_type & WIDGET_GROUP_MASK) & WIDGET_GROUP_MRLBROWSER) &&
+	   (w->widget_type & WIDGET_GROUP_WIDGET))) {
+
     private_data = (mrlbrowser_private_data_t *)w->private_data;
     
     if(enabled)
@@ -312,7 +422,9 @@ void xitk_mrlbrowser_set_tips_timeout(xitk_widget_t *w, int enabled, unsigned lo
 Window xitk_mrlbrowser_get_window_id(xitk_widget_t *w) {
   mrlbrowser_private_data_t *private_data;
   
-  if(w && (w->widget_type & WIDGET_TYPE_MRLBROWSER)) {
+  if(w && (((w->widget_type & WIDGET_GROUP_MASK) & WIDGET_GROUP_MRLBROWSER) &&
+	   (w->widget_type & WIDGET_GROUP_WIDGET))) {
+
     private_data = (mrlbrowser_private_data_t *)w->private_data;
     return private_data->window;
   }
@@ -327,7 +439,9 @@ Window xitk_mrlbrowser_get_window_id(xitk_widget_t *w) {
 int xitk_mrlbrowser_get_window_info(xitk_widget_t *w, window_info_t *inf) {
   mrlbrowser_private_data_t *private_data;
 
-  if(w && (w->widget_type & WIDGET_TYPE_MRLBROWSER)) {
+  if(w && (((w->widget_type & WIDGET_GROUP_MASK) & WIDGET_GROUP_MRLBROWSER) &&
+	   (w->widget_type & WIDGET_GROUP_WIDGET))) {
+
     private_data = (mrlbrowser_private_data_t *)w->private_data;
     
     return((xitk_get_window_info(private_data->widget_key, inf))); 
@@ -342,7 +456,9 @@ int xitk_mrlbrowser_get_window_info(xitk_widget_t *w, window_info_t *inf) {
 int xitk_mrlbrowser_is_running(xitk_widget_t *w) {
   mrlbrowser_private_data_t *private_data;
  
-  if(w && (w->widget_type & WIDGET_TYPE_MRLBROWSER)) {
+  if(w && (((w->widget_type & WIDGET_GROUP_MASK) & WIDGET_GROUP_MRLBROWSER) &&
+	   (w->widget_type & WIDGET_GROUP_WIDGET))) {
+
     private_data = (mrlbrowser_private_data_t *)w->private_data;
     return (private_data->running);
   }
@@ -356,7 +472,9 @@ int xitk_mrlbrowser_is_running(xitk_widget_t *w) {
 int xitk_mrlbrowser_is_visible(xitk_widget_t *w) {
   mrlbrowser_private_data_t *private_data;
 
-  if(w && (w->widget_type & WIDGET_TYPE_MRLBROWSER)) {
+  if(w && (((w->widget_type & WIDGET_GROUP_MASK) & WIDGET_GROUP_MRLBROWSER) &&
+	   (w->widget_type & WIDGET_GROUP_WIDGET))) {
+
     private_data = (mrlbrowser_private_data_t *)w->private_data;
     return (private_data->visible);
   }
@@ -370,7 +488,9 @@ int xitk_mrlbrowser_is_visible(xitk_widget_t *w) {
 void xitk_mrlbrowser_hide(xitk_widget_t *w) {
   mrlbrowser_private_data_t *private_data;
 
-  if(w && (w->widget_type & WIDGET_TYPE_MRLBROWSER)) {
+  if(w && (((w->widget_type & WIDGET_GROUP_MASK) & WIDGET_GROUP_MRLBROWSER) &&
+	   (w->widget_type & WIDGET_GROUP_WIDGET))) {
+
     private_data = (mrlbrowser_private_data_t *)w->private_data;
 
     if(private_data->visible) {
@@ -391,7 +511,9 @@ void xitk_mrlbrowser_hide(xitk_widget_t *w) {
 void xitk_mrlbrowser_show(xitk_widget_t *w) {
   mrlbrowser_private_data_t *private_data;
 
-  if(w && (w->widget_type & WIDGET_TYPE_MRLBROWSER)) {
+  if(w && (((w->widget_type & WIDGET_GROUP_MASK) & WIDGET_GROUP_MRLBROWSER) &&
+	   (w->widget_type & WIDGET_GROUP_WIDGET))) {
+
     private_data = (mrlbrowser_private_data_t *)w->private_data;
 
     xitk_show_widgets(private_data->widget_list);
@@ -410,7 +532,8 @@ void xitk_mrlbrowser_show(xitk_widget_t *w) {
 void xitk_mrlbrowser_set_transient(xitk_widget_t *w, Window window) {
   mrlbrowser_private_data_t *private_data;
 
-  if(w && (w->widget_type & WIDGET_TYPE_MRLBROWSER) && (window != None)) {
+  if(w && (((w->widget_type & WIDGET_GROUP_MASK) & WIDGET_GROUP_MRLBROWSER) &&
+	   (w->widget_type & WIDGET_GROUP_WIDGET)) && (window != None)) {
     private_data = (mrlbrowser_private_data_t *)w->private_data;
 
     if(private_data->visible) {
@@ -429,7 +552,8 @@ void xitk_mrlbrowser_set_transient(xitk_widget_t *w, Window window) {
 void xitk_mrlbrowser_destroy(xitk_widget_t *w) {
   mrlbrowser_private_data_t *private_data;
 
-  if(w && (w->widget_type & WIDGET_TYPE_MRLBROWSER)) {
+  if(w && (((w->widget_type & WIDGET_GROUP_MASK) & WIDGET_GROUP_MRLBROWSER) && 
+	   (w->widget_type & WIDGET_GROUP_WIDGET))) {
     private_data = (mrlbrowser_private_data_t *)w->private_data;
 
     private_data->running = 0;
@@ -462,8 +586,15 @@ void xitk_mrlbrowser_destroy(xitk_widget_t *w) {
       for(i = private_data->mrls_num; i > 0; i--) {
 	MRL_ZERO(private_data->mc->mrls[i]);
 	XITK_FREE(private_data->mc->mrls[i]);
+	MRL_ZERO(private_data->mc->filtered_mrls[i]);
+	XITK_FREE(private_data->mc->filtered_mrls[i]);
 	XITK_FREE(private_data->mc->mrls_disp[i]);
       }
+      
+      for(i = 0; private_data->filters[i] != NULL; i++)
+	free(private_data->filters[i]);
+
+      free(private_data->filters);
     }
     
     XITK_FREE(private_data->skin_element_name);
@@ -495,7 +626,8 @@ void xitk_mrlbrowser_change_skins(xitk_widget_t *w, xitk_skin_config_t *skonfig)
   mrlbrowser_private_data_t *private_data;
   XSizeHints                 hint;
 
-  if(w && (w->widget_type & WIDGET_TYPE_MRLBROWSER)) {
+  if(w && (((w->widget_type & WIDGET_GROUP_MASK) & WIDGET_GROUP_MRLBROWSER) &&
+	   (w->widget_type & WIDGET_GROUP_WIDGET))) {
     private_data = (mrlbrowser_private_data_t *)w->private_data;
     
     xitk_skin_lock(skonfig);
@@ -564,8 +696,8 @@ void xitk_mrlbrowser_change_skins(xitk_widget_t *w, xitk_skin_config_t *skonfig)
  * 
  */
 static void mrlbrowser_select_mrl(mrlbrowser_private_data_t *private_data,
-				  int j, int do_callback) {
-  mrl_t *ms = private_data->mc->mrls[j];
+				  int j, int add_callback, int play_callback) {
+  mrl_t *ms = private_data->mc->filtered_mrls[j];
   char   buf[XITK_PATH_MAX + XITK_NAME_MAX + 1];
   
   memset(&buf, '\0', sizeof(buf));
@@ -622,16 +754,20 @@ static void mrlbrowser_select_mrl(mrlbrowser_private_data_t *private_data,
     mrlbrowser_create_enlighted_entries(private_data);
     xitk_browser_update_list(private_data->mrlb_list, 
 			     private_data->mc->mrls_disp, 
-			     private_data->mrls_num, 0);
+			     private_data->mc->mrls_to_disp, 0);
     
   }
   else {
     
     xitk_browser_release_all_buttons(private_data->mrlb_list);
-    if(do_callback && private_data->add_callback)
-      private_data->add_callback(NULL, (void *) j, 
-				 private_data->mc->mrls[j]);
 
+    if(add_callback && private_data->add_callback)
+      private_data->add_callback(NULL, (void *) j, 
+				 private_data->mc->filtered_mrls[j]);
+    
+    if(play_callback && private_data->play_callback)
+      private_data->play_callback(NULL, (void *) j,
+				  private_data->mc->filtered_mrls[j]);
   }
 }
 
@@ -643,7 +779,7 @@ static void mrlbrowser_select(xitk_widget_t *w, void *data) {
   int j = -1;
 
   if((j = xitk_browser_get_current_selected(private_data->mrlb_list)) >= 0) {
-    mrlbrowser_select_mrl(private_data, j, 1);
+    mrlbrowser_select_mrl(private_data, j, 1, 0);
   }
 }
 
@@ -651,15 +787,11 @@ static void mrlbrowser_select(xitk_widget_t *w, void *data) {
  * Handle selection in mrlbrowser, then 
  */
 static void mrlbrowser_play(xitk_widget_t *w, void *data) {
-
   mrlbrowser_private_data_t *private_data = (mrlbrowser_private_data_t *)data;
   int j = -1;
   
-  if(private_data->play_callback && 
-     ((j = xitk_browser_get_current_selected(private_data->mrlb_list)) >= 0)) {
-    
-    mrlbrowser_select_mrl(private_data, j, 0);
-    private_data->play_callback(NULL, (void *) j, private_data->mc->mrls[j]);
+  if((j = xitk_browser_get_current_selected(private_data->mrlb_list)) >= 0) {
+    mrlbrowser_select_mrl(private_data, j, 0, 1);
   }
 }
 
@@ -668,8 +800,34 @@ static void mrlbrowser_play(xitk_widget_t *w, void *data) {
  */
 static void handle_dbl_click(xitk_widget_t *w, void *data, int selected) {
   mrlbrowser_private_data_t *private_data = (mrlbrowser_private_data_t *)data;
+  XEvent                     xev;
+  int                        modifier;
+  
+  XLOCK(private_data->imlibdata->x.disp);
+  XPeekEvent(private_data->imlibdata->x.disp, &xev);
+  XUNLOCK(private_data->imlibdata->x.disp);
 
-  mrlbrowser_select_mrl(private_data, selected, 1);
+  xitk_get_key_modifier(&xev, &modifier);
+
+  if((modifier & MODIFIER_CTRL) && (modifier & MODIFIER_SHIFT))
+    mrlbrowser_select_mrl(private_data, selected, 1, 1);
+  else if(modifier & MODIFIER_CTRL)
+    mrlbrowser_select_mrl(private_data, selected, 1, 0);
+  else
+    mrlbrowser_select_mrl(private_data, selected, 0, 1); 
+}
+
+/*
+ * Refresh filtered list
+ */
+static void combo_filter_select(xitk_widget_t *w, void *data, int select) {
+  mrlbrowser_private_data_t *private_data = (mrlbrowser_private_data_t *)data;
+
+  private_data->filter_selected = select;
+  mrlbrowser_create_enlighted_entries(private_data);
+  xitk_browser_update_list(private_data->mrlb_list, 
+  			   private_data->mc->mrls_disp, 
+  			   private_data->mc->mrls_to_disp, 0);
 }
 
 /*
@@ -677,21 +835,16 @@ static void handle_dbl_click(xitk_widget_t *w, void *data, int selected) {
  */
 static void mrlbrowser_handle_event(XEvent *event, void *data) {
   mrlbrowser_private_data_t *private_data = (mrlbrowser_private_data_t *)data;
-  XKeyEvent      mykeyevent;
-  KeySym         mykey;
-  char           kbuf[256];
-  int            len;
   
   switch(event->type) {
 
-  case EnterNotify:
+  case ButtonPress: {
+    XButtonEvent *bevent = (XButtonEvent *) event;
+
     XLOCK(private_data->imlibdata->x.disp);
     XRaiseWindow(private_data->imlibdata->x.disp, private_data->window);
     XUNLOCK(private_data->imlibdata->x.disp);
-    break;
 
-  case ButtonPress: {
-    XButtonEvent *bevent = (XButtonEvent *) event;
     if (bevent->button == Button4)
       xitk_browser_step_down((xitk_widget_t *)private_data->mrlb_list, NULL);
     else if(bevent->button == Button5)
@@ -705,7 +858,15 @@ static void mrlbrowser_handle_event(XEvent *event, void *data) {
     XUNLOCK(private_data->imlibdata->x.disp);
     break;
     
-  case KeyPress:
+  case KeyPress: {
+    XKeyEvent      mykeyevent;
+    KeySym         mykey;
+    char           kbuf[256];
+    int            len;
+    int            modifier;
+
+    xitk_get_key_modifier(event, &modifier);
+    
     mykeyevent = event->xkey;
     
     XLOCK (private_data->imlibdata->x.disp);
@@ -713,43 +874,42 @@ static void mrlbrowser_handle_event(XEvent *event, void *data) {
     XUNLOCK (private_data->imlibdata->x.disp);
     
     switch (mykey) {
-      
-    case XK_space:
-    case XK_s:
-    case XK_S:
-      mrlbrowser_select(NULL, (void *)private_data);
+
+    case XK_d: 
+    case XK_D:
+#ifdef DEBUG_MRLB
+      /* This is for debugging purpose */
+      if(modifier & MODIFIER_CTRL)
+	mrlbrowser_dumpmrl(NULL, (void *)private_data);
+#endif
       break;
 
-    case XK_Return:
-      mrlbrowser_play(NULL, (void *)private_data);
+    case XK_s:
+    case XK_S:
+      if(modifier & MODIFIER_CTRL)
+	mrlbrowser_select(NULL, (void *)private_data);
       break;
+
+    case XK_Return: {
+      int selected;
       
-      /* This is for debugging purpose */
-    case XK_d:
-    case XK_D:
-      mrlbrowser_dumpmrl(NULL, (void *)private_data);
-      break;
-      
-    case XK_Down:
-    case XK_Next:
-      xitk_browser_step_up((xitk_widget_t *)private_data->mrlb_list, NULL);
-      break;
-      
-    case XK_Up:
-    case XK_Prior:
-      xitk_browser_step_down((xitk_widget_t *)private_data->mrlb_list, NULL);
-      break;
+      if((selected = xitk_browser_get_current_selected(private_data->mrlb_list)) >= 0)
+	mrlbrowser_select_mrl(private_data, selected, 0, 1);
     }
-    
     break;
-    
+
+    }
+  }
+  break;
+
   }
 }
 
 /*
  * Create mrlbrowser window.
  */
-xitk_widget_t *xitk_mrlbrowser_create(xitk_skin_config_t *skonfig, xitk_mrlbrowser_widget_t *mb) {
+xitk_widget_t *xitk_mrlbrowser_create(xitk_widget_list_t *wl,
+				      xitk_skin_config_t *skonfig, xitk_mrlbrowser_widget_t *mb) {
   GC                          gc;
   XSizeHints                  hint;
   XSetWindowAttributes        attr;
@@ -765,6 +925,7 @@ xitk_widget_t *xitk_mrlbrowser_create(xitk_skin_config_t *skonfig, xitk_mrlbrows
   xitk_labelbutton_widget_t   lb;
   xitk_button_widget_t        pb;
   xitk_label_widget_t         lbl;
+  xitk_combo_widget_t         cmb;
   xitk_widget_t              *w;
   long                        data[1];
   
@@ -783,6 +944,7 @@ xitk_widget_t *xitk_mrlbrowser_create(xitk_skin_config_t *skonfig, xitk_mrlbrows
   XITK_WIDGET_INIT(&pb, mb->imlibdata);
   XITK_WIDGET_INIT(&lbl, mb->imlibdata);
   XITK_WIDGET_INIT(&mb->browser, mb->imlibdata);
+  XITK_WIDGET_INIT(&cmb, mb->imlibdata);
   
   mywidget                        = (xitk_widget_t *) xitk_xmalloc(sizeof(xitk_widget_t));
   
@@ -904,8 +1066,6 @@ xitk_widget_t *xitk_mrlbrowser_create(xitk_skin_config_t *skonfig, xitk_mrlbrows
 
   private_data->widget_list                = xitk_widget_list_new() ;
   private_data->widget_list->l             = xitk_list_new ();
-  private_data->widget_list->focusedWidget = NULL;
-  private_data->widget_list->pressedWidget = NULL;
   private_data->widget_list->win           = private_data->window;
   private_data->widget_list->gc            = gc;
   
@@ -916,14 +1076,16 @@ xitk_widget_t *xitk_mrlbrowser_create(xitk_skin_config_t *skonfig, xitk_mrlbrows
   lb.userdata          = (void *)private_data;
   lb.skin_element_name = mb->select.skin_element_name;
   xitk_list_append_content(private_data->widget_list->l,
-			   (w = xitk_labelbutton_create (skonfig, &lb)));
+		   (w = xitk_labelbutton_create (private_data->widget_list, skonfig, &lb)));
+  w->widget_type |= WIDGET_GROUP | WIDGET_GROUP_MRLBROWSER;
   xitk_set_widget_tips(w, _("Select current entry"));
   
   pb.skin_element_name = mb->play.skin_element_name;
   pb.callback          = mrlbrowser_play;
   pb.userdata          = (void *)private_data;
   xitk_list_append_content(private_data->widget_list->l,
-			   (w = xitk_button_create (skonfig, &pb)));
+		   (w = xitk_button_create (private_data->widget_list, skonfig, &pb)));
+  w->widget_type |= WIDGET_GROUP | WIDGET_GROUP_MRLBROWSER;
   xitk_set_widget_tips(w, _("Play selected entry"));
 
   lb.button_type       = CLICK_BUTTON;
@@ -933,27 +1095,30 @@ xitk_widget_t *xitk_mrlbrowser_create(xitk_skin_config_t *skonfig, xitk_mrlbrows
   lb.userdata          = (void *)mywidget;
   lb.skin_element_name = mb->dismiss.skin_element_name;
   xitk_list_append_content(private_data->widget_list->l,
-			   (w = xitk_labelbutton_create (skonfig, &lb)));
+		   (w = xitk_labelbutton_create (private_data->widget_list, skonfig, &lb)));
+  w->widget_type |= WIDGET_GROUP | WIDGET_GROUP_MRLBROWSER;
   xitk_set_widget_tips(w, _("Close MRL browser window"));
   
   private_data->add_callback      = mb->select.callback;
   private_data->play_callback     = mb->play.callback;
   private_data->kill_callback     = mb->kill.callback;
   mb->browser.dbl_click_callback  = handle_dbl_click;
-  mb->browser.dbl_click_time      = DEFAULT_DBL_CLICK_TIME;
   mb->browser.userdata            = (void *)private_data;
   mb->browser.parent_wlist        = private_data->widget_list;
   xitk_list_append_content (private_data->widget_list->l,
-			   (private_data->mrlb_list = 
-			    xitk_browser_create(skonfig, &mb->browser)));
+		    (private_data->mrlb_list = 
+		     xitk_browser_create(private_data->widget_list, skonfig, &mb->browser)));
+  private_data->mrlb_list->widget_type |= WIDGET_GROUP | WIDGET_GROUP_MRLBROWSER;
 
   lbl.label             = "";
   lbl.skin_element_name = mb->origin.skin_element_name;
   lbl.window            = private_data->widget_list->win;
   lbl.gc                = private_data->widget_list->gc;
+  lbl.callback          = NULL;
   xitk_list_append_content(private_data->widget_list->l,
 			  (private_data->widget_origin = 
-			   xitk_label_create(skonfig, &lbl)));
+			   xitk_label_create(private_data->widget_list, skonfig, &lbl)));
+  private_data->widget_origin->widget_type |= WIDGET_GROUP | WIDGET_GROUP_MRLBROWSER;
   
   memset(&private_data->current_origin, 0, strlen(private_data->current_origin));
   if(mb->origin.cur_origin)
@@ -965,8 +1130,11 @@ xitk_widget_t *xitk_mrlbrowser_create(xitk_skin_config_t *skonfig, xitk_mrlbrows
     lbl.skin_element_name = mb->ip_name.label.skin_element_name;
     lbl.window            = private_data->widget_list->win;
     lbl.gc                = private_data->widget_list->gc;
+    lbl.callback          = NULL;
     xitk_list_append_content(private_data->widget_list->l, 
-			    xitk_label_create (skonfig, &lbl));
+			     (w = xitk_label_create (private_data->widget_list, skonfig, &lbl)));
+    w->widget_type |= WIDGET_GROUP | WIDGET_GROUP_MRLBROWSER;
+
   }
 
   /*
@@ -989,9 +1157,12 @@ xitk_widget_t *xitk_mrlbrowser_create(xitk_skin_config_t *skonfig, xitk_mrlbrows
       lb.userdata       = (void *)private_data;
       lb.skin_element_name = mb->ip_name.button.skin_element_name;
       xitk_list_append_content(private_data->widget_list->l,
-			       (private_data->autodir_plugins[i] = xitk_labelbutton_create (skonfig, &lb)));
+		       (private_data->autodir_plugins[i] = 
+			xitk_labelbutton_create (private_data->widget_list, skonfig, &lb)));
       xitk_set_widget_tips(private_data->autodir_plugins[i], 
 			   xine_get_input_plugin_description(mb->xine, mb->ip_availables[i]));
+
+      private_data->autodir_plugins[i]->widget_type |= WIDGET_GROUP | WIDGET_GROUP_MRLBROWSER;
       
       (void) xitk_set_widget_pos(private_data->autodir_plugins[i], x, y);
 
@@ -1001,9 +1172,34 @@ xitk_widget_t *xitk_mrlbrowser_create(xitk_skin_config_t *skonfig, xitk_mrlbrows
     if(i)
       private_data->autodir_plugins[i+1] = NULL;
   }
-    
+
+
+  private_data->filters = (char **) xitk_xmalloc(sizeof(char *) * 
+						 (sizeof(mrl_filters) / sizeof(mrl_filter_t)));
+  {
+    int i;
+
+    for(i = 0; mrl_filters[i].name != NULL; i++)
+      private_data->filters[i] = strdup(mrl_filters[i].name);
+
+    private_data->filters[i] = NULL;
+    private_data->filter_selected = 0;
+  }
+  
+  cmb.skin_element_name = mb->combo.skin_element_name;
+  cmb.parent_wlist      = private_data->widget_list;
+  cmb.entries           = private_data->filters;
+  cmb.parent_wkey       = &private_data->widget_key;
+  cmb.callback          = combo_filter_select;
+  cmb.userdata          = (void *)private_data;
+  xitk_list_append_content(private_data->widget_list->l, 
+		   (private_data->combo_filter = 
+		    xitk_combo_create(private_data->widget_list, skonfig, &cmb, NULL, NULL)));
+
   private_data->visible        = 1;
   
+  mywidget->widget_list        = NULL;
+
   mywidget->enable             = 1;
   mywidget->running            = 1;
   mywidget->visible            = 1;
@@ -1013,7 +1209,7 @@ xitk_widget_t *xitk_mrlbrowser_create(xitk_skin_config_t *skonfig, xitk_mrlbrows
   mywidget->y                  = mb->y;
   mywidget->width              = private_data->bg_image->width;
   mywidget->height             = private_data->bg_image->height;
-  mywidget->widget_type        = WIDGET_TYPE_MRLBROWSER | WIDGET_TYPE_GROUP;
+  mywidget->widget_type        = WIDGET_GROUP | WIDGET_GROUP_WIDGET | WIDGET_GROUP_MRLBROWSER;
   mywidget->paint              = NULL;
   mywidget->notify_click       = NULL;
   mywidget->notify_focus       = NULL;
