@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: skin.c,v 1.13 2002/04/18 23:37:00 f1rmb Exp $
+ * $Id: skin.c,v 1.15 2002/06/06 11:30:31 f1rmb Exp $
  *
  */
 #ifdef HAVE_CONFIG_H
@@ -141,16 +141,17 @@ static char *_get_expanded_command(xitk_skin_config_t *skonfig, char *cmd) {
  * Nullify all entried from s element.
  */
 static void _nullify_me(xitk_skin_element_t *s) {
-  s->prev        = NULL;
-  s->next        = NULL;
-  s->section     = NULL;
-  s->pixmap      = NULL;
-  s->pixmap_pad  = NULL;
-  s->pixmap_font = NULL;
-  s->color       = NULL;
-  s->color_focus = NULL;
-  s->color_click = NULL;
-  s->font        = NULL;
+  s->prev            = NULL;
+  s->next            = NULL;
+  s->section         = NULL;
+  s->pixmap          = NULL;
+  s->pixmap_pad      = NULL;
+  s->pixmap_font     = NULL;
+  s->color           = NULL;
+  s->color_focus     = NULL;
+  s->color_click     = NULL;
+  s->font            = NULL;
+  s->browser_entries = -2;
 }
 
 /*
@@ -215,7 +216,7 @@ static void skin_clean_eol(xitk_skin_config_t *skonfig) {
   if(p) {
     while(*p != '\0') {
       if(*p == '\n' || *p == '\r' || (*p == '#' && (istriplet(p) == 0)) || *p == ';' 
-	 || (*p == '/' && (*(p+1) == '/' || *(p+1) == '*'))) {
+	 || (*p == '/' && *(p+1) == '*')) {
 	*p = '\0';
 	break;
       }
@@ -335,7 +336,21 @@ static void skin_parse_subsection(xitk_skin_config_t *skonfig) {
     *(skonfig->ln + brace_offset) = '\0';
     skin_clean_eol(skonfig);
 
-    if(!strncasecmp(skonfig->ln, "coords", 6)) {
+    if(!strncasecmp(skonfig->ln, "browser", 7)) {
+
+      while(skin_end_section(skonfig) < 0) {
+	skin_get_next_line(skonfig);
+	p = skonfig->ln;
+
+	if(!strncasecmp(skonfig->ln, "entries", 7)) {
+	  skin_set_pos_to_value(&p);
+	  skonfig->celement->browser_entries = strtol(p, &p, 10);
+	}
+
+      }
+      skin_get_next_line(skonfig);
+    }
+    else if(!strncasecmp(skonfig->ln, "coords", 6)) {
 
       while(skin_end_section(skonfig) < 0) {
 	skin_get_next_line(skonfig);
@@ -547,6 +562,10 @@ static void skin_parse_section(xitk_skin_config_t *skonfig) {
 	    skonfig->date = strdup(p);
 	    return;
 	  }
+	  else if(!strncasecmp(section, "logo", 4)) {
+	    skonfig->logo = strdup(p);
+	    return;
+	  }
 	  else if(!strncasecmp(section, "url", 3)) {
 	    skonfig->url = strdup(p);
 	    return;
@@ -582,6 +601,7 @@ static void check_skonfig(xitk_skin_config_t *skonfig) {
     printf("     load cmd  '%s'\n", skonfig->load_command);
     printf("     uload cmd '%s'\n", skonfig->unload_command);
     printf("     URL       '%s'\n", skonfig->url);
+    printf("     logo      '%s'\n", skonfig->logo);
 
     while(s) {
       printf("Section '%s'\n", s->section);
@@ -596,6 +616,9 @@ static void check_skonfig(xitk_skin_config_t *skonfig) {
 	printf("  pad pixmap  = '%s'\n", s->pixmap_pad);
       }
 
+      if(s->browser_entries > -1)
+	printf("  browser entries = %d\n", s->browset_entries);
+
       printf("  animation   = %d\n", s->animation);
       printf("  print       = %d\n", s->print);
       printf("  static      = %d\n", s->staticity);
@@ -665,7 +688,7 @@ xitk_skin_config_t *xitk_skin_init_config(void) {
   skonfig->version = -1;
   skonfig->first = skonfig->last = skonfig->celement = NULL;
   skonfig->name = skonfig->author = skonfig->date = skonfig->url = 
-    skonfig->load_command = skonfig->unload_command = NULL;
+    skonfig->load_command = skonfig->unload_command = skonfig->logo = NULL;
   skonfig->skinfile = skonfig->path = NULL;
 
   skonfig->ln = skonfig->buf;
@@ -707,6 +730,7 @@ void xitk_skin_free_config(xitk_skin_config_t *skonfig) {
   XITK_FREE(skonfig->author);
   XITK_FREE(skonfig->date);
   XITK_FREE(skonfig->url);
+  XITK_FREE(skonfig->logo);
   XITK_FREE(skonfig->path);
   XITK_FREE(skonfig->load_command);
   XITK_FREE(skonfig->unload_command);
@@ -1058,6 +1082,29 @@ int xitk_skin_get_slider_radius(xitk_skin_config_t *skonfig, const char *str) {
   return 0;
 }
 
+/*
+ *
+ */
+char *xitk_skin_get_logo(xitk_skin_config_t *skonfig) {
+  assert(skonfig);
+  
+  return skonfig->logo;
+}
+
+/*
+ *
+ */
+int xitk_skin_get_browser_entries(xitk_skin_config_t *skonfig, const char *str) {
+  xitk_skin_element_t *s;
+  
+  assert(skonfig);
+  
+  if((s = skin_lookup_section(skonfig, str)) != NULL)
+    return s->browser_entries;
+
+  return -1;
+}
+
 /*
  *
  */
