@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: skin.h,v 1.12 2002/04/18 23:37:00 f1rmb Exp $
+ * $Id: skin.h,v 1.14 2002/06/06 11:30:31 f1rmb Exp $
  *
  */
 #ifndef HAVE_XITK_SKIN_H
@@ -38,6 +38,9 @@ typedef struct xitk_skin_element_s {
   int                           x;
   int                           y;
 
+  /* browser */
+  int                           browser_entries;
+  
   /* slider */
   char                         *pixmap_pad;
   int                           slider_type;
@@ -68,6 +71,7 @@ typedef struct {
   char                         *author;
   char                         *date;
   char                         *url;
+  char                         *logo;
 
   pthread_mutex_t               mutex;
   char                         *load_command;
