@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: tabs.c,v 1.7 2002/04/12 21:38:51 f1rmb Exp $
+ * $Id: tabs.c,v 1.13 2002/07/17 20:29:32 f1rmb Exp $
  *
  */
 #ifdef HAVE_CONFIG_H
@@ -48,9 +48,11 @@ typedef struct {
  */
 static void notify_destroy(xitk_widget_t *w, void *data) {
   tabs_private_data_t *private_data;
-
-  if(w->widget_type & WIDGET_TYPE_TABS) {
+  
+  if(w && (((w->widget_type & WIDGET_GROUP_MASK) & WIDGET_GROUP_TABS) &&
+	   (w->widget_type & WIDGET_GROUP_WIDGET))) {
     int i;
+    
     private_data = (tabs_private_data_t *) w->private_data;
 
     XITK_FREE(private_data->skin_element_name);
@@ -67,7 +69,8 @@ static void notify_destroy(xitk_widget_t *w, void *data) {
 static void tabs_arrange(xitk_widget_t *w) {
   tabs_private_data_t  *private_data;
 
-  if((w->widget_type & WIDGET_TYPE_TABS) && (w->visible == 1)) {
+  if(w && ((((w->widget_type & WIDGET_GROUP_MASK) & WIDGET_GROUP_TABS) &&
+	    (w->widget_type & WIDGET_GROUP_WIDGET)) && (w->visible == 1))) {
     int i = 0, width, x;
     
     private_data = (tabs_private_data_t*) w->private_data;
@@ -143,9 +146,12 @@ static void tabs_arrange(xitk_widget_t *w) {
  */
 static void paint(xitk_widget_t *w, Window win, GC gc) {
   tabs_private_data_t *private_data;
+  
+  if(w && (((w->widget_type & WIDGET_GROUP_MASK) == WIDGET_GROUP_TABS) &&
+	   (w->widget_type & WIDGET_GROUP_WIDGET))) {
 
-  if(w->widget_type & WIDGET_TYPE_TABS) {
     private_data = (tabs_private_data_t*)w->private_data;
+
     if((w->visible == 1)) {
       tabs_arrange(w);
       if(private_data->old_offset == -2)
@@ -216,7 +222,9 @@ void xitk_tabs_set_current_selected(xitk_widget_t *w, int select) {
   if(!w)
     XITK_WARNING("%s(): widget is NULL\n", __FUNCTION__);
   
-  if(w->widget_type & WIDGET_TYPE_TABS) {
+  if(w && (((w->widget_type & WIDGET_GROUP_MASK) == WIDGET_GROUP_TABS) &&
+	   (w->widget_type & WIDGET_GROUP_WIDGET))) {
+    
     private_data = (tabs_private_data_t*)w->private_data;
     
     if(select <= private_data->num_entries) {
@@ -239,7 +247,9 @@ int xitk_tabs_get_current_selected(xitk_widget_t *w) {
     return -1;
   }
     
-  if(w->widget_type & WIDGET_TYPE_TABS) {
+  if(((w->widget_type & WIDGET_GROUP_MASK) == WIDGET_GROUP_TABS) &&
+     (w->widget_type & WIDGET_GROUP_WIDGET)) {
+
     private_data = (tabs_private_data_t*)w->private_data;
     return (private_data->selected);
   }
@@ -259,7 +269,9 @@ char *xitk_tabs_get_current_tab_selected(xitk_widget_t *w) {
     return NULL;
   }
 
-  if(w->widget_type & WIDGET_TYPE_TABS) {
+  if(((w->widget_type & WIDGET_GROUP_MASK) == WIDGET_GROUP_TABS) &&
+     (w->widget_type & WIDGET_GROUP_WIDGET)) {
+
     private_data = (tabs_private_data_t*)w->private_data;
     return ((xitk_labelbutton_get_label(private_data->tabs[private_data->selected])));
   }
@@ -270,7 +282,9 @@ char *xitk_tabs_get_current_tab_selected(xitk_widget_t *w) {
 /*
  *
  */
-xitk_widget_t *xitk_noskin_tabs_create(xitk_tabs_widget_t *t, int x, int y, int width) {
+xitk_widget_t *xitk_noskin_tabs_create(xitk_widget_list_t *wl,
+				       xitk_tabs_widget_t *t, 
+				       int x, int y, int width, char *fontname) {
   xitk_widget_t         *mywidget;
   tabs_private_data_t   *private_data;
   
@@ -307,15 +321,15 @@ xitk_widget_t *xitk_noskin_tabs_create(xitk_tabs_widget_t *t, int x, int y, int
     int                        xx = x;
     btnlist_t                 *bt;
       
-    fs = xitk_font_load_font(t->imlibdata->x.disp, DEFAULT_FONT_12);
+    fs = xitk_font_load_font(t->imlibdata->x.disp, fontname);
 
     xitk_font_set_font(fs, t->parent_wlist->gc);
-    fheight = xitk_font_get_string_height(fs, "HEIGHT");
+    fheight = xitk_font_get_string_height(fs, FONT_HEIGHT_MODEL);
 
     XITK_WIDGET_INIT(&lb, t->imlibdata);
     XITK_WIDGET_INIT(&b, t->imlibdata);
 
-    private_data->bheight = fheight * 3;
+    private_data->bheight = fheight * 2;
 
     for(i = 0; i < t->num_entries; i++) {
 
@@ -333,11 +347,12 @@ xitk_widget_t *xitk_noskin_tabs_create(xitk_tabs_widget_t *t, int x, int y, int
       lb.state_callback    = tabs_select;
       lb.userdata          = (void *)bt;
       xitk_list_append_content (t->parent_wlist->l, 
-				(private_data->tabs[i] = 
-				 xitk_noskin_labelbutton_create (&lb, xx, y, fwidth + 20, 
-								 private_data->bheight, 
-								 "Black", "Black", "Black",
-								 DEFAULT_FONT_12)));
+			(private_data->tabs[i] = 
+			 xitk_noskin_labelbutton_create (t->parent_wlist, &lb, xx, y, fwidth + 20, 
+							 private_data->bheight, 
+							 "Black", "Black", "Black",
+							 fontname)));
+      private_data->tabs[i]->widget_type |= WIDGET_GROUP | WIDGET_GROUP_TABS;
       xx += fwidth + 20;
 
       xitk_hide_widget(private_data->parent_wlist, private_data->tabs[i]);
@@ -347,7 +362,6 @@ xitk_widget_t *xitk_noskin_tabs_create(xitk_tabs_widget_t *t, int x, int y, int
 
     /* 
        Add left/rigth arrows 
-       FIXME: draw arrows
     */
     {
       xitk_image_t  *wimage;
@@ -356,8 +370,10 @@ xitk_widget_t *xitk_noskin_tabs_create(xitk_tabs_widget_t *t, int x, int y, int
       b.callback          = tabs_select_prev;
       b.userdata          = (void *)mywidget;
       xitk_list_append_content(t->parent_wlist->l,
-       (private_data->left = xitk_noskin_button_create(&b, (private_data->x + width) - 40, 
-						       (y-1) + (private_data->bheight - 20), 20, 20)));
+	       (private_data->left = 
+		xitk_noskin_button_create(t->parent_wlist, &b, (private_data->x + width) - 40, 
+					  (y-1) + (private_data->bheight - 20), 20, 20)));
+      private_data->left->widget_type |= WIDGET_GROUP | WIDGET_GROUP_TABS;
       
       wimage = xitk_get_widget_foreground_skin(private_data->left);
       if(wimage)
@@ -368,8 +384,10 @@ xitk_widget_t *xitk_noskin_tabs_create(xitk_tabs_widget_t *t, int x, int y, int
       b.callback          = tabs_select_next;
       b.userdata          = (void *)mywidget;
       xitk_list_append_content(t->parent_wlist->l,
-       (private_data->right = xitk_noskin_button_create(&b, (private_data->x + width) - 20,
-							(y-1) + (private_data->bheight - 20), 20, 20)));
+	       (private_data->right = 
+		xitk_noskin_button_create(t->parent_wlist, &b, (private_data->x + width) - 20,
+					  (y-1) + (private_data->bheight - 20), 20, 20)));
+      private_data->right->widget_type |= WIDGET_GROUP | WIDGET_GROUP_TABS;
 
       wimage = xitk_get_widget_foreground_skin(private_data->right);
       if(wimage)
@@ -386,6 +404,8 @@ xitk_widget_t *xitk_noskin_tabs_create(xitk_tabs_widget_t *t, int x, int y, int
 
   mywidget->private_data          = private_data;
 
+  mywidget->widget_list           = wl;
+
   mywidget->enable                = 1;
   mywidget->running               = 1;
   mywidget->visible               = 1;
@@ -393,7 +413,7 @@ xitk_widget_t *xitk_noskin_tabs_create(xitk_tabs_widget_t *t, int x, int y, int
   mywidget->have_focus            = FOCUS_LOST; 
   mywidget->imlibdata             = private_data->imlibdata;
   mywidget->x = mywidget->y = mywidget->width = mywidget->height = 0;
-  mywidget->widget_type           = WIDGET_TYPE_TABS | WIDGET_TYPE_GROUP;
+  mywidget->widget_type           = WIDGET_GROUP | WIDGET_GROUP_WIDGET | WIDGET_GROUP_TABS;
   mywidget->paint                 = paint;
   mywidget->notify_click          = NULL;
   mywidget->notify_focus          = NULL;
