@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: tabs.h,v 1.3 2002/03/21 00:22:53 f1rmb Exp $
+ * $Id: tabs.h,v 1.5 2002/07/17 20:29:32 f1rmb Exp $
  *
  */
 #ifndef HAVE_XITK_TABS_H
@@ -59,7 +59,9 @@ typedef struct {
 
 } tabs_private_data_t;
 
-xitk_widget_t *xitk_noskin_tabs_create(xitk_tabs_widget_t *t, int x, int y, int width);
+xitk_widget_t *xitk_noskin_tabs_create(xitk_widget_list_t *wl,
+				       xitk_tabs_widget_t *t, 
+				       int x, int y, int width, char *fontname);
 int xitk_tabs_get_current_selected(xitk_widget_t *w);
 char *xitk_tabs_get_current_tab_selected(xitk_widget_t *w);
 void xitk_tabs_set_current_selected(xitk_widget_t *w, int select);
