@@ -82,6 +82,9 @@ typedef struct {
   xitk_widget_t        *intbox;
   int                   oldintvalue;
 
+  /* checkbox */
+  xitk_widget_t        *checkbox;
+
   xitk_widget_list_t   *widget_list;
   xitk_register_key_t   kreg;
 } test_t;
@@ -89,7 +92,9 @@ typedef struct {
 
 static test_t *test;
 static int nlab = 0;
-static int align = LABEL_ALIGN_CENTER;
+static int align = LABEL_ALIGN_LEFT;
+#define FONT_HEIGHT_MODEL "azertyuiopqsdfghjklmwxcvbnAZERTYUIOPQSDFGHJKLMWXCVBN&(-_)=~#{[|`\\^@]}%"
+
 /*
  *
  */
@@ -169,44 +174,37 @@ void test_handle_event(XEvent *event, void *data) {
     break;
 
   case KeyPress: {
-    xitk_widget_t *w = xitk_get_focused_widget(test->widget_list);
+    int modifier;
     
-    if(w && (w->widget_type & WIDGET_TYPE_INPUTTEXT)) {
-      xitk_send_key_event(test->widget_list, w, event);
-    }
-    else {
-      int modifier;
+    (void) xitk_get_key_modifier(event, &modifier);
+    
+    mykeyevent = event->xkey;
+    
+    XLockDisplay (test->display);
+    len = XLookupString(&mykeyevent, kbuf, sizeof(kbuf), &mykey, NULL);
+    XUnlockDisplay (test->display);
+    
+    switch (mykey) {
       
-      (void) xitk_get_key_modifier(event, &modifier);
+    case XK_q:
+    case XK_Q:
+      if(modifier & MODIFIER_CTRL)
+	test_end(NULL, NULL);
+      break;
       
-      mykeyevent = event->xkey;
-      
-      XLockDisplay (test->display);
-      len = XLookupString(&mykeyevent, kbuf, sizeof(kbuf), &mykey, NULL);
-      XUnlockDisplay (test->display);
-      
-      switch (mykey) {
-	
-      case XK_q:
-      case XK_Q:
-	if(modifier & MODIFIER_CTRL)
-	  test_end(NULL, NULL);
-	break;
-	
-      }   
-    }
+    }   
   }
   break;
-
+  
   case MappingNotify:
     XLockDisplay(test->display);
     XRefreshKeyboardMapping((XMappingEvent *) event);
     XUnlockDisplay(test->display);
     break;
     
-  case ConfigureNotify:
-    xitk_combo_update_pos(test->combo);
-    break;
+    //  case ConfigureNotify:
+    //    xitk_combo_update_pos(test->combo);
+    //    break;
   }
 }
 
@@ -282,10 +280,8 @@ static void change_label(xitk_widget_t *w, void *data) {
   //  xitk_window_dialog_ok_with_width(test->imlibdata, "Long error message", window_message_cb, NULL, 500, ALIGN_DEFAULT, "**Thisprogramisfreesoftware;youcanredistributeitand/ormodify**itunderthetermsoftheGNUGeneralPublicLicenseaspublishedby**TheFreeSoftwareFoundation;eitherversion2oftheLicense,or**(atyouroption)anylaterversion.");
   xitk_window_dialog_error(test->imlibdata, 
 			   "Stream number %d <%s.mpg> is not valid.\n", nlab, labels[nlab]);
-
 }
 
-
 /*
  *
  */
@@ -366,18 +362,38 @@ static void create_intbox(void) {
   ib.callback          = notify_intbox_change;
   ib.userdata          = NULL;
   xitk_list_append_content (test->widget_list->l,
-			    (test->intbox = 
-			     xitk_noskin_intbox_create(&ib, x, y, 60, 20, NULL, NULL, NULL)));
+	    (test->intbox = 
+	     xitk_noskin_intbox_create(test->widget_list, &ib, x, y, 60, 20, NULL, NULL, NULL)));
 
   xitk_set_widget_tips_default(test->intbox, "This is a intbox");
 }
 
+/*
+ *
+ */
+static void create_checkbox(void) {
+  int x = 250, y = 300;
+  xitk_checkbox_widget_t cb;
+
+  XITK_WIDGET_INIT(&cb, test->imlibdata);
+
+  cb.skin_element_name = NULL;
+  cb.callback          = NULL;
+  cb.userdata          = NULL;
+  xitk_list_append_content (test->widget_list->l,
+		    (test->checkbox = 
+		     xitk_noskin_checkbox_create(test->widget_list, &cb, x, y, 20, 20)));
+
+  xitk_set_widget_tips_default(test->checkbox, "This is a checkbox");
+}
+
 /*
  *
  */
 static void create_tabs(void) {
   Pixmap              bg;
   xitk_tabs_widget_t  t;
+  char               *fontname = "*-helvetica-medium-r-*-*-12-*-*-*-*-*-*-*";
   int                 x = 150, y = 200, w = 300;
   int                 width, height;
   static char        *tabs_labels[] = {
@@ -391,7 +407,7 @@ static void create_tabs(void) {
   XCopyArea(test->display, (xitk_window_get_background(test->xwin)), bg,
 	    test->widget_list->gc, 0, 0, width, height, 0, 0);
   
-  draw_rectangular_outter_box(test->imlibdata, bg, x, y+20, w, 60);
+  draw_rectangular_outter_box(test->imlibdata, bg, x, y+20, (w-1), 60);
   xitk_window_change_background(test->imlibdata, test->xwin, bg, width, height);
   XFreePixmap(test->display, bg);
   
@@ -402,8 +418,8 @@ static void create_tabs(void) {
   t.callback          = NULL;
   t.userdata          = NULL;
   xitk_list_append_content (test->widget_list->l,
-			    (test->tabs = 
-			     xitk_noskin_tabs_create(&t, x, y, w)));
+		    (test->tabs = 
+		     xitk_noskin_tabs_create(test->widget_list, &t, x, y, w, fontname)));
 
 }
 
@@ -413,7 +429,7 @@ static void create_tabs(void) {
 static void create_frame(void) {
   Pixmap    bg;
   int       width, height;
-  char     *fontname = "*-lucida-*-r-*-*-12-*-*-*-*-*-*-*";
+  char     *fontname = "*-helvetica-bold-r-*-*-12-*-*-*-*-*-*-*";
   int       x = 350, y = 50, w = 200, h = 150;
   
   xitk_window_get_window_size(test->xwin, &width, &height);
@@ -433,7 +449,7 @@ static void create_frame(void) {
  */
 static void create_inputtext(void) {
   xitk_inputtext_widget_t  inp;
-  char                    *fontname = "*-lucida-*-r-*-*-12-*-*-*-*-*-*-*";
+  char                    *fontname = "*-helvetica-medium-r-*-*-10-*-*-*-*-*-*-*";
 
   XITK_WIDGET_INIT(&inp, test->imlibdata);
 
@@ -443,10 +459,10 @@ static void create_inputtext(void) {
   inp.callback          = change_browser_entry;
   inp.userdata          = NULL;
   xitk_list_append_content (test->widget_list->l,
-			   (test->input = 
-			    xitk_noskin_inputtext_create(&inp,
-							 150, 150, 150, 20,
-							 "Black", "Black", fontname)));
+	   (test->input = 
+	    xitk_noskin_inputtext_create(test->widget_list, &inp,
+					 150, 150, 150, 20,
+					 "Black", "Black", fontname)));
 
   xitk_set_widget_tips_default(test->input, "This is an inputtext");
 }
@@ -456,7 +472,7 @@ static void create_inputtext(void) {
  */
 static void create_label(void) {
   xitk_label_widget_t   lbl;
-  char                 *fontname = "*-lucida-*-r-*-*-14-*-*-*-*-*-*-*";
+  char                 *fontname = "*-helvetica-medium-r-*-*-10-*-*-*-*-*-*-*";
   int                   x = 150, y = 120, len = 100;
   xitk_font_t          *fs;
   int                   lbear, rbear, wid, asc, des;
@@ -469,14 +485,15 @@ static void create_label(void) {
   xitk_font_string_extent(fs, label, &lbear, &rbear, &wid, &asc, &des);
   xitk_font_unload_font(fs);
 
-  lbl.window    = xitk_window_get_window(test->xwin);
-  lbl.gc        = test->widget_list->gc;
+  lbl.window            = xitk_window_get_window(test->xwin);
+  lbl.gc                = test->widget_list->gc;
   lbl.skin_element_name = NULL;
   lbl.label             = label;
+  lbl.callback          = NULL;
   xitk_list_append_content(test->widget_list->l, 
-			   (test->label = 
-			    xitk_noskin_label_create(&lbl,
-						     x, y, len, (asc+des)*2, fontname)));
+	   (test->label = 
+	    xitk_noskin_label_create(test->widget_list, &lbl,
+				     x, y, len, (asc+des)*2, fontname)));
 
   xitk_set_widget_tips_default(test->label, "This is a label");
 
@@ -502,9 +519,9 @@ static void create_button(void) {
   b.callback          = change_label;
   b.userdata          = NULL;
   xitk_list_append_content(test->widget_list->l, 
-			   (test->button = 
-			    xitk_noskin_button_create(&b,
-						      x, y, width, height)));
+	   (test->button = 
+	    xitk_noskin_button_create(test->widget_list, &b,
+				      x, y, width, height)));
   
   xitk_set_widget_tips_default(test->button, "This is a button");
 
@@ -514,7 +531,7 @@ static void create_button(void) {
     if(wimage) {
       unsigned int   col;
       xitk_font_t   *fs = NULL;
-      char          *fontname = "*-lucida-*-r-*-*-14-*-*-*-*-*-*-*";
+      char          *fontname = "*-helvetica-bold-r-*-*-14-*-*-*-*-*-*-*";
       int            lbear, rbear, wid, asc, des;
       char          *label = _("Fire !!");
 
@@ -580,9 +597,9 @@ static void create_sliders(void) {
   sl.motion_callback          = move_sliders;
   sl.motion_userdata          = NULL;
   xitk_list_append_content(test->widget_list->l,
-			   (test->hslider = xitk_noskin_slider_create(&sl,
-								      17, 200, 117, 20,
-								      XITK_HSLIDER)));
+		   (test->hslider = xitk_noskin_slider_create(test->widget_list, &sl,
+							      17, 200, 117, 20,
+							      XITK_HSLIDER)));
   xitk_slider_set_pos(test->widget_list, test->hslider, 0);
 
   xitk_set_widget_tips_default(test->hslider, "This is an horizontal slider");
@@ -596,9 +613,9 @@ static void create_sliders(void) {
   sl.motion_callback          = move_sliders;
   sl.motion_userdata          = NULL;
   xitk_list_append_content(test->widget_list->l,
-			   (test->vslider = xitk_noskin_slider_create(&sl,
-								      17, 230, 20, 117,
-								      XITK_VSLIDER)));
+		   (test->vslider = xitk_noskin_slider_create(test->widget_list, &sl,
+							      17, 230, 20, 117,
+							      XITK_VSLIDER)));
   xitk_slider_set_pos(test->widget_list, test->vslider, 0);
 
   xitk_set_widget_tips_default(test->vslider, "This is a vertical slider");
@@ -612,12 +629,13 @@ static void create_sliders(void) {
   sl.motion_callback          = move_sliders;
   sl.motion_userdata          = NULL;
   xitk_list_append_content(test->widget_list->l,
-			   (test->rslider = xitk_noskin_slider_create(&sl,
-								      50, 240, 80, 80,
-								      XITK_RSLIDER)));
+		   (test->rslider = xitk_noskin_slider_create(test->widget_list, &sl,
+							      50, 240, 80, 80,
+							      XITK_RSLIDER)));
   xitk_slider_set_pos(test->widget_list, test->rslider, 0);
   
   xitk_set_widget_tips_default(test->rslider, "This is a rotate button");
+
 }
 
 static void combo_select(xitk_widget_t *w, void *data, int select) {
@@ -645,7 +663,7 @@ static void combo_select(xitk_widget_t *w, void *data, int select) {
  */
 static void create_combo(void) {
   xitk_combo_widget_t    cmb;
-  char                  *fontname = "*-lucida-*-r-*-*-12-*-*-*-*-*-*-*";
+  char                  *fontname = "*-helvetica-medium-r-*-*-10-*-*-*-*-*-*-*";
   int                    x = 150, y = 36, width = 100, height;
   xitk_font_t           *fs;
 
@@ -658,7 +676,7 @@ static void create_combo(void) {
   */
   fs = xitk_font_load_font(test->display, fontname);
   xitk_font_set_font(fs, test->widget_list->gc);
-  height = xitk_font_get_string_height(fs, "HEIGHT");
+  height = xitk_font_get_string_height(fs, FONT_HEIGHT_MODEL);
   xitk_font_unload_font(fs);
   /*
   draw_rectangular_inner_box(test->imlibdata, bg, (x - 4), (y - 4), (width + 8), (height + 7));
@@ -674,10 +692,9 @@ static void create_combo(void) {
   cmb.callback          = combo_select;
   cmb.userdata          = NULL;
   xitk_list_append_content(test->widget_list->l, 
-			   (test->combo = 
-			    xitk_noskin_combo_create(&cmb,
-						     x, y, width, NULL, NULL)));
-
+		   (test->combo = 
+		    xitk_noskin_combo_create(test->widget_list, &cmb,
+					     x, y, width, NULL, NULL)));
 }
 
 /*
@@ -685,7 +702,7 @@ static void create_combo(void) {
  */
 static void create_browser(void) {
   xitk_browser_widget_t  browser;
-  char                  *fontname = "*-lucida-*-r-*-*-12-*-*-*-*-*-*-*";
+  char                  *fontname = "*-helvetica-medium-r-*-*-10-*-*-*-*-*-*-*";
   Pixmap                 bg;
   int                    width, height;
 
@@ -707,7 +724,7 @@ static void create_browser(void) {
   {
     int i;
     char buf[64];
-
+    
     for(i = 0; i < 25; i++) {
       memset(&buf, 0, sizeof(buf));
       sprintf(buf, "Entry %d", i);
@@ -726,14 +743,13 @@ static void create_browser(void) {
   browser.browser.entries               = test->entries;
   browser.callback                      = change_inputtext;
   browser.dbl_click_callback            = change_inputtext_dbl_click;
-  browser.dbl_click_time                = DEFAULT_DBL_CLICK_TIME;
   browser.parent_wlist                  = test->widget_list;
   browser.userdata                      = NULL;
   xitk_list_append_content (test->widget_list->l, 
-			    (test->browser = 
-			     xitk_noskin_browser_create(&browser,
-							test->widget_list->gc, 20, 30, 
-							100, 20, 12, fontname)));
+		    (test->browser = 
+		     xitk_noskin_browser_create(test->widget_list, &browser,
+						test->widget_list->gc, 20, 30, 
+						100, 20, 12, fontname)));
   
   xitk_browser_update_list(test->browser, test->entries, test->num_entries, 0);
 
@@ -745,7 +761,7 @@ static void create_browser(void) {
 int main(int argc, char **argv) {
   GC                          gc;
   xitk_labelbutton_widget_t   lb;
-  char                       *fontname = "*-lucida-*-r-*-*-14-*-*-*-*-*-*-*";
+  char                       *fontname = "*-helvetica-bold-r-*-*-12-*-*-*-*-*-*-*";
   int                         windoww = 600, windowh = 400;
   xitk_widget_t              *w;
   
@@ -776,8 +792,6 @@ int main(int argc, char **argv) {
 
   test->widget_list                = xitk_widget_list_new();
   test->widget_list->l             = xitk_list_new ();
-  test->widget_list->focusedWidget = NULL;
-  test->widget_list->pressedWidget = NULL;
   test->widget_list->win           = (xitk_window_get_window(test->xwin));
   test->widget_list->gc            = gc;
 
@@ -791,7 +805,7 @@ int main(int argc, char **argv) {
   lb.userdata          = NULL;
   lb.skin_element_name = NULL;
   xitk_list_append_content(test->widget_list->l, 
-	   (w = xitk_noskin_labelbutton_create(&lb,
+	   (w = xitk_noskin_labelbutton_create(test->widget_list, &lb,
 					       (windoww / 2) - 50, windowh - 50,
 					       100, 30,
 					       "Black", "Black", "White", fontname)));
@@ -806,6 +820,7 @@ int main(int argc, char **argv) {
   create_frame();
   create_tabs();
   create_intbox();
+  create_checkbox();
   
   test->kreg = xitk_register_event_handler("test", 
 					   (xitk_window_get_window(test->xwin)),
