@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: tips.c,v 1.12 2002/04/21 22:17:12 f1rmb Exp $
+ * $Id: tips.c,v 1.16 2002/05/23 11:24:42 f1rmb Exp $
  *
  */
 #ifdef HAVE_CONFIG_H
@@ -76,6 +76,23 @@ static void _tips_kill_running(void) {
   }
 }
 
+/*
+ *
+ */
+static void _tips_handle_event(XEvent *event, void *data) {
+  /* tips_private_t      *tp = (tips_private_t *) data; */
+  
+  switch(event->type) {
+    
+  case MotionNotify:
+  case ButtonRelease:
+  case ButtonPress:
+    _tips_kill_running();
+    break;
+    
+  }
+}
+
 /*
  *
  */
@@ -87,7 +104,7 @@ static void *_tips_destroy_thread(void *data) {
   pthread_detach(pthread_self());
 
   /* Waiting enought time to read the tips */
-  xitk_usec_sleep(1500000);
+  xitk_usec_sleep(3000000);
 
   pthread_setcancelstate(PTHREAD_CANCEL_DISABLE,NULL);
 
@@ -122,7 +139,7 @@ static void *_tips_thread(void *data) {
   xitk_font_t        *fs;
   XWindowAttributes   wattr;
   Status              status;
-  unsigned int        cyellow, cblack;
+  unsigned int        cwarnfore, cwarnback;
 
   pthread_detach(pthread_self());
 
@@ -142,12 +159,12 @@ static void *_tips_thread(void *data) {
   string_length = xitk_font_get_string_length(fs, tp->w->tips_string);
   xitk_font_unload_font(fs);
 
-  cblack  = xitk_get_pixel_color_black(tp->w->imlibdata);
-  cyellow = xitk_get_pixel_color_from_rgb(tp->w->imlibdata, 255, 255, 0);
-
+  cwarnfore = xitk_get_pixel_color_warning_foreground(tp->w->imlibdata);
+  cwarnback = xitk_get_pixel_color_warning_background(tp->w->imlibdata);
+  
   i = xitk_image_create_image_with_colors_from_string(tp->w->imlibdata, DEFAULT_FONT_10,
 						      string_length + 1, ALIGN_LEFT, 
-						      tp->w->tips_string, cblack, cyellow);
+						      tp->w->tips_string, cwarnfore, cwarnback);
 
   
   /* Create the tips window, horizontaly centered from parent widget */
@@ -181,10 +198,10 @@ static void *_tips_thread(void *data) {
     XCopyArea(tp->w->imlibdata->x.disp, (xitk_window_get_background(tp->xwin)), bg,
 	      gc, 0, 0, width, height, 0, 0);
 
-    XSetForeground(tp->w->imlibdata->x.disp, gc, cblack);
+    XSetForeground(tp->w->imlibdata->x.disp, gc, cwarnfore);
     XDrawRectangle(tp->w->imlibdata->x.disp, bg, gc, 0, 0, width - 1, height - 1);
     
-    XSetForeground(tp->w->imlibdata->x.disp, gc, cyellow);
+    XSetForeground(tp->w->imlibdata->x.disp, gc, cwarnback);
     XFillRectangle(tp->w->imlibdata->x.disp, bg, gc, 1, 1, width - 2, height - 2);
 
     XCopyArea(tp->w->imlibdata->x.disp, i->image, bg,
@@ -216,11 +233,11 @@ static void *_tips_thread(void *data) {
   /* TODO: forward key event to parent window */
   tp->key = xitk_register_event_handler("xitk tips", 
 					(xitk_window_get_window(tp->xwin)),
+					_tips_handle_event,
 					NULL,
 					NULL,
 					NULL,
-					NULL,
-					NULL);
+					(void *) tp);
 
   /* Create a thread which will destroy the tips window */  
   {
