@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: window.c,v 1.15 2002/04/19 20:59:49 f1rmb Exp $
+ * $Id: window.c,v 1.21 2002/07/17 20:29:32 f1rmb Exp $
  *
  */
 #ifdef HAVE_CONFIG_H
@@ -70,6 +70,26 @@ static void _xitk_window_destroy_window(xitk_widget_t *, void *);
 
 #define TITLE_BAR_HEIGHT 20
 
+/*
+ * Is window is size match with given args
+ */
+int xitk_is_window_visible(Display *display, Window window) {
+  XWindowAttributes  wattr;
+  Status             status;
+  
+  if((display == NULL) || (window == None))
+    return -1;
+  
+  XLOCK(display);
+  status = XGetWindowAttributes(display, window, &wattr);
+  XUNLOCK(display);
+  
+  if((status != BadDrawable) && (status != BadWindow) && (wattr.map_state == IsViewable))
+    return 1;
+  
+  return 0;
+}
+
 /*
  * Is window is size match with given args
  */
@@ -381,7 +401,7 @@ xitk_window_t *xitk_window_create_dialog_window(ImlibData *im, char *title,
   XDrawLine(im->x.disp, bar, gc, 0, TITLE_BAR_HEIGHT - 1, width - 1, TITLE_BAR_HEIGHT - 1);
   XUNLOCK(im->x.disp);
 
-  fs = xitk_font_load_font(im->x.disp, DEFAULT_FONT_12);
+  fs = xitk_font_load_font(im->x.disp, DEFAULT_BOLD_FONT_12);
   xitk_font_set_font(fs, gc);
   xitk_font_string_extent(fs, title, &lbear, &rbear, &wid, &asc, &des);
 
@@ -509,6 +529,20 @@ static void _window_handle_event(XEvent *event, void *data) {
   
   switch(event->type) {
 
+  case Expose:
+    wd->widget_list->widget_focused = wd->default_button;
+    if (wd->widget_list->widget_focused->notify_focus 
+	&& wd->widget_list->widget_focused->enable == WIDGET_ENABLE) {
+      xitk_widget_t *w = wd->widget_list->widget_focused;
+
+      (void) (w->notify_focus) (wd->widget_list, w, FOCUS_RECEIVED);
+      w->have_focus = FOCUS_RECEIVED;
+
+      if(w->paint)
+	(w->paint) (w, wd->widget_list->win, wd->widget_list->gc);
+    }
+    break;
+    
   case MappingNotify:
     XLOCK(wd->imlibdata->x.disp);
     XRefreshKeyboardMapping((XMappingEvent *) event);
@@ -638,7 +672,7 @@ void xitk_window_dialog_one_button_with_width(ImlibData *im, char *title, char *
       return;
     
     while(1) {
-      
+
       va_start(args, message);
       n = vsnprintf(buf, size, message, args);
       va_end(args);
@@ -671,8 +705,6 @@ void xitk_window_dialog_one_button_with_width(ImlibData *im, char *title, char *
   
   wd->widget_list                = xitk_widget_list_new();
   wd->widget_list->l             = xitk_list_new ();
-  wd->widget_list->focusedWidget = NULL;
-  wd->widget_list->pressedWidget = NULL;
   wd->widget_list->win           = (xitk_window_get_window(wd->xwin));
 
   XLOCK(wd->imlibdata->x.disp);
@@ -697,10 +729,10 @@ void xitk_window_dialog_one_button_with_width(ImlibData *im, char *title, char *
   lb.skin_element_name = NULL;
   xitk_list_append_content(wd->widget_list->l, 
 	   (wd->wyes =
-	    xitk_noskin_labelbutton_create(&lb,
+	    xitk_noskin_labelbutton_create(wd->widget_list, &lb,
 					   bx, by,
 					   bwidth, 30,
-					   "Black", "Black", "White", DEFAULT_FONT_12)));
+					   "Black", "Black", "White", DEFAULT_BOLD_FONT_12)));
   wd->default_button = wd->wyes;
 
   /* Draw text area */
@@ -757,7 +789,7 @@ void xitk_window_dialog_ok_with_width(ImlibData *im, char *title,
     return;
   
   while(1) {
-    
+
     va_start(args, message);
     n = vsnprintf(buf, size, message, args);
     va_end(args);
@@ -774,8 +806,12 @@ void xitk_window_dialog_ok_with_width(ImlibData *im, char *title,
       return;
   }
   
-  xitk_window_dialog_one_button_with_width(im, title, _("OK"), cb, userdata, window_width,
-					   align, buf);
+  {
+    char buf2[(strlen(buf) * 2) + 1];
+    xitk_subst_special_chars(buf, buf2);
+    xitk_window_dialog_one_button_with_width(im, title, _("OK"), cb, userdata, window_width,
+					     align, buf2);
+  }
   XITK_FREE(buf);
 }
 
@@ -842,8 +878,6 @@ void xitk_window_dialog_two_buttons_with_width(ImlibData *im, char *title,
   
   wd->widget_list                = xitk_widget_list_new();
   wd->widget_list->l             = xitk_list_new ();
-  wd->widget_list->focusedWidget = NULL;
-  wd->widget_list->pressedWidget = NULL;
   wd->widget_list->win           = (xitk_window_get_window(wd->xwin));
   XLOCK(wd->imlibdata->x.disp);
   wd->widget_list->gc            = (XCreateGC(im->x.disp, (xitk_window_get_window(wd->xwin)),
@@ -868,10 +902,10 @@ void xitk_window_dialog_two_buttons_with_width(ImlibData *im, char *title,
   lb.skin_element_name = NULL;
   xitk_list_append_content(wd->widget_list->l, 
 	   (wd->wyes =
-	    xitk_noskin_labelbutton_create(&lb,
+	    xitk_noskin_labelbutton_create(wd->widget_list, &lb,
 					   bx1, by,
 					   bwidth, 30,
-					   "Black", "Black", "White", DEFAULT_FONT_12)));
+					   "Black", "Black", "White", DEFAULT_BOLD_FONT_12)));
 
   lb.button_type       = CLICK_BUTTON;
   lb.label             = button2_label;
@@ -882,10 +916,10 @@ void xitk_window_dialog_two_buttons_with_width(ImlibData *im, char *title,
   lb.skin_element_name = NULL;
   xitk_list_append_content(wd->widget_list->l, 
 	   (wd->wno =
-	    xitk_noskin_labelbutton_create(&lb,
+	    xitk_noskin_labelbutton_create(wd->widget_list, &lb,
 					   bx2, by,
 					   bwidth, 30,
-					   "Black", "Black", "White", DEFAULT_FONT_12)));
+					   "Black", "Black", "White", DEFAULT_BOLD_FONT_12)));
 
   wd->default_button = wd->wno;
 
@@ -963,8 +997,14 @@ void xitk_window_dialog_yesno_with_width(ImlibData *im, char *title,
       return;
   }
   
-  xitk_window_dialog_two_buttons_with_width(im, title, _("Yes"), _("No"), 
-					    ycb, ncb, userdata, window_width, align, buf);
+  {
+    char buf2[(strlen(buf) * 2) + 1];
+    
+    xitk_subst_special_chars(buf, buf2);
+    xitk_window_dialog_two_buttons_with_width(im, title, _("Yes"), _("No"), 
+					      ycb, ncb, userdata, window_width, align, buf2);
+  }
+
   XITK_FREE(buf);
 }
 
@@ -1023,20 +1063,19 @@ void xitk_window_dialog_three_buttons_with_width(ImlibData *im, char *title,
 
   windowh = (i->height + 50) + (TITLE_BAR_HEIGHT + 40);
 
-  wd->imlibdata = im;
-  wd->type = DIALOG_TYPE_YESNOCANCEL;
-  wd->yescallback = cb1;
-  wd->nocallback = cb2;
+  wd->imlibdata      = im;
+  wd->type           = DIALOG_TYPE_YESNOCANCEL;
+  wd->yescallback    = cb1;
+  wd->nocallback     = cb2;
   wd->cancelcallback = cb3;
-  wd->userdata = userdata;
-  wd->xwin = xitk_window_create_dialog_window(im, ((title != NULL) ? title : _("Question?")), 
-					      0, 0, windoww, windowh);  
+  wd->userdata       = userdata;
+  wd->xwin           = xitk_window_create_dialog_window(im, ((title != NULL) 
+							     ? title : _("Question?")), 
+							0, 0, windoww, windowh);  
   xitk_window_center_window(im, wd->xwin);
   
   wd->widget_list                = xitk_widget_list_new();
   wd->widget_list->l             = xitk_list_new ();
-  wd->widget_list->focusedWidget = NULL;
-  wd->widget_list->pressedWidget = NULL;
   wd->widget_list->win           = (xitk_window_get_window(wd->xwin));
   XLOCK(wd->imlibdata->x.disp);
   wd->widget_list->gc            = (XCreateGC(im->x.disp, (xitk_window_get_window(wd->xwin)),
@@ -1062,10 +1101,10 @@ void xitk_window_dialog_three_buttons_with_width(ImlibData *im, char *title,
   lb.skin_element_name = NULL;
   xitk_list_append_content(wd->widget_list->l, 
 	   (wd->wyes =
-	    xitk_noskin_labelbutton_create(&lb,
+	    xitk_noskin_labelbutton_create(wd->widget_list, &lb,
 					   bx1, by,
 					   bwidth, 30,
-					   "Black", "Black", "White", DEFAULT_FONT_12)));
+					   "Black", "Black", "White", DEFAULT_BOLD_FONT_12)));
 
   lb.button_type       = CLICK_BUTTON;
   lb.label             = button2_label;
@@ -1076,10 +1115,10 @@ void xitk_window_dialog_three_buttons_with_width(ImlibData *im, char *title,
   lb.skin_element_name = NULL;
   xitk_list_append_content(wd->widget_list->l, 
 	   (wd->wno =
-	    xitk_noskin_labelbutton_create(&lb,
+	    xitk_noskin_labelbutton_create(wd->widget_list, &lb,
 					   bx2, by,
 					   bwidth, 30,
-					   "Black", "Black", "White", DEFAULT_FONT_12)));
+					   "Black", "Black", "White", DEFAULT_BOLD_FONT_12)));
 
   lb.button_type       = CLICK_BUTTON;
   lb.label             = button3_label;
@@ -1090,10 +1129,10 @@ void xitk_window_dialog_three_buttons_with_width(ImlibData *im, char *title,
   lb.skin_element_name = NULL;
   xitk_list_append_content(wd->widget_list->l, 
 	   (wd->wcancel =
-	    xitk_noskin_labelbutton_create(&lb,
+	    xitk_noskin_labelbutton_create(wd->widget_list, &lb,
 					   bx3, by,
 					   bwidth, 30,
-					   "Black", "Black", "White", DEFAULT_FONT_12)));
+					   "Black", "Black", "White", DEFAULT_BOLD_FONT_12)));
 
   wd->default_button = wd->wcancel;
 
@@ -1169,7 +1208,12 @@ void xitk_window_dialog_yesnocancel_with_width(ImlibData *im, char *title,
       return;
   }
   
-  xitk_window_dialog_three_buttons_with_width(im, title, _("Yes"), _("No"), _("Cancel"),
-					      ycb, ncb, ccb, userdata, window_width, align, buf);
+  {
+    char buf2[(strlen(buf) * 2) + 1];
+    
+    xitk_subst_special_chars(buf, buf2);
+    xitk_window_dialog_three_buttons_with_width(im, title, _("Yes"), _("No"), _("Cancel"),
+						ycb, ncb, ccb, userdata, window_width, align, buf2);
+  }
   XITK_FREE(buf);
 }
