@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: window.h,v 1.6 2002/04/19 20:59:49 f1rmb Exp $
+ * $Id: window.h,v 1.7 2002/04/29 23:43:02 f1rmb Exp $
  *
  */
 #ifndef HAVE_XITK_WINDOW_H
@@ -34,6 +34,11 @@
 #define XITK_WINDOW_ANSWER_NO      3
 #define XITK_WINDOW_ANSWER_CANCEL  4
 
+/*
+ * Is window visible.
+ */
+int xitk_is_window_visible(Display *display, Window window);
+
 /*
  * Is window is size match with given args
  */
