@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: xitk.c,v 1.35 2002/04/21 22:17:12 f1rmb Exp $
+ * $Id: xitk.c,v 1.51 2002/07/21 23:07:28 f1rmb Exp $
  *
  */
 #define _XITK_C_
@@ -41,7 +41,6 @@
 #include <unistd.h>
 #include <signal.h>
 #include <time.h>
-#include <setjmp.h>
 #include <sys/types.h>
 #include <sys/wait.h>
 
@@ -49,10 +48,17 @@
 
 #include <X11/Xlib.h>
 #include <X11/Xatom.h>
+#include <X11/keysym.h>
 
 #include "widget.h"
 #include "list.h"
 #include "dnd.h"
+#include "inputtext.h"
+#include "checkbox.h"
+#include "browser.h"
+#include "slider.h"
+#include "combo.h"
+#include "tips.h"
 #include "_config.h"
 
 #include "_xitk.h"
@@ -110,6 +116,7 @@ typedef struct {
   int                         height;
 
   XEvent                     *old_event;
+
   xitk_widget_list_t         *widget_list;
   char                       *name;
   widget_event_callback_t     xevent_callback;
@@ -123,6 +130,7 @@ typedef struct {
   Display                    *display;
   xitk_list_t                *list;
   xitk_list_t                *gfx;
+
   pthread_mutex_t             mutex;
   int                         running;
   xitk_register_key_t         key;
@@ -133,7 +141,6 @@ typedef struct {
 
 static __xitk_t    *gXitk;
 static pid_t        xitk_pid;
-static sigjmp_buf   kill_jmp;
 
 
 void widget_stop(void);
@@ -206,58 +213,32 @@ static void xitk_signal_handler(int sig) {
   pid_t cur_pid = getppid();
 
   /* First, call registered handler */
-  if(gXitk->sig_callback)
-    gXitk->sig_callback(sig, gXitk->sig_data);
+  if(cur_pid == xitk_pid) {
+    if(gXitk->sig_callback)
+      gXitk->sig_callback(sig, gXitk->sig_data);
+  }
+  
+  gXitk->running = 0;
   
   switch (sig) {
-
+    
   case SIGINT:
   case SIGTERM:
   case SIGQUIT:
     if(cur_pid == xitk_pid) {
-      __gfx_t  *fx;
       
-      XITK_WARNING("XITK killed with signal %d\n", sig);
-
-      MUTLOCK();
-      gXitk->running = 0;
+      xitk_list_free(gXitk->list);
+      xitk_list_free(gXitk->gfx);
+      xitk_config_deinit(gXitk->config);
       
-      /* 
-       * Tada.... and now Ladies and Gentlemen, the dirty hack
-       */
-      fx = (__gfx_t *) xitk_list_first_content(gXitk->gfx);
-
-      while(fx) {
-	if(fx->window) {
-	  XEvent event;
-	  
-	  event.xany.type = ClientMessage;
-	  event.xclient.type = ClientMessage;
-	  event.xclient.send_event = True;
-	  event.xclient.display = gXitk->display;
-	  event.xclient.window = fx->window;
-	  event.xclient.message_type = fx->XA_XITK;
-	  event.xclient.format = 32;
-	  memset(&event.xclient.data, 0, sizeof(event.xclient.data));
-	  snprintf(event.xclient.data.b, 20, "%s", "KILL");
-	  
-	  XLOCK(gXitk->display);
-	  if (!XSendEvent (gXitk->display, fx->window, True, 0L, &event)) {
-	    XITK_WARNING("XSendEvent(display, 0x%x ...) failed.\n", (unsigned int) fx->window);
-	  }
-	  XUNLOCK(gXitk->display);
-
-	  MUTUNLOCK();
-	  siglongjmp(kill_jmp, 1);
-	  return;
-	}
-	fx = (__gfx_t *) xitk_list_next_content(gXitk->gfx);
-      }
-      MUTUNLOCK();
+      XITK_FREE(gXitk);
+      exit(1);
     }
     break;
   }
+
 }
+
 /*
  * Create a new widget_list, store the pointer in private
  * list of xitk_widget_list_t, then return the widget_list pointer.
@@ -269,6 +250,10 @@ xitk_widget_list_t *xitk_widget_list_new (void) {
 
   l = (xitk_widget_list_t *) xitk_xmalloc(sizeof(xitk_widget_list_t));
 
+  l->widget_focused     = NULL;
+  l->widget_under_mouse = NULL;
+  l->widget_pressed     = NULL;
+
   xitk_list_append_content(gXitk->list, l);
 
   MUTUNLOCK();
@@ -351,8 +336,17 @@ xitk_register_key_t xitk_register_event_handler(char *name, Window window,
     XUNLOCK(gXitk->display);
     
     if(err != BadDrawable && err != BadWindow) {
-      fx->new_pos.x = wattr.x;
-      fx->new_pos.y = wattr.y;
+      Window c;
+      
+      XLOCK(gXitk->display);
+      XTranslateCoordinates(gXitk->display, fx->window, wattr.root, 
+			    0, 0, &(fx->new_pos.x), &(fx->new_pos.y), &c);
+      XUNLOCK(gXitk->display);
+      
+      /*
+	fx->new_pos.x = wattr.x;
+	fx->new_pos.y = wattr.y;
+      */
       fx->width     = wattr.width;
       fx->height    = wattr.height;
     }
@@ -452,12 +446,19 @@ int xitk_get_window_info(xitk_register_key_t key, window_info_t *winf) {
   while(fx) {
 
     if((fx->key == key) && (fx->window != None)) {
+      Window c;
       
       winf->window = fx->window;
-
+      
       if(fx->name)
 	winf->name = strdup(fx->name);
       
+      XLOCK(gXitk->display);
+      XTranslateCoordinates(gXitk->display, fx->window, DefaultRootWindow(gXitk->display), 
+			    0, 0, &(fx->new_pos.x), &(fx->new_pos.y), &c);
+      XUNLOCK(gXitk->display);
+      
+      
       winf->x      = fx->new_pos.x;
       winf->y      = fx->new_pos.y;
       winf->height = fx->height;
@@ -481,7 +482,6 @@ int xitk_get_window_info(xitk_register_key_t key, window_info_t *winf) {
  */
 void xitk_xevent_notify(XEvent *event) {
   __gfx_t  *fx;
-
     
   fx = (__gfx_t *) xitk_list_first_content(gXitk->gfx);
 
@@ -495,20 +495,171 @@ void xitk_xevent_notify(XEvent *event) {
 	
 	switch(event->type) {
 
-	case Expose:
-	  if (fx->widget_list && (event->xexpose.count == 0)) {
-	    xitk_paint_widget_list (fx->widget_list);
+	case MappingNotify:
+	  XLOCK(gXitk->display);
+	  XRefreshKeyboardMapping((XMappingEvent *) event);
+	  XUNLOCK(gXitk->display);
+	  break;
+
+	case KeyPress: {
+	  XKeyEvent      mykeyevent;
+	  KeySym         mykey;
+	  char           kbuf[256];
+	  int            len;
+	  int            modifier;
+	  int            handled = 0;
+	  xitk_widget_t *w = NULL;
+	  
+	  mykeyevent = event->xkey;
+
+	  xitk_get_key_modifier(event, &modifier);
+
+	  XLOCK(gXitk->display);
+	  len = XLookupString(&mykeyevent, kbuf, sizeof(kbuf), &mykey, NULL);
+	  XUNLOCK(gXitk->display);
+
+	  if(fx->widget_list && fx->widget_list->widget_under_mouse)
+	    xitk_tips_tips_kill(fx->widget_list->widget_under_mouse);
+	  
+	  if(fx->widget_list && fx->widget_list->widget_focused)
+	    xitk_tips_tips_kill(fx->widget_list->widget_focused);
+
+
+	  if(fx->widget_list && fx->widget_list->widget_focused) {
+	    w = fx->widget_list->widget_focused;
 	  }
-	  /*
-	  else {
+
+	  if(w && (((w->widget_type & WIDGET_TYPE_MASK) == WIDGET_TYPE_INPUTTEXT) &&
+		   (mykey != XK_Tab) && (mykey != XK_KP_Tab) && (mykey != XK_ISO_Left_Tab))) {
 	    
-	    if(event->xexpose.count > 0) {
-	      while(XCheckWindowEvent(gXitk->display, fx->window, 
-				      ExposureMask, event) == True);
+	    xitk_send_key_event(fx->widget_list, w, event);
+	    
+	    if((mykey == XK_Return) || 
+	       (mykey == XK_KP_Enter) || (mykey == XK_ISO_Enter) || (mykey == XK_ISO_Enter)) {
+	      
+	      if(w->paint)
+		(w->paint) (w, fx->widget_list->win, fx->widget_list->gc);
+	      
+	      xitk_set_focus_to_next_widget(fx->widget_list, 0);
+	    }
+
+	    return;
+	  }
+	  
+	  /* set focus to next widget */
+	  if((mykey == XK_Tab) || (mykey == XK_KP_Tab) || (mykey == XK_ISO_Left_Tab)) {
+	    if(fx->widget_list) {
+	      handled = 1;
+	      xitk_set_focus_to_next_widget(fx->widget_list, (modifier & MODIFIER_SHIFT));
+	    }
+	  }
+	  /* simulate click event on space/return/enter key event */
+	  else if((mykey == XK_space) || (mykey == XK_Return) || 
+		  (mykey == XK_KP_Enter) || (mykey == XK_ISO_Enter) || (mykey == XK_ISO_Enter)) {
+	    if(w && (w->notify_click && w->visible && w->enable)) {
+	      
+	      if(w && (((w->widget_type & WIDGET_TYPE_MASK) == WIDGET_TYPE_BUTTON) ||
+		       ((w->widget_type & WIDGET_TYPE_MASK) == WIDGET_TYPE_LABELBUTTON) ||
+		       ((w->widget_type & WIDGET_TYPE_MASK) == WIDGET_TYPE_CHECKBOX))) {
+		handled = 1;
+
+		w->notify_click(fx->widget_list, w, 0, w->x, w->y);
+		w->notify_click(fx->widget_list, w, 1, w->x, w->y);
+
+		//		if(fx->xevent_callback)
+		//		  fx->xevent_callback(event, fx->user_data);
+	      }
+	    }
+	  }
+	  /* move sliders */
+	  else if(((mykey == XK_Left) || (mykey == XK_Right) 
+		   || (mykey == XK_Up) || (mykey == XK_Down)) 
+		  && ((modifier & 0xFFFFFFEF) == MODIFIER_NOMOD)) {
+	    
+	    if(w && ((w->widget_type & WIDGET_GROUP_MASK) & WIDGET_GROUP_BROWSER)) {
+	      xitk_widget_t *b = xitk_browser_get_browser(fx->widget_list, w);
+	      
+	      if(b) {
+		if(mykey == XK_Up)
+		  xitk_browser_step_down(b, NULL);
+		else if(mykey == XK_Down)
+		  xitk_browser_step_up(b, NULL);
+	      }
+	    }
+	    else if(w && ((w->widget_type & WIDGET_TYPE_MASK) == WIDGET_TYPE_SLIDER)) {
+	      if((mykey == XK_Left) || (mykey == XK_Down)) {
+		handled = 1;
+		xitk_slider_make_backstep(fx->widget_list, w);
+		xitk_slider_callback_exec(w);
+	      }
+	      else {
+		handled = 1;
+		xitk_slider_make_step(fx->widget_list, w);
+		xitk_slider_callback_exec(w);
+	      }
 	    }
 
 	  }
-	  */
+	  else if((mykey == XK_0) || (mykey == XK_1) || (mykey == XK_2) || (mykey == XK_3) || 
+		  (mykey == XK_4) || (mykey == XK_5) || (mykey == XK_6) || (mykey == XK_7) || 
+		  (mykey == XK_8) || (mykey == XK_9) || 
+		  (mykey == XK_a) || (mykey == XK_A) || (mykey == XK_b) || (mykey == XK_B) ||
+		  (mykey == XK_c) || (mykey == XK_C) || (mykey == XK_d) || (mykey == XK_D) ||
+		  (mykey == XK_e) || (mykey == XK_E) || (mykey == XK_f) || (mykey == XK_F) ||
+		  (mykey == XK_g) || (mykey == XK_G) || (mykey == XK_h) || (mykey == XK_H) ||
+		  (mykey == XK_i) || (mykey == XK_I) || (mykey == XK_j) || (mykey == XK_J) ||
+		  (mykey == XK_k) || (mykey == XK_K) || (mykey == XK_l) || (mykey == XK_L) ||
+		  (mykey == XK_m) || (mykey == XK_M) || (mykey == XK_n) || (mykey == XK_N) ||
+		  (mykey == XK_o) || (mykey == XK_O) || (mykey == XK_p) || (mykey == XK_P) ||
+		  (mykey == XK_q) || (mykey == XK_Q) || (mykey == XK_r) || (mykey == XK_R) ||
+		  (mykey == XK_s) || (mykey == XK_S) || (mykey == XK_t) || (mykey == XK_T) ||
+		  (mykey == XK_u) || (mykey == XK_U) || (mykey == XK_v) || (mykey == XK_V) ||
+		  (mykey == XK_w) || (mykey == XK_W) || (mykey == XK_x) || (mykey == XK_X) ||
+		  (mykey == XK_y) || (mykey == XK_Y) || (mykey == XK_z) || (mykey == XK_Z)) {
+
+	    
+	    if(w && ((w->widget_type & WIDGET_GROUP_MASK) & WIDGET_GROUP_BROWSER)) {
+	      xitk_widget_t *b = xitk_browser_get_browser(fx->widget_list, w);
+	      
+	      if(b)
+		xitk_browser_warp_jump(b, kbuf, modifier);
+
+	    }
+	  }
+
+	  if(!handled) {
+	    if((w == NULL) || (w && (((w->widget_type & WIDGET_TYPE_MASK) == WIDGET_TYPE_INPUTTEXT) == 0))) {
+	      if(fx->xevent_callback) {
+		fx->xevent_callback(event, fx->user_data);
+	      }
+	    }
+	    
+	    if(w && ((w->widget_type & WIDGET_TYPE_MASK) == WIDGET_TYPE_INPUTTEXT) && 
+	       ((mykey == XK_Return) || (mykey == XK_KP_Enter) 
+		|| (mykey == XK_ISO_Enter) || (mykey == XK_ISO_Enter))) {
+	      
+	      if(w->paint)
+		(w->paint) (w, fx->widget_list->win, fx->widget_list->gc);
+	      
+	      xitk_set_focus_to_next_widget(fx->widget_list, 0);
+	      
+	    }
+	  }
+	  return;
+	}
+	break;
+
+	case Expose:
+	  if (fx->widget_list) {
+	    XEvent xev;
+
+	    xitk_paint_widget_list (fx->widget_list);
+
+	    XLOCK(gXitk->display);
+	    while(XCheckTypedWindowEvent(gXitk->display, fx->window, 
+					 Expose, &xev) == True);
+	    XUNLOCK(gXitk->display);
+	  }
 	  break;
 	  
 	case MotionNotify: {
@@ -568,13 +719,13 @@ void xitk_xevent_notify(XEvent *event) {
 	  XLOCK(gXitk->display);
 	  status = XGetWindowAttributes(gXitk->display, fx->window, &wattr);
 	  XGetInputFocus(gXitk->display, &focused_window, &revert);
-
+	  
 	  /* 
 	   * Give focus(and raise) to a window after a click, and 
 	   * only if window isn't the current focused one.
 	   */
 	  if((status != BadDrawable) && (status != BadWindow) 
-	     && (wattr.map_state == IsViewable) && (focused_window != fx->window)) {
+	     && (wattr.map_state == IsViewable)/* && (focused_window != fx->window)*/) {
 	    XRaiseWindow(gXitk->display, fx->window);
 	    XSetInputFocus(gXitk->display, fx->window, RevertToParent, CurrentTime);
 	  }
@@ -611,23 +762,37 @@ void xitk_xevent_notify(XEvent *event) {
 	case ButtonRelease:
 	  
 	  if(fx->move.enabled) {
+
 	    fx->move.enabled = 0;
 	    /* Inform application about window movement. */
+
 	    if(fx->newpos_callback)
 	      fx->newpos_callback(fx->new_pos.x, fx->new_pos.y, 
 				  fx->width, fx->height);
 	  }
-	  
-	  if(fx->widget_list) {
-	    xitk_click_notify_widget_list (fx->widget_list, event->xbutton.x, 
-					   event->xbutton.y, 1);
+	  else {
+	    if(fx->widget_list) {
+	      xitk_click_notify_widget_list (fx->widget_list, event->xbutton.x, 
+					     event->xbutton.y, 1);
+	    }
 	  }
 	  break;
 	  
 	case ConfigureNotify: {
 	  XWindowAttributes wattr;
 	  Status            err;
-	  
+
+	  if(fx->widget_list && fx->widget_list->l) {
+	    xitk_widget_t *w = (xitk_widget_t *) xitk_list_first_content(fx->widget_list->l);
+	    while (w) {
+	      if(((w->widget_type & WIDGET_GROUP_MASK) & WIDGET_GROUP_COMBO) &&
+		 (w->widget_type & WIDGET_GROUP_WIDGET)) {
+		xitk_combo_update_pos(w);
+	      }
+	      w = (xitk_widget_t *) xitk_list_next_content (fx->widget_list->l);
+	    }
+	  }
+
 	  XLOCK(gXitk->display);
 	  err = XGetWindowAttributes(gXitk->display, fx->window, &wattr);
 	  XUNLOCK(gXitk->display);
@@ -660,7 +825,7 @@ void xitk_xevent_notify(XEvent *event) {
       
       /* Don't forward event to all of windows */
       if(fx->xevent_callback 
-	 && (fx->window != None && event->type != KeyPress)) {
+	 && (fx->window != None && event->type != KeyRelease)) {
 	fx->xevent_callback(event, fx->user_data);
       }
       
@@ -744,6 +909,10 @@ void xitk_run(void) {
   
   gXitk->running = 1;
   
+  XLOCK(gXitk->display);
+  XSync(gXitk->display, True);
+  XUNLOCK(gXitk->display);
+
   /*
    * Force to repain the widget list if it exist
    */
@@ -779,28 +948,22 @@ void xitk_run(void) {
    */
   while(gXitk->running) {
     /* XLOCK(gXitk->display); 
-
+       
        if(XPending (gXitk->display)) { 
     */
-
-    if(sigsetjmp(kill_jmp, 1) != 0)
-      goto killing_end;
-    
-      XNextEvent (gXitk->display, &myevent) ;
+    XNextEvent (gXitk->display, &myevent) ;
       /* XUNLOCK(gXitk->display);  */
-      xitk_xevent_notify(&myevent);
-      /*
-    } else {  
+    xitk_xevent_notify(&myevent);
+    /*
+      } else {  
       XUNLOCK(gXitk->display); 
       xitk_usec_sleep(16666); // 1/60 sec
-    } 
-
+      } 
+      
     */
   }
   /*XUNLOCK(gXitk->display); */
 
- killing_end:
-
   xitk_list_free(gXitk->list);
   xitk_list_free(gXitk->gfx);
   xitk_config_deinit(gXitk->config);
@@ -839,6 +1002,58 @@ int xitk_get_select_color(void) {
 unsigned long xitk_get_timer_label_animation(void) {
   return xitk_config_get_timer_label_animation(gXitk->config);
 }
+unsigned long xitk_get_warning_foreground(void) {
+  return xitk_config_get_warning_foreground(gXitk->config);
+}
+unsigned long xitk_get_warning_background(void) {
+  return xitk_config_get_warning_background(gXitk->config);
+}
+long int xitk_get_timer_dbl_click(void) {
+  return xitk_config_get_timer_dbl_click(gXitk->config);
+}
+
+/*
+ * copy src to dest and substitute special chars. dest should have 
+ * enought space to store chars.
+ */
+void xitk_subst_special_chars(char *src, char *dest) {
+  char *s, *d;
+  
+  if((src == NULL) || (dest == NULL)) {
+    XITK_WARNING("%s(): pass NULL argument(s)\n", __FUNCTION__);
+    return;
+  }
+  
+  if(!strlen(src))
+    return;
+
+  memset(dest, 0, sizeof(dest));
+  s = src;
+  d = dest;
+  while(*s != '\0') {
+    
+    switch(*s) {
+    case '%':
+      /* %20 is ' ' */
+      if((*(s) == '%') && (*(s + 1) == '2') && (*(s + 2) == '0')) {
+	*d = ' ';
+	d += 2;
+      }
+      else {
+	*d++ = '%';
+	*d = '%';
+      }
+      break;
+      
+    default:
+      *d = *s;
+      break;
+    }
+    s++;
+    d++;
+  }
+  *d = '\0';
+}
 
 /*
  *
@@ -866,11 +1081,11 @@ const char *xitk_get_homedir(void) {
   int ret;
   struct passwd pwd;
   char *buffer = NULL;
-  int bufsize = 128;
+  int bufsize = 256;
 
   buffer = (char *) xitk_xmalloc(bufsize);
   
-  if((ret = getpwuid_r(getuid(), &pwd, buffer, bufsize, &pw)) < 0) {
+  if((ret = getpwuid_r(getuid(), &pwd, buffer, bufsize, &pw)) != 0) {
 #else
   if((pw = getpwuid(getuid())) == NULL) {
 #endif
