@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: xitk.h,v 1.54 2002/04/19 20:59:49 f1rmb Exp $
+ * $Id: xitk.h,v 1.69 2002/07/17 20:29:32 f1rmb Exp $
  *
  * Unique public header file for xitk TK.
  *
@@ -125,47 +125,57 @@ typedef struct {
   xitk_node_t           *first, *last, *cur;
 } xitk_list_t;
 
-struct xitk_widget_list_s;
+typedef struct xitk_widget_list_s xitk_widget_list_t;
 
-struct xitk_widget_s;
+typedef struct xitk_widget_s xitk_widget_t;
 
 typedef void xitk_skin_config_t;
 
-typedef void (*widget_paint_callback_t)(struct xitk_widget_s *, Window, GC);
+typedef void (*widget_paint_callback_t)(xitk_widget_t *, Window, GC);
 
-typedef int (*widget_click_callback_t) (struct xitk_widget_list_s *, struct xitk_widget_s *, int, int, int);
+typedef int (*widget_click_callback_t) (xitk_widget_list_t *, xitk_widget_t *, int, int, int);
 
-typedef int (*widget_focus_callback_t)(struct xitk_widget_list_s *, struct xitk_widget_s *, int);
+typedef int (*widget_focus_callback_t)(xitk_widget_list_t *, xitk_widget_t *, int);
 
-typedef void (*widget_keyevent_callback_t)(struct xitk_widget_list_s *, struct xitk_widget_s *, XEvent *);
+typedef void (*widget_keyevent_callback_t)(xitk_widget_list_t *, xitk_widget_t *, XEvent *);
 
-typedef int (*widget_inside_callback_t)(struct xitk_widget_s *, int, int);
+typedef int (*widget_inside_callback_t)(xitk_widget_t *, int, int);
 
-typedef void (*widget_change_skin_callback_t)(struct xitk_widget_list_s *, struct xitk_widget_s *, xitk_skin_config_t *);
+typedef void (*widget_change_skin_callback_t)(xitk_widget_list_t *, xitk_widget_t *, xitk_skin_config_t *);
 
-typedef xitk_image_t *(*widget_get_skin_t)(struct xitk_widget_s *, int);
+typedef xitk_image_t *(*widget_get_skin_t)(xitk_widget_t *, int);
 
-typedef void (*widget_destroy_t)(struct xitk_widget_s *, void *);
+typedef void (*widget_destroy_t)(xitk_widget_t *, void *);
 
-#define WIDGET_TYPE_GROUP         0xFFFF8000
+/* Group of widgets widget */
+#define WIDGET_GROUP              0x80000000
+/* Grouped widget, itself */
+#define WIDGET_GROUP_WIDGET       0x40000000
 
+/* Grouped widgets */
+#define WIDGET_GROUP_MASK         0x3FFF8000 
+#define WIDGET_GROUP_BROWSER      0x00080000
+#define WIDGET_GROUP_FILEBROWSER  0x00100000
+#define WIDGET_GROUP_MRLBROWSER   0x00200000
+#define WIDGET_GROUP_COMBO        0x00400000
+#define WIDGET_GROUP_TABS         0x00800000
+#define WIDGET_GROUP_INTBOX       0x01000000
+
+/* Real widgets. */
+#define WIDGET_TYPE_MASK          0x00007FFF
 #define WIDGET_TYPE_BUTTON        0x00000001
 #define WIDGET_TYPE_LABELBUTTON   0x00000002
+#define WIDGET_TYPE_LABEL         0x00000003
 #define WIDGET_TYPE_SLIDER        0x00000004
-#define WIDGET_TYPE_LABEL         0x00000008
-#define WIDGET_TYPE_CHECKBOX      0x00000010
-#define WIDGET_TYPE_IMAGE         0x00000020
-#define WIDGET_TYPE_BROWSER       0x00000040
-#define WIDGET_TYPE_FILEBROWSER   0x00000080
-#define WIDGET_TYPE_MRLBROWSER    0x00000100
-#define WIDGET_TYPE_INPUTTEXT     0x00000200
-#define WIDGET_TYPE_COMBO         0x00000400
-#define WIDGET_TYPE_TABS          0x00000800
-#define WIDGET_TYPE_INTBOX        0x00001000
+#define WIDGET_TYPE_CHECKBOX      0x00000005
+#define WIDGET_TYPE_IMAGE         0x00000006
+#define WIDGET_TYPE_INPUTTEXT     0x00000007
 
-typedef struct xitk_widget_s {
+struct xitk_widget_s {
   ImlibData                      *imlibdata;
 
+  xitk_widget_list_t             *widget_list;
+
   int                             x;
   int                             y;
   int                             width;
@@ -200,17 +210,18 @@ typedef struct xitk_widget_s {
 
   void                           *private_data;
   uint32_t                        widget_type;
-} xitk_widget_t;
+};
 
-typedef struct witk_widget_list_s {
-  xitk_list_t          *l;
+struct xitk_widget_list_s {
+  xitk_list_t         *l;
 
-  xitk_widget_t            *focusedWidget;
-  xitk_widget_t            *pressedWidget;
+  xitk_widget_t       *widget_focused;
+  xitk_widget_t       *widget_under_mouse;
+  xitk_widget_t       *widget_pressed;
 
   Window               win;
   GC                   gc;
-} xitk_widget_list_t;
+};
 
 
 typedef void (*xitk_simple_callback_t)(xitk_widget_t *, void *);
@@ -319,6 +330,7 @@ void xitk_unregister_event_handler(xitk_register_key_t *key);
  */
 
 int xitk_get_window_info(xitk_register_key_t key, window_info_t *winf);
+
 /*
  * Initialization function, should be the first call to widget lib.
  */
@@ -343,6 +355,23 @@ void xitk_run(void);
  */
 void xitk_stop(void);
 
+/*
+ * Some user settings values.
+ */
+char *xitk_get_system_font(void);
+char *xitk_get_default_font(void);
+int xitk_get_black_color(void);
+int xitk_get_white_color(void);
+int xitk_get_background_color(void);
+int xitk_get_focus_color(void);
+int xitk_get_select_color(void);
+void xitk_subst_special_chars(char *src, char *dest);
+unsigned long xitk_get_timer_label_animation(void);
+long int xitk_get_timer_dbl_click(void);
+unsigned long xitk_get_warning_foreground(void);
+unsigned long xitk_get_warning_background(void);
+
+
 /*
  *
  ****** */
@@ -352,11 +381,21 @@ void xitk_stop(void);
  */
 void *xitk_xmalloc(size_t);
 
+/**
+ *
+ */
+xitk_color_names_t *xitk_get_color_name(char *color);
+
 /**
  * return pointer to the xitk_color_names struct.
  */
 xitk_color_names_t *xitk_get_color_names(void);
 
+/**
+ * Free color object.
+ */
+void xitk_free_color_name(xitk_color_names_t *color);
+
 /**
  * (re)Paint a widget list.
  */
@@ -574,12 +613,14 @@ typedef struct {
 /**
  * Create a slider
  */
-xitk_widget_t *xitk_slider_create(xitk_skin_config_t *skonfig, xitk_slider_widget_t *sl);
+xitk_widget_t *xitk_slider_create(xitk_widget_list_t *wl,
+				  xitk_skin_config_t *skonfig, xitk_slider_widget_t *sl);
 
 /**
  *
  */
-xitk_widget_t *xitk_noskin_slider_create (xitk_slider_widget_t *s,
+xitk_widget_t *xitk_noskin_slider_create (xitk_widget_list_t *wl,
+					  xitk_slider_widget_t *s,
 					  int x, int y, int width, int height, int type);
 
 /**
@@ -632,6 +673,10 @@ void xitk_slider_make_step(xitk_widget_list_t *, xitk_widget_t *);
  */
 void xitk_slider_make_backstep(xitk_widget_list_t *, xitk_widget_t *);
 
+/**
+ * Call callback for current position
+ */
+void xitk_slider_callback_exec(xitk_widget_t *);
 
 /*
  * *** Nodes ***
@@ -710,12 +755,14 @@ typedef struct {
 /**
  * Create a labeled button.
  */
-xitk_widget_t *xitk_labelbutton_create(xitk_skin_config_t *skonfig, xitk_labelbutton_widget_t *b);
+xitk_widget_t *xitk_labelbutton_create(xitk_widget_list_t *wl,
+				       xitk_skin_config_t *skonfig, xitk_labelbutton_widget_t *b);
 
 /**
  *
  */
-xitk_widget_t *xitk_noskin_labelbutton_create (xitk_labelbutton_widget_t *b,
+xitk_widget_t *xitk_noskin_labelbutton_create (xitk_widget_list_t *wl,
+					       xitk_labelbutton_widget_t *b,
 					       int x, int y, int width, int height,
 					       char *ncolor, char *fcolor, char *ccolor, 
 					       char *fname);
@@ -756,22 +803,26 @@ int xitk_labelbutton_get_alignment(xitk_widget_t *);
  */
 typedef struct {
   int                       magic;
-  ImlibData              *imlibdata;
-  Window                  window;
-  GC                      gc;
-  char                   *label;
-  char                   *skin_element_name;
+  ImlibData                *imlibdata;
+  Window                    window;
+  GC                        gc;
+  char                     *label;
+  char                     *skin_element_name;
+  xitk_simple_callback_t    callback;
+  void                     *userdata;
 } xitk_label_widget_t;
 
 /**
  * Create a label widget.
  */
-xitk_widget_t *xitk_label_create(xitk_skin_config_t *skonfig, xitk_label_widget_t *l);
+xitk_widget_t *xitk_label_create(xitk_widget_list_t *wl,
+				 xitk_skin_config_t *skonfig, xitk_label_widget_t *l);
 
 /**
  *
  */
-xitk_widget_t *xitk_noskin_label_create(xitk_label_widget_t *l,
+xitk_widget_t *xitk_noskin_label_create(xitk_widget_list_t *wl,
+					xitk_label_widget_t *l,
 					int x, int y, int width, int height, char *fontname);
 
 /**
@@ -802,12 +853,14 @@ typedef struct {
 /**
  * Create an image widget type.
  */
-xitk_widget_t *xitk_image_create(xitk_skin_config_t *skonfig, xitk_image_widget_t *im);
+xitk_widget_t *xitk_image_create(xitk_widget_list_t *wl,
+				 xitk_skin_config_t *skonfig, xitk_image_widget_t *im);
 
 /**
  * Same as above, without skin.
  */
-xitk_widget_t *xitk_noskin_image_create (xitk_image_widget_t *im, 
+xitk_widget_t *xitk_noskin_image_create (xitk_widget_list_t *wl,
+					 xitk_image_widget_t *im, 
 					 xitk_image_t *image, int x, int y);
 
 /**
@@ -834,20 +887,22 @@ void xitk_unset_dnd_callback(xitk_dnd_t *);
  */
 typedef struct {
   int                       magic;
-  ImlibData              *imlibdata;
-  xitk_state_callback_t  callback;
-  void                   *userdata;
-  char                   *skin_element_name;
+  ImlibData                *imlibdata;
+  xitk_state_callback_t     callback;
+  void                     *userdata;
+  char                     *skin_element_name;
 } xitk_checkbox_widget_t;
 /**
  * Create a checkbox.
  */
-xitk_widget_t *xitk_checkbox_create (xitk_skin_config_t *skonfig, xitk_checkbox_widget_t *cp);
+xitk_widget_t *xitk_checkbox_create (xitk_widget_list_t *wl,
+				     xitk_skin_config_t *skonfig, xitk_checkbox_widget_t *cp);
 
 /*
  * Same as above, without skinable feature.
  */
-xitk_widget_t *xitk_noskin_checkbox_create(xitk_checkbox_widget_t *cb,
+xitk_widget_t *xitk_noskin_checkbox_create(xitk_widget_list_t *wl,
+					   xitk_checkbox_widget_t *cb,
 					   int x, int y, int width, int height);
 
 
@@ -878,12 +933,14 @@ typedef struct {
 /**
  *
  */
-xitk_widget_t *xitk_button_create (xitk_skin_config_t *skonfig, xitk_button_widget_t *b);
+xitk_widget_t *xitk_button_create (xitk_widget_list_t *wl,
+				   xitk_skin_config_t *skonfig, xitk_button_widget_t *b);
 
 /**
  *
  */
-xitk_widget_t *xitk_noskin_button_create (xitk_button_widget_t *b,
+xitk_widget_t *xitk_noskin_button_create (xitk_widget_list_t *wl,
+					  xitk_button_widget_t *b,
 					  int x, int y, int width, int height);
 
 
@@ -894,7 +951,6 @@ xitk_widget_t *xitk_noskin_button_create (xitk_button_widget_t *b,
  * Create the list browser
  */
 /* Default time (in ms) between mouse click to act as double click */
-#define DEFAULT_DBL_CLICK_TIME 200
 typedef struct {
   int                       magic;
   ImlibData              *imlibdata;
@@ -918,7 +974,6 @@ typedef struct {
     char                **entries;
   } browser;
   
-  int                     dbl_click_time;
   xitk_state_callback_t   dbl_click_callback;
 
   /* Callback on selection function */
@@ -931,12 +986,14 @@ typedef struct {
 /**
  *
  */
-xitk_widget_t *xitk_browser_create(xitk_skin_config_t *skonfig, xitk_browser_widget_t *b);
+xitk_widget_t *xitk_browser_create(xitk_widget_list_t *wl,
+				   xitk_skin_config_t *skonfig, xitk_browser_widget_t *b);
 
 /**
  *
  */
-xitk_widget_t *xitk_noskin_browser_create(xitk_browser_widget_t *br, GC gc, 
+xitk_widget_t *xitk_noskin_browser_create(xitk_widget_list_t *wl,
+					  xitk_browser_widget_t *br, GC gc, 
 					  int x, int y, 
 					  int itemw, int itemh, int slidw, char *fontname);
 
@@ -975,6 +1032,11 @@ void xitk_browser_set_select(xitk_widget_t *w, int select);
  */
 void xitk_browser_release_all_buttons(xitk_widget_t *w);
 
+/**
+ * Return the number of displayed entries
+ */
+int xitk_browser_get_num_entries(xitk_widget_t *w);
+
 /**
  * Return the real number of first displayed in list
  */
@@ -985,6 +1047,13 @@ int xitk_browser_get_current_start(xitk_widget_t *w);
  */
 void xitk_browser_set_alignment(xitk_widget_t *w, int align);
 
+/*
+ * Jump to entry in list which match with the alphanum char key.
+ */
+void xitk_browser_warp_jump(xitk_widget_t *w, char *key, int modifier);
+
+xitk_widget_t *xitk_browser_get_browser(xitk_widget_list_t *wl, xitk_widget_t *w);
+
 /*
  * Filebrowser
  */
@@ -1042,7 +1111,8 @@ typedef struct {
 /**
  *
  */
-xitk_widget_t *xitk_filebrowser_create(xitk_skin_config_t *skonfig, xitk_filebrowser_widget_t *fb);
+xitk_widget_t *xitk_filebrowser_create(xitk_widget_list_t *wl,
+				       xitk_skin_config_t *skonfig, xitk_filebrowser_widget_t *fb);
 
 /**
  *
@@ -1154,13 +1224,18 @@ typedef struct {
   xine_t                    *xine;
 
   xitk_browser_widget_t      browser;
+  
+  struct {
+    char                    *skin_element_name;
+  } combo;
 
 } xitk_mrlbrowser_widget_t;
 
 /**
  *
  */
-xitk_widget_t *xitk_mrlbrowser_create(xitk_skin_config_t *skonfig, xitk_mrlbrowser_widget_t *mb);
+xitk_widget_t *xitk_mrlbrowser_create(xitk_widget_list_t *wl,
+				      xitk_skin_config_t *skonfig, xitk_mrlbrowser_widget_t *mb);
 
 /**
  *
@@ -1246,12 +1321,14 @@ int xitk_get_key_modifier(XEvent *xev, int *modifier);
 /**
  * Create an input text box.
  */
-xitk_widget_t *xitk_inputtext_create (xitk_skin_config_t *skonfig, xitk_inputtext_widget_t *it);
+xitk_widget_t *xitk_inputtext_create (xitk_widget_list_t *wl,
+				      xitk_skin_config_t *skonfig, xitk_inputtext_widget_t *it);
 
 /**
  *
  */
-xitk_widget_t *xitk_noskin_inputtext_create (xitk_inputtext_widget_t *it,
+xitk_widget_t *xitk_noskin_inputtext_create (xitk_widget_list_t *wl,
+					     xitk_inputtext_widget_t *it,
 					     int x, int y, int width, int height,
 					     char *ncolor, char *fcolor, char *fontname);
 /**
@@ -1386,12 +1463,22 @@ int xitk_skin_get_slider_radius(xitk_skin_config_t *, const char *);
 /*
  *
  */
-void xitk_skin_lock(xitk_skin_config_t *skonfig);
+char *xitk_skin_get_logo(xitk_skin_config_t *);
 
 /*
  *
  */
-void xitk_skin_unlock(xitk_skin_config_t *skonfig);
+int xitk_skin_get_browser_entries(xitk_skin_config_t *, const char *);
+
+/*
+ *
+ */
+void xitk_skin_lock(xitk_skin_config_t *);
+
+/*
+ *
+ */
+void xitk_skin_unlock(xitk_skin_config_t *);
 
 
 /**
@@ -1482,7 +1569,7 @@ void xitk_font_set_font(xitk_font_t *xtfs, GC gc);
 
 
 typedef struct {
-  int                       magic;
+  int                      magic;
   ImlibData               *imlibdata;
   char                    *skin_element_name;
   xitk_widget_list_t      *parent_wlist;
@@ -1496,13 +1583,15 @@ typedef struct {
 /**
  *
  */
-xitk_widget_t *xitk_combo_create(xitk_skin_config_t *skonfig, xitk_combo_widget_t *c,
+xitk_widget_t *xitk_combo_create(xitk_widget_list_t *wl,
+				 xitk_skin_config_t *skonfig, xitk_combo_widget_t *c,
 				 xitk_widget_t **lw, xitk_widget_t **bw);
 
 /**
  *
  */
-xitk_widget_t *xitk_noskin_combo_create(xitk_combo_widget_t *c, int x, int y, int width,
+xitk_widget_t *xitk_noskin_combo_create(xitk_widget_list_t *wl,
+					xitk_combo_widget_t *c, int x, int y, int width,
 					xitk_widget_t **lw, xitk_widget_t **bw);
 
 /**
@@ -1564,6 +1653,24 @@ unsigned int xitk_get_pixel_color_darkgray(ImlibData *im);
 /**
  *
  */
+unsigned int xitk_get_pixel_color_warning_foreground(ImlibData *im);
+
+/**
+ *
+ */
+unsigned int xitk_get_pixel_color_warning_background(ImlibData *im);
+
+/**
+ *
+ */
+xitk_image_t *xitk_image_create_image_with_colors_from_string(ImlibData *im, 
+							      char *fontname, 
+							      int width, int align, char *str,
+							      unsigned int foreground,
+							      unsigned int background);
+xitk_image_t *xitk_image_create_image_from_string(ImlibData *im, 
+						  char *fontname, 
+						  int width, int align, char *str);
 xitk_image_t *xitk_image_create_image(ImlibData *im, int width, int height);
 
 /**
@@ -1600,11 +1707,13 @@ void draw_bevel_two_state(ImlibData *im, xitk_image_t *p);
  *
  */
 void draw_inner(ImlibData *im, Pixmap p, int w, int h);
+void draw_inner_light(ImlibData *im, Pixmap p, int w, int h);
 
 /**
  *
  */
 void draw_outter(ImlibData *im, Pixmap p, int w, int h);
+void draw_outter_light(ImlibData *im, Pixmap p, int w, int h);
 
 /**
  *
@@ -1654,6 +1763,9 @@ void draw_tab(ImlibData *im, xitk_image_t *p);
 void draw_paddle_rotate(ImlibData *im, xitk_image_t *p);
 void draw_rotate_button(ImlibData *im, xitk_image_t *p);
 
+void draw_button_plus(ImlibData *im, xitk_image_t *p);
+void draw_button_minus(ImlibData *im, xitk_image_t *p);
+
 #define XITK_WINDOW_ANSWER_UNKNOWN 0
 #define XITK_WINDOW_ANSWER_OK      1
 #define XITK_WINDOW_ANSWER_YES     2
@@ -1742,6 +1854,11 @@ void xitk_set_window_title(Display *display, Window window, char *title);
  */
 void xitk_window_set_window_title(ImlibData *im, xitk_window_t *w, char *title);
 
+/*
+ *
+ */
+int xitk_is_window_visible(Display *display, Window window);
+
 /*
  *
  */
@@ -1781,12 +1898,12 @@ void xitk_window_dialog_ok(ImlibData *im, char *title,
 #define xitk_window_dialog_error(im, message, args...) {                                         \
   if(((im) == NULL) || ((message) == NULL))                                                      \
     return;                                                                                      \
-  xitk_window_dialog_ok_with_width(im, "Error", NULL, NULL, 400, ALIGN_CENTER, message, ##args); \
+  xitk_window_dialog_ok_with_width(im, _("Error"), NULL, NULL, 400, ALIGN_CENTER, message, ##args); \
 }
 #define xitk_window_dialog_info(im, message, args...) {                                                \
   if(((im) == NULL) || ((message) == NULL))                                                            \
     return;                                                                                            \
-  xitk_window_dialog_ok_with_width(im, "Information", NULL, NULL, 400, ALIGN_CENTER, message, ##args); \
+  xitk_window_dialog_ok_with_width(im, _("Information"), NULL, NULL, 400, ALIGN_CENTER, message, ##args); \
 }
 #else
 #define xitk_window_dialog_ok(im, title, cb, userdata, align, ...) {                               \
@@ -1797,12 +1914,12 @@ void xitk_window_dialog_ok(ImlibData *im, char *title,
 #define xitk_window_dialog_error(im, ...) {                                                        \
   if(((im) == NULL))                                                                               \
     return;                                                                                        \
-  xitk_window_dialog_ok_with_width(im, "Error", NULL, NULL, 400, ALIGN_CENTER, __VA_ARGS__);       \
+  xitk_window_dialog_ok_with_width(im, _("Error"), NULL, NULL, 400, ALIGN_CENTER, __VA_ARGS__);       \
 }
 #define xitk_window_dialog_info(im, ...) {                                                         \
   if(((im) == NULL))                                                                               \
     return;                                                                                        \
-  xitk_window_dialog_ok_with_width(im, "Information", NULL, NULL, 400, ALIGN_CENTER, __VA_ARGS__); \
+  xitk_window_dialog_ok_with_width(im, _("Information"), NULL, NULL, 400, ALIGN_CENTER, __VA_ARGS__); \
 }
 #endif
 
@@ -1904,7 +2021,9 @@ typedef struct {
 
 } xitk_tabs_widget_t;
 
-xitk_widget_t *xitk_noskin_tabs_create(xitk_tabs_widget_t *t, int x, int y, int width);
+xitk_widget_t *xitk_noskin_tabs_create(xitk_widget_list_t *wl,
+				       xitk_tabs_widget_t *t, 
+				       int x, int y, int width, char *fontname);
 int xitk_tabs_get_current_selected(xitk_widget_t *w);
 char *xitk_tabs_get_current_tab_selected(xitk_widget_t *w);
 void xitk_tabs_set_current_selected(xitk_widget_t *w, int select);
@@ -1924,7 +2043,8 @@ typedef struct {
   void                   *userdata;
 } xitk_intbox_widget_t;
 
-xitk_widget_t *xitk_noskin_intbox_create(xitk_intbox_widget_t *ib,
+xitk_widget_t *xitk_noskin_intbox_create(xitk_widget_list_t *wl,
+					 xitk_intbox_widget_t *ib,
 					 int x, int y, int width, int height, 
 					 xitk_widget_t **iw, xitk_widget_t **mw, xitk_widget_t **lw);
 void xitk_intbox_set_value(xitk_widget_t *, int);
